package cn.iocoder.yudao.module.shipment.dal.dataobject;

import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.shipment.enums.*;
import lombok.*;

import java.util.*;

import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

/**
 * 出货 DO
 *
 * @author lalala
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("ecw_box")
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class BoxDO extends BaseDO {

    /**
     * 主键ID
     */
    @TableId
    private Long id;
    /**
     * 自编号
     */
    private String selfNo;
    /**
     * 柜号
     */
    private String cubNo;
    /**
     * 柜型ID
     */
    private Long cabinetId;
    /**
     * 始发地id
     */
    private Long startWarehouseId;

    /**
     * 目的地id
     */
    private Long destWarehouseId;
    /**
     * 运输方式
     */
    private String transportType;
    /**
     * 出货渠道ID
     */
    private Long shippingChannelId;

    /**
     * 预装时间
     */
    private Date yzDate;
    /**
     * 装柜时间
     */
    private Date zgDate;
    /**
     * 起运时间
     */
    private Date qyDate;
    /**
     * 到港时间
     */
    private Date dgDate;
    /**
     * 清关时间
     */
    private Date qgDate;
    /**
     * 提货时间
     */
    private Date thDate;

    /**
     * 出仓时间
     */
    private Date ckDate;

    /**
     * 订舱状态：11、未订舱；12、已订舱
     */
    private Integer bkStatus;

    /**
     * 预装状态：21、未预装；22、预装中；23、预装审核中；23、预装审核失败；24、预装审核成功
     */
    private Integer prStatus;

    /**
     * 理货状态：2111、未理货；2112、已理货
     */
    private Integer tyStatus;

    /**
     * 提交理货时间
     */
    private Date tyTime;

    /**
     * 理货操作人
     */
    private String tyOperator;

    /**
     * 拖车状态：31、未派车；32、已派车
     */
    private Integer tlStatus;

    /**
     * 拖车提柜状态：1、已派车，待提柜；2、已提柜，待还柜；3、已还柜
     */
    private Integer tlContainerStatus;

    /**
     * 装柜状态：41、未装柜；42、装柜中；43、已装柜、待封柜；44、封柜审核中；45、封柜审核失败；46、封柜审核成功
     */
    private Integer ldStatus;

    /**
     * 报关状态：51、未报关；52、报关中；53、已报关
     */
    private Integer dcStatus;
    /**
     * 报关放行状态，1：报关中，2：放行，3：查验
     */
    private Integer dcCustomsStatus;
    /**
     * 查验状态，1：退场，2：部分退场，3：查验后放行
     */
    private Integer dcCheckStatus;

    /**
     * 退场处理状态,1-已处理,0-未处理
     */
    private Integer checkDealStatus;

    /**
     * 退场审核状态,
     */
    private Integer checkExamineStatus;

    /**
     * 过机状态,1：顺利，2：异常
     */
    private Integer overMachineStatus;

    /**
     * 过机异常状态，1：敏感货(删单退场)，2：重量误报
     */
    private Integer overMachineAbnormalStatus;

    /**
     * 1-退场时间，2-返回仓库,  退场时间的话不需要进入退场列表
     */
    private Integer deleteType;

    /**
     * 删单退场审核状态，1：退场审核中，2：退场审核成功，3：退场审核失败，4：退场审核取消
     */
    private Integer overMachineExamineStatus;


    /**
     * 配船状态：61、未配船；62、已配船
     */
    private Integer saStatus;

    /**
     * 提单补料状态：71、未提单补料；72、已提单补料
     */
    private Integer blStatus;

    /**
     * 驳船状态：81、未驳船；82、已驳船
     */
    private Integer bgStatus;

    /**
     * 起运状态：91、未起运；92、已起运
     */
    private Integer dtStatus;

    /**
     * 提单COPY状态：101、未上传；102、已上传
     */
    private Integer cpStatus;

    /**
     * 清关文件状态：111、未清关文件；112、已清关文件
     */
    private Integer cdStatus;

    /**
     * 到港状态：121、未到港；112、已到港
     */
    private Integer apStatus;

    /**
     * 清关状态：131、未清关；132、已清关
     */
    private Integer clStatus;

    /**
     * 二程起飞状态：141、未起飞；142已起飞
     */
    private Integer stkStatus;

    /**
     * 二程到港状态：151、未到港；152、已到港
     */
    private Integer sapStatus;

    /**
     * 卸柜状态：181、未卸柜；182、卸柜中；183、卸柜审核中；184、卸柜审核失败；185、卸柜审核成功；186、已卸柜
     */
    private Integer ulStatus;

    /**
     * 结算状态：191、未结算；192、结算中；193、已结算
     */
    private Integer slStatus;

    /**
     * 出货状态
     */
    private Integer shipmentStatus;

    private Integer status;

    private Integer sortingStatus;

    /**
     * 出仓状态
     */
    private Integer checkoutStatus;

    /**
     * 始发国家
     */
    private Long startCountryId;

    /**
     * 目的国家
     */
    private Long destCountryId;

    /**
     * 序号
     */
    private Integer num;

    /**
     * 目的港清关 ：1、我司清关 2、合作方清关 3、客户
     */
    private Integer destinationClearance;

    /**
     * 发货方式 ：1、多票，2、单票
     */
    private Integer deliveryType;

    /**
     * 合包状态
     */
    private Integer mergePkgStatus;

    /**
     * 空运出货状态
     */
    private Integer airShipmentStatus;

    /**
     * 到仓状态
     */
    private Integer toWarehouseStatus;

    /**
     * 起飞状态
     */
    private Integer flyStatus;

    /**
     * 空运主状态
     */
    private Integer shipmentStatusAir;

    //订舱
    public void setBkStatus(Integer bkStatus) {
        this.bkStatus = bkStatus;

        if(bkStatus == null) return;
        //已订舱，修改出货主状态
        if (bkStatus == BookStatusEnum.BOOKED.getBkStatus()) {
            if(transportType.equals(TransportTypeEnum.AIR.getType())) {
                this.shipmentStatusAir = BoxAirStatusEnum.BOOKING_FOR_SORTING.getStatus();
            } else {
                this.shipmentStatus = BoxStatusEnum.BOOKING_FOR_PREINSTALL.getStatus();
            }
        }
    }

    //排单
    public void setPrStatus(Integer prStatus) {
        this.prStatus = prStatus;

        if(prStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if(prStatus == StStatusEnum.SORTING.getStStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SORTING.getStatus();
            } else if (prStatus == StStatusEnum.SORTING_APPROVAL_ING.getStStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SORTING_EXAMINE.getStatus();
            } else if (prStatus == StStatusEnum.SORTING_APPROVAL_FAIL.getStStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SORTING_EXAMINE_FAIL.getStatus();
            } else if (prStatus == StStatusEnum.SORTING_APPROVAL_SUCCESS.getStStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SORTING_EXAMINE_SUCCESS.getStatus();
            }
        } else {
            if(prStatus == PrStatusEnum.PRELOAD_ING.getPrStatus()) {
                this.shipmentStatus = BoxStatusEnum.PREINSTALLING.getStatus();
            } else if (prStatus == PrStatusEnum.PRELOAD_APPROVAL_ING.getPrStatus()) {
                this.shipmentStatus = BoxStatusEnum.PREINSTALL_EXAMINE.getStatus();
            } else if (prStatus == PrStatusEnum.PRELOAD_APPROVAL_FAIL.getPrStatus()) {
                this.shipmentStatus = BoxStatusEnum.PREINSTALL_EXAMINE_FAIL.getStatus();
            } else if (prStatus == PrStatusEnum.PRELOAD_APPROVAL_SUCCESS.getPrStatus()) {
                this.shipmentStatus = BoxStatusEnum.PREINSTALL_EXAMINE_SUCCESS.getStatus();
            }
        }
    }

    //理货
    public void setTyStatus(Integer tyStatus) {
        this.tyStatus = tyStatus;
        if(tyStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if(tyStatus == TyStatueEnum.HAS_TALLY.getTyStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.TALLY_COMPLETE.getStatus();
            }
        }
    }

    //合包
    public void setMergePkgStatus(Integer mergePkgStatus) {
        this.mergePkgStatus = mergePkgStatus;
        if(mergePkgStatus == null) return;
        if(mergePkgStatus == MergePkgStatusEnum.MERGED.getStatus()) {
//            this.shipmentStatusAir = BoxAirStatusEnum.WAIT_SET_SHIPMENT.getStatus();
        }
    }

    //出货
    public void setAirShipmentStatus(Integer airShipmentStatus) {
        this.airShipmentStatus = airShipmentStatus;
        if(airShipmentStatus == null) return;
        if(airShipmentStatus == AirShipmentStatusEnum.SHIPMENT.getStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.AIR_SHIPPED.getStatus();
        }
    }

    //出仓
    public void setCheckoutStatus(Integer checkoutStatus) {
        this.checkoutStatus = checkoutStatus;
        if(checkoutStatus == null) return;
        if(checkoutStatus == AirCheckoutStatusEnum.CHECKOUT.getStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.WAREHOUSED.getStatus();
        }
    }

    public void setTlContainerStatus(Integer tlContainerStatus) {
        this.tlContainerStatus = tlContainerStatus;
        if(tlContainerStatus == null) return;
        if (tlContainerStatus == 1) {
            this.shipmentStatus = BoxStatusEnum.DISPATCHED_WAIT_CABINETS.getStatus();
        } else if (tlContainerStatus == 2) {
            this.shipmentStatus = BoxStatusEnum.LIFTED_WAIT_RETURN.getStatus();
        } else if (tlContainerStatus == 3) {
            this.shipmentStatus = BoxStatusEnum.CABINETS_RETURNED.getStatus();
        }
    }

    //装柜
    public void setLdStatus(Integer ldStatus) {
        this.ldStatus = ldStatus;

        if(ldStatus == null) return;
        if (ldStatus == LdStatusEnum.IN_CABINET.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.IN_THE_CABINETS.getStatus();
        } else if (ldStatus == LdStatusEnum.INSTALLED_TO_BE_SEALED.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.INSTALLED_WAIT_SEALED.getStatus();
        } else if (ldStatus == LdStatusEnum.CABINET_EXAMINE_ING.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.SEAL_EXAMINE.getStatus();
        } else if (ldStatus == LdStatusEnum.CABINET_EXAMINE_FAIL.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.SEAL_EXAMINE_FAIL.getStatus();
        } else if (ldStatus == LdStatusEnum.CABINET_EXAMINE_SUCCESS.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.SEAL_EXAMINE_SUCCESS.getStatus();
        } else if (ldStatus == LdStatusEnum.CABINET_SEALED.getLdStatus()) {
            this.shipmentStatus = BoxStatusEnum.SEALED.getStatus();
        }
    }


    //报关
    public void setDcStatus(Integer dcStatus) {
        this.dcStatus = dcStatus;
        if(dcStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if(dcStatus == DcStatueEnum.DECLARED.getDcStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_COMPLETE.getStatus();
            }
        }
    }

    public void setDcCustomsStatus(Integer dcCustomsStatus) {
        this.dcCustomsStatus = dcCustomsStatus;
        //1：报关中，2：放行，3：查验
        if(dcCustomsStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if (dcCustomsStatus == 1) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMSING.getStatus();
            } else if (dcCustomsStatus == 2) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_PASS.getStatus();
            }
        } else {
            if (dcCustomsStatus == 1) {
                this.shipmentStatus = BoxStatusEnum.CUSTOMSING.getStatus();
            } else if (dcCustomsStatus == 2) {
                this.shipmentStatus = BoxStatusEnum.CUSTOMS_PASS.getStatus();
            }
        }
    }

    //查验
    public void setDcCheckStatus(Integer dcCheckStatus) {
        this.dcCheckStatus = dcCheckStatus;
        //1：退场，2：部分退场，3：查验后放行
        if(dcCheckStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if (dcCheckStatus == 2) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_PART_EXIT.getStatus();
            } else if (dcCheckStatus == 1) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_FULL_EXIT.getStatus();
            } else if (dcCheckStatus == 3) {
                this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_CHECK_PASS.getStatus();
            }
        } else {
            if (dcCheckStatus == 2) {
                this.shipmentStatus = BoxStatusEnum.CUSTOMS_PART_EXIT.getStatus();
            } else if (dcCheckStatus == 1) {
                this.shipmentStatus = BoxStatusEnum.CUSTOMS_FULL_EXIT.getStatus();
            } else if (dcCheckStatus == 3) {
                this.shipmentStatus = BoxStatusEnum.CUSTOMS_CHECK_PASS.getStatus();
            }
        }
    }

    //过机异常审核状态
    public void setOverMachineExamineStatus(Integer overMachineExamineStatus) {
        this.overMachineExamineStatus = overMachineExamineStatus;
        if(overMachineExamineStatus == null) return;
        if(overMachineExamineStatus.equals(BpmProcessInstanceResultEnum.PROCESS.getResult())) {
            this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_ABNORMAL_EXAMINE.getStatus();
        } else if(overMachineExamineStatus.equals(BpmProcessInstanceResultEnum.APPROVE.getResult())) {
            this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_ABNORMAL_EXAMINE_SUCCESS.getStatus();
        }
    }

    //删单退场、敏感货
    public void setOverMachineAbnormalStatus(Integer overMachineAbnormalStatus) {
        this.overMachineAbnormalStatus = overMachineAbnormalStatus;
        if(overMachineAbnormalStatus == null) return;
        if(overMachineAbnormalStatus == 2) {
            this.shipmentStatusAir = BoxAirStatusEnum.CUSTOMS_ABNORMAL_WEIGHT.getStatus();
        }
    }

    //起运
    public void setDtStatus(Integer dtStatus) {
        this.dtStatus = dtStatus;

        if(dtStatus == null) return;
        if (dtStatus == DtStatueEnum.HAS_SHIPPING.getDtStatus()) {
            this.shipmentStatus = BoxStatusEnum.SHIPPING.getStatus();
        }
    }

    //起飞
    public void setFlyStatus(Integer flyStatus) {
        this.flyStatus = flyStatus;
        if(flyStatus == null) return;
        if(flyStatus == FlyStatueEnum.FLYING.getDtStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.FLYING.getStatus();
        }
    }

    public void setClStatus(Integer clStatus) {
        this.clStatus = clStatus;
        if(clStatus == null) return;

        if (clStatus == ClStatueEnum.HAS_CLEARANCE.getClStatus()) {
            if(transportType.equals(TransportTypeEnum.AIR.getType())) {
                this.shipmentStatusAir = BoxAirStatusEnum.CLEARANCE_WAIT_WAREHOUSE.getStatus();
            } else {
                this.shipmentStatus = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
            }
        }
    }

    //到仓
    public void setToWarehouseStatus(Integer toWarehouseStatus) {
        this.toWarehouseStatus = toWarehouseStatus;
        if(toWarehouseStatus == null) return;

        if(toWarehouseStatus == TWStatusEnum.TO_WAREHOUSING.getTwStatus()) {
            //到仓审核取消后
            this.shipmentStatusAir = BoxAirStatusEnum.CLEARANCE_WAIT_WAREHOUSE.getStatus();
        } else if (toWarehouseStatus == TWStatusEnum.TO_WAREHOUSE_EXAMINE_ING.getTwStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.TO_WAREHOUSE_EXAMINE.getStatus();
        } else if (toWarehouseStatus == TWStatusEnum.TO_WAREHOUSE_EXAMINE_FAIL.getTwStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.TO_WAREHOUSE_EXAMINE_FAIL.getStatus();
        } else if (toWarehouseStatus == TWStatusEnum.TO_WAREHOUSE_EXAMINE_SUCCESS.getTwStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.TO_WAREHOUSE_EXAMINE_SUCCESS.getStatus();
        } else if (toWarehouseStatus == TWStatusEnum.TO_WAREHOUSED_SEALED.getTwStatus()) {
            this.shipmentStatusAir = BoxAirStatusEnum.TO_WAREHOUSED.getStatus();
        }
    }

    //卸柜
    public void setUlStatus(Integer ulStatus) {
        this.ulStatus = ulStatus;
        if(ulStatus == null) return;

        if(ulStatus == ULStatusEnum.UNLOADING_CABINET.getUlStatus()) {
            //卸柜审核取消后
            this.shipmentStatus = BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus();
        } else if (ulStatus == ULStatusEnum.UNLOADING_EXAMINE_ING.getUlStatus()) {
            this.shipmentStatus = BoxStatusEnum.UNLOAD_EXAMINE.getStatus();
        } else if (ulStatus == ULStatusEnum.UNLOADING_EXAMINE_FAIL.getUlStatus()) {
            this.shipmentStatus = BoxStatusEnum.UNLOAD_EXAMINE_FAIL.getStatus();
        } else if (ulStatus == ULStatusEnum.UNLOADING_EXAMINE_SUCCESS.getUlStatus()) {
            this.shipmentStatus = BoxStatusEnum.UNLOAD_EXAMINE_SUCCESS.getStatus();
        } else if (ulStatus == ULStatusEnum.UNLOADING_SEALED.getUlStatus()) {
            this.shipmentStatus = BoxStatusEnum.UNLOADED.getStatus();
        }
    }

    //结算
    public void setSlStatus(Integer slStatus) {
        this.slStatus = slStatus;
        if(slStatus == null) return;
        if(transportType.equals(TransportTypeEnum.AIR.getType())) {
            if (slStatus == SlStatueEnum.NO_SETTLEMENT.getSlStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.WAIT_SETTLEMENT.getStatus();
            } else if (slStatus == SlStatueEnum.SETTLEMENT_ING.getSlStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SETTLEMENTING.getStatus();
            } else if (slStatus == SlStatueEnum.HAS_SETTLEMENT.getSlStatus()) {
                this.shipmentStatusAir = BoxAirStatusEnum.SETTLEMENTED.getStatus();
            }
        } else {
            if (slStatus == SlStatueEnum.NO_SETTLEMENT.getSlStatus()) {
                this.shipmentStatus = BoxStatusEnum.WAIT_SETTLEMENT.getStatus();
            } else if (slStatus == SlStatueEnum.SETTLEMENT_ING.getSlStatus()) {
                this.shipmentStatus = BoxStatusEnum.SETTLEMENTING.getStatus();
            } else if (slStatus == SlStatueEnum.HAS_SETTLEMENT.getSlStatus()) {
                this.shipmentStatus = BoxStatusEnum.SETTLEMENTED.getStatus();
            }
        }

    }

    public void setStkStatus(Integer stkStatus) {
        this.stkStatus = stkStatus;

        if(stkStatus == null) return;

        if(stkStatus == StkStatueEnum.HAS_TAKE_OFF.getStkStatus()) {
            this.shipmentStatus = BoxStatusEnum.TWO_WAY_TAKEOFF.getStatus();
        }
    }

    //空运到港或者海空二程到港
    public void setSapStatus(Integer sapStatus) {
        this.sapStatus = sapStatus;

        if(sapStatus == null) return;
        if(sapStatus == SapStatueEnum.HAS_ARRIVE.getSapStatus()) {
            if(transportType.equals(TransportTypeEnum.AIR.getType())) {
                this.shipmentStatusAir = BoxAirStatusEnum.ARRIVAL_WAIT_DISCHARGE.getStatus();
            } else {
                this.shipmentStatus = BoxStatusEnum.TWO_WAY_ARRIVAL.getStatus();
            }

        }
    }
}
