package cn.iocoder.yudao.module.shipment.dal.mysql;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirCheckoutDO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 空运出仓 Mapper
 * @author 江楠
 */
@Mapper
public interface BoxAirCheckoutMapper extends AbstractMapper<BoxAirCheckoutDO> {
    @Override
    default PageResult<BoxAirCheckoutDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxAirCheckoutQueryVO) {
            BoxAirCheckoutQueryVO vo = (BoxAirCheckoutQueryVO)object;
            return selectPage(page, new LambdaQuery<BoxAirCheckoutDO>()
                .eqIfPresent(BoxAirCheckoutDO::getShipmentId, vo.getShipmentId())
                .betweenIfPresent(BoxAirCheckoutDO::getCheckoutTime, vo.getBeginCheckoutTime(), vo.getEndCheckoutTime())
                .eqIfPresent(BoxAirCheckoutDO::getCheckoutFiles, vo.getCheckoutFiles())
                .eqIfPresent(BoxAirCheckoutDO::getRemark, vo.getRemark())
                .betweenIfPresent(BoxAirCheckoutDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxAirCheckoutDO::getId));
        }

        return null;
    }

    @Override
    default List<BoxAirCheckoutDO> selectList(Object object) {
        if (object instanceof BoxAirCheckoutQueryVO) {
            BoxAirCheckoutQueryVO vo = (BoxAirCheckoutQueryVO)object;
            return selectList(new LambdaQuery<BoxAirCheckoutDO>()
                .eqIfPresent(BoxAirCheckoutDO::getShipmentId, vo.getShipmentId())
                .betweenIfPresent(BoxAirCheckoutDO::getCheckoutTime, vo.getBeginCheckoutTime(), vo.getEndCheckoutTime())
                .eqIfPresent(BoxAirCheckoutDO::getCheckoutFiles, vo.getCheckoutFiles())
                .eqIfPresent(BoxAirCheckoutDO::getRemark, vo.getRemark())
                .betweenIfPresent(BoxAirCheckoutDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxAirCheckoutDO::getId));
        }

        return null;
    }

    @Select("<script>" +
            "select " +
            "t.id," +
            "t.self_no," +
            "t.transport_type," +
            "t.shipment_status_air," +
            "t1.title_en," +
            "t.checkout_status " +
            "from ecw_box t " +
            "left JOIN ecw_warehouse t1 on t.dest_warehouse_id=t1.id " +
            "where t.transport_type in (3,4) " +
            "and t.air_shipment_status=175 " +
            "AND (t.start_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach> or t.dest_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach>)"+
            "<when test = 'selfNo != null and selfNo != \"\" '>"+
            "and t.self_no LIKE concat('%',#{selfNo},'%')"+
            "and t.checkout_status in (221,222)" +
            "</when>"+
            "<when test = 'selfNo == null or selfNo == \"\" '>"+
            "and t.checkout_status =221" +
            "</when>"+
            "</script>")
    IPage<BoxAirCheckoutPDAVO> getBoxAirCheckout4PDAPage(@Param("mpPage") IPage<BoxAirCheckoutPDAVO> page,@Param("warehouseIdList") Set<Long> warehouseIds,@Param("selfNo") String selfNo);
}
