package cn.iocoder.yudao.module.shipment.dal.mysql;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetUnloadDO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadQueryVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 卸柜 Mapper
 * @author wlh
 */
@Mapper
public interface BoxCabinetUnloadMapper extends AbstractMapper<BoxCabinetUnloadDO> {
    @Override
    default PageResult<BoxCabinetUnloadDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxCabinetUnloadQueryVO) {
            BoxCabinetUnloadQueryVO vo = (BoxCabinetUnloadQueryVO)object;
            return selectPage(page, new LambdaQuery<BoxCabinetUnloadDO>()
                .eqIfPresent(BoxCabinetUnloadDO::getShipmentId, vo.getShipmentId())
                .eqIfPresent(BoxCabinetUnloadDO::getUlOutletsId, vo.getUlOutletsId())
                .betweenIfPresent(BoxCabinetUnloadDO::getUlWarehouseTime, vo.getBeginUlWarehouseTime(), vo.getEndUlWarehouseTime())
                .betweenIfPresent(BoxCabinetUnloadDO::getUlBoxTime, vo.getBeginUlBoxTime(), vo.getEndUlBoxTime())
                .betweenIfPresent(BoxCabinetUnloadDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxCabinetUnloadDO::getId));
        }

        return null;
    }

    @Override
    default List<BoxCabinetUnloadDO> selectList(Object object) {
        if (object instanceof BoxCabinetUnloadQueryVO) {
            BoxCabinetUnloadQueryVO vo = (BoxCabinetUnloadQueryVO)object;
            return selectList(new LambdaQuery<BoxCabinetUnloadDO>()
                .eqIfPresent(BoxCabinetUnloadDO::getShipmentId, vo.getShipmentId())
                .eqIfPresent(BoxCabinetUnloadDO::getUlOutletsId, vo.getUlOutletsId())
                .betweenIfPresent(BoxCabinetUnloadDO::getUlWarehouseTime, vo.getBeginUlWarehouseTime(), vo.getEndUlWarehouseTime())
                .betweenIfPresent(BoxCabinetUnloadDO::getUlBoxTime, vo.getBeginUlBoxTime(), vo.getEndUlBoxTime())
                .betweenIfPresent(BoxCabinetUnloadDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxCabinetUnloadDO::getId));
        }

        return null;
    }

    @Select("<script>" +
            "select " +
            "t.id," +
            "t.self_no," +
            "t.transport_type," +
            "t1.title_en," +
            "t.shipping_channel_id, " +
            "t.to_warehouse_status as checkoutStatus, " +
            "t3.name_zh as shippingChannelNameZh, " +
            "t3.name_en as shippingChannelNameEn, " +
            "t2.ul_warehouse_time as arrivalWarehouseTime "+
            "from ecw_box t " +
            "left join ecw_box_cabinet_unload t2 on t.id=t2.shipment_id "+
            "left JOIN ecw_warehouse t1 on t.dest_warehouse_id=t1.id " +
            "left join ecw_channel t3 on t.shipping_channel_id=t3.channel_id " +
            "where t.transport_type=3 " +
            "and t.cl_status=132 " +
            "and t.to_warehouse_status in (181,182,183,184,185,186) " +
            "AND (t.start_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach> or t.dest_warehouse_id in "+
            "<foreach item='id' index='index' collection='warehouseIdList' open='(' separator=',' close=')'>"+
            "#{id}"+
            "</foreach>)"+
            "<when test = 'selfNo != null and selfNo != \"\"'>"+
            "and t.self_no LIKE concat('%',#{selfNo},'%')"+
            "</when>"+
            "<when test = 'ulStatus != null'>"+
            "and t.to_warehouse_status = #{ulStatus}"+
            "</when>"+
            "<when test = 'selfNo == \"\"'>"+
            "and t.to_warehouse_status = 181" +
            "</when>"+
            "</script>")
    IPage<BoxCabinetUnloadPDAVO> getBoxCabinetUnload4PDAPage(@Param("mpPage") IPage<BoxCabinetUnloadPDAVO> page, @Param("warehouseIdList") Set<Long> warehouseIds, @Param("selfNo") String selfNo,@Param("ulStatus") Integer ulStatus);
}
