package cn.iocoder.yudao.module.shipment.dal.mysql;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.BoxPkgOrderQueryVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 合包订单 Mapper
 * @author 江楠
 */
@Mapper
public interface BoxPkgOrderMapper extends AbstractMapper<BoxPkgOrderDO> {
    @Override
    default PageResult<BoxPkgOrderDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxPkgOrderQueryVO) {
            BoxPkgOrderQueryVO vo = (BoxPkgOrderQueryVO)object;
            return selectPage(page, new LambdaQuery<BoxPkgOrderDO>()
                .betweenIfPresent(BoxPkgOrderDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(BoxPkgOrderDO::getPkgId, vo.getPkgId())
                .eqIfPresent(BoxPkgOrderDO::getOrderId, vo.getOrderId())
                .eqIfPresent(BoxPkgOrderDO::getOrderItemId, vo.getOrderItemId())
                    .orderByDesc(BoxPkgOrderDO::getId));
        }

        return null;
    }

    @Override
    default List<BoxPkgOrderDO> selectList(Object object) {
        if (object instanceof BoxPkgOrderQueryVO) {
            BoxPkgOrderQueryVO vo = (BoxPkgOrderQueryVO)object;
            return selectList(new LambdaQuery<BoxPkgOrderDO>()
                .betweenIfPresent(BoxPkgOrderDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(BoxPkgOrderDO::getPkgId, vo.getPkgId())
                .eqIfPresent(BoxPkgOrderDO::getOrderId, vo.getOrderId())
                .eqIfPresent(BoxPkgOrderDO::getOrderItemId, vo.getOrderItemId())
                    .orderByDesc(BoxPkgOrderDO::getId));
        }

        return null;
    }

    @Select({"<script>" +
            "select t.pkg_id as id,ifnull(sum(t1.warehouse_in_info ->> '$.quantityAll'),0) as sumNum from ecw_box_pkg_order t LEFT JOIN ecw_order_item t1 on t.order_item_id=t1.order_item_id " +
            " where t.deleted=0 and t1.deleted=0 and t.pkg_id in " +
            "<foreach item='id' index='index' collection='pkgIds' open='(' separator=',' close=')'>#{id}</foreach>" +
            " GROUP BY t.pkg_id" +
            "</script>"})
    @ResultType(BoxMergePkgDO.class)
    List<BoxMergePkgDO> getMergeOrderQualityByPkgIds(@Param("pkgIds") List<Long> pkgIds);
}
