package cn.iocoder.yudao.module.shipment.dal.mysql;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadGoodsDO;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.PreloadedReq;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Map;

/**
 * 预装货物 Mapper
 * @author wlh
 */
@Mapper
public interface BoxPreloadGoodsMapper extends AbstractMapper<BoxPreloadGoodsDO> {
    @Override
    default PageResult<BoxPreloadGoodsDO> selectPage(PageVO page, Object object) {
        if (object instanceof BoxPreloadGoodsQueryVO) {
            BoxPreloadGoodsQueryVO vo = (BoxPreloadGoodsQueryVO)object;
            return selectPage(page, new LambdaQuery<BoxPreloadGoodsDO>()
                .eqIfPresent(BoxPreloadGoodsDO::getShipmentId, vo.getShipmentId())
                .eqIfPresent(BoxPreloadGoodsDO::getSecId, vo.getSecId())
                .eqIfPresent(BoxPreloadGoodsDO::getOrderId, vo.getOrderId())
                .eqIfPresent(BoxPreloadGoodsDO::getOrderItemId, vo.getOrderItemId())
                .betweenIfPresent(BoxPreloadGoodsDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxPreloadGoodsDO::getId));
        }

        return null;
    }

    @Override
    default List<BoxPreloadGoodsDO> selectList(Object object) {
        if (object instanceof BoxPreloadGoodsQueryVO) {
            BoxPreloadGoodsQueryVO vo = (BoxPreloadGoodsQueryVO)object;
            return selectList(new LambdaQuery<BoxPreloadGoodsDO>()
                .eqIfPresent(BoxPreloadGoodsDO::getShipmentId, vo.getShipmentId())
                .eqIfPresent(BoxPreloadGoodsDO::getSecId, vo.getSecId())
                .eqIfPresent(BoxPreloadGoodsDO::getOrderId, vo.getOrderId())
                .eqIfPresent(BoxPreloadGoodsDO::getIsRemove, vo.getIsRemove())
                .eqIfPresent(BoxPreloadGoodsDO::getOrderItemId, vo.getOrderItemId())
                .betweenIfPresent(BoxPreloadGoodsDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(BoxPreloadGoodsDO::getId));
        }

        return null;
    }

    @ResultType(BoxPreloadGoodsBackVO.class)
    @Select({
            "<script>",
            "select ebpg.shipment_id, ",
            "ebpg.id, ",
            "ebpg.sec_id, ",
            "ebpg.order_id, ",
            "ebpg.order_item_id, ",
            "ebpg.tidan_num, ",
            "eo.order_no, ",
            "eo.transport_id, ",
            "(select min(owi.`in_time`) from ecw_order_warehouse_in owi where owi.deleted = 0 and owi.order_id = eo.order_id ) as rucang_time,",
            "eo.product_record, ",
            "eo.customs_type, ",
            "eo.adjust_to_dest_warehouse_id, ",
            "eo.warehouse_type, ",
            "eo.line_id, ",
            "eo.is_external_warehouse, ",
            "eo.cost ->> '$.totalWorth' as total_worth, ",
            "eo.air_shipment, ",
            "eo.status, ",
            "eo.abnormal_state, ",
            "eo.in_warehouse_state, ",
            "eo.shipment_state, ",
            "eo.audit_type, ",
            "eo.audit_result, ",
            "eo.guan_lian_order_status, ",
            "eo.exception_reason, ",
            "eo.package_type as advance_type, ",
            "eoi.prod_title_zh, ",
            "eoi.prod_title_en, ",
            "eoi.box_gauge, ",
            "eoi.num, ",
            "eoi.quantity, ",
            "eoi.worth, ",
            "eoi.brand_type, ",
            "eoi.brand, ",
            "eoi.sea_freight_currency, ",
            "eoi.clearance_freight_currency, ",
            "eoi.prod_attr_ids, ",
            "eoi.warehouse_in_prod_attr_ids, ",
            "eoi.fee_type, ",
            "if(#{query.lang} = 0, ew_start.title_zh, ew_start.title_en) as start_warehouse_name, ",
            "if(#{query.lang} = 0, ew_dest.title_zh, ew_dest.title_en) as dest_warehouse_name, ",
            "eoi.unit, ",
            "eoi.volume, ",
            "eoi.warehouse_in_info, ",
            "eoi.weight,",
            "eoi.charge_weight, ",
            "eoi.charge_volume, ",
            "eoi.charge_quantity, ",
            "eoi.material,",
            "eoi.sea_freight_volume, ",
            "eoi.clearance_freight_volume, ",
            "IF(#{query.lang} = 0, pd.title_zh, pd.title_en) as brand_name, ",
            "ewl.dest_warehouse_id, ",
            "IFNULL((select a.pkg_id from ecw_box_pkg_order a where a.deleted=0 and a.order_item_id=ebpg.order_item_id limit 1),0) as pkgId, ",
            "(select t1.pkg_num from ecw_box_pkg_order t INNER JOIN ecw_box_merge_pkg t1 on t.pkg_id=t1.id where t.deleted=0 and t1.deleted=0 and t.order_id=ebpg.order_id limit 1) as pkgNum ",
            "FROM ecw_box_preload_goods ebpg ",
            "INNER JOIN ecw_box_preload_section ebps ON ebpg.sec_id = ebps.id ",
            "INNER JOIN ecw_order eo ON eo.order_id = ebpg.order_id ",
            "INNER JOIN ecw_order_item eoi ON ebpg.order_item_id = eoi.order_item_id ",
            "INNER JOIN ecw_warehouse_line ewl ON eo.line_id = ewl.id ",
            "INNER JOIN ecw_warehouse ew_start ON ewl.start_warehouse_id = ew_start.id ",
            "INNER JOIN ecw_warehouse ew_dest ON ewl.dest_warehouse_id = ew_dest.id ",
            "LEFT JOIN ecw_product_brank pd on eoi.brand = pd.id ",
            "WHERE eo.deleted=0 ",
            "and eoi.deleted=0 ",
            "<when test = 'query.isDel == null'>",
            "and ebpg.deleted=0 ",
            "</when>",
            "<when test = 'query.shipmentId != null'>",
            "AND ebpg.shipment_id = #{query.shipmentId}",
            "</when>",
            "<when test = 'query.isCover != null'>",
            "AND ebpg.`is_cover` = #{query.isCover}",
            "</when>",
            "<when test = 'query.isRemove != null'>",
            "AND ebpg.`is_remove` = #{query.isRemove}",
            "</when>",
            "<when test = 'query.rucangTimeStart != null'>",
            "AND eo.`rucang_time` &gt;= #{query.rucangTimeStart}",
            "</when>",
            "<when test = 'query.rucangTimeEnd != null'>",
            "AND eo.`rucang_time` &lt;= #{query.rucangTimeEnd}",
            "</when>",
            "<when test = 'query.productRecord != null'>",
            "AND eo.`product_record` = #{query.productRecord}",
            "</when>",
            "<when test = 'query.orderId != null'>",
            "AND eo.`order_id` = #{query.orderId}",
            "</when>",
            "<when test = 'query.orderNo != null and query.orderNo != \"\" '>",
            "AND eo.`order_no` like concat('%',concat(#{query.orderNo},'%'))",
            "</when>",
            "<when test = 'query.customsType != null '>",
            "AND eo.`customs_type` = #{query.customsType}",
            "</when>",
            "<when test = 'query.weightRatioMin != null '>",
            "AND eo.`weight_ratio` &gt;= #{query.weightRatioMin}",
            "</when>",
            "<when test = 'query.weightRatioMax != null '>",
            "AND eo.`weight_ratio` &lt;= #{query.weightRatioMax} ",
            "</when>",
            "<when test = 'query.startWarehouseId != null'>",
            "AND ewl.`start_warehouse_id` = #{query.startWarehouseId}",
            "</when>",
            "<when test = 'query.destWarehouseIdList != null and query.destWarehouseIdList.size() > 0'>",
            "AND ewl.`dest_warehouse_id` in ",
            "<foreach item='id' index='index' collection='query.destWarehouseIdList' open='(' separator=',' close=')'>#{id}</foreach>",
            "</when>",
            "<when test = 'query.itemName != null and query.itemName != \"\" '>",
            "AND (eoi.`prod_title_zh` like concat('%',concat(#{query.itemName},'%')) or eoi.`prod_title_en` like concat('%',concat(#{query.itemName},'%')) )",
            "</when>",

            "<when test = 'query.shipmentIdList != null and query.shipmentIdList.size() > 0'>",
            "AND ebpg.`shipment_id` in ",
            "<foreach item='id' index='index' collection='query.shipmentIdList' open='(' separator=',' close=')'>#{id}</foreach>",
            "</when>",

            "order by ebpg.create_time",
            "</script>"
    })
    List<BoxPreloadGoodsBackVO> orderPreloaded(@Param("query") PreloadedReq query);


    @Select("SELECT GROUP_CONCAT(DISTINCT customs_type) customs_type from ecw_order eo where EXISTS (SELECT order_id from  ecw_box_preload_goods ebpg where shipment_id=#{shipmentId} and deleted=0 and is_remove=0 and eo.order_id=ebpg.order_id)")
    Map<String, String> getShipmentCustomTypes(@Param("shipmentId") Long shipmentId);

    @Select("SELECT " +
            "sum( t2.sum_volume ) AS mergePlanVolume, " +
            "sum( t2.sum_weight ) AS mergePlanWeight, " +
            "ROUND( " +
            "sum(( " +
            "SELECT " +
            "count(*)  " +
            "FROM " +
            "ecw_box_load_info a  " +
            "WHERE " +
            "a.order_id = t1.order_id  " +
            "AND a.deleted = 0  " +
            ")/ t2.sum_num * t2.sum_volume  " +
            "), " +
            "2  " +
            ") AS mergeRealVolume, " +
            "ROUND( " +
            "sum(( " +
            "SELECT " +
            "count(*)  " +
            "FROM " +
            "ecw_box_load_info a  " +
            "WHERE " +
            "a.order_id = t1.order_id  " +
            "AND a.deleted = 0  " +
            ")/ t2.sum_num * t2.sum_weight  " +
            "), " +
            "2  " +
            ") AS mergeRealWeight  " +
            "FROM " +
            "( " +
            "SELECT DISTINCT " +
            "t3.order_id  " +
            "FROM " +
            "ecw_box_pkg_order t3 " +
            "LEFT JOIN ecw_box_merge_pkg t ON t.id = t3.pkg_id  " +
            "WHERE " +
            "t.shipment_id = #{shipmentId}  " +
            "AND t.deleted = 0  " +
            "AND t3.deleted = 0  " +
            ") t1 " +
            "LEFT JOIN ecw_order t2 ON t1.order_id = t2.order_id  " +
            "WHERE " +
            "t2.deleted =0")
    @ResultType(BoxGoodsStatistics.class)
    BoxGoodsStatistics queryMergeCountData(@Param("shipmentId") Long shipmentId);
}
