package cn.iocoder.yudao.module.shipment.dal.mysql;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.FutureBoxDO;
import cn.iocoder.yudao.module.shipment.vo.futureBox.FutureBoxBackVO;
import cn.iocoder.yudao.module.shipment.vo.futureBox.FutureBoxQueryVO;
import cn.iocoder.yudao.module.shipment.vo.futureBox.FutureBoxTotalQueryVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;

/**
 * 预定舱计划 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface FutureBoxMapper extends AbstractMapper<FutureBoxDO> {
    @Override
    default PageResult<FutureBoxDO> selectPage(PageVO page, Object object) {
        if (object instanceof FutureBoxQueryVO) {
            FutureBoxQueryVO vo = (FutureBoxQueryVO) object;
            return selectPage(page, new LambdaQuery<FutureBoxDO>().eqIfPresent(FutureBoxDO::getStartWarehouseId, vo.getStartWarehouseId()).eqIfPresent(FutureBoxDO::getDestWarehouseId, vo.getDestWarehouseId()).eqIfPresent(FutureBoxDO::getTransportType, vo.getTransportType()).eqIfPresent(FutureBoxDO::getReadyWarehousingNumber, vo.getReadyWarehousingNumber()).eqIfPresent(FutureBoxDO::getWarehousedNumber, vo.getWarehousedNumber()).eqIfPresent(FutureBoxDO::getWarehousingNumber, vo.getWarehousingNumber()).eqIfPresent(FutureBoxDO::getLoadingNumber, vo.getLoadingNumber()).eqIfPresent(FutureBoxDO::getFutureNumber, vo.getFutureNumber()).eqIfPresent(FutureBoxDO::getGpNumber, vo.getGpNumber()).eqIfPresent(FutureBoxDO::getHqNumber, vo.getHqNumber()).eqIfPresent(FutureBoxDO::getFuturePlanNumber, vo.getFuturePlanNumber()).eqIfPresent(FutureBoxDO::getActualNumber, vo.getActualNumber()).eqIfPresent(FutureBoxDO::getShippingChannelId, vo.getShippingChannelId()).eqIfPresent(FutureBoxDO::getStatus, vo.getStatus()).betweenIfPresent(FutureBoxDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime()).orderByDesc(FutureBoxDO::getId));
        }

        return null;
    }

    @Override
    default List<FutureBoxDO> selectList(Object object) {
        if (object instanceof FutureBoxQueryVO) {
            FutureBoxQueryVO vo = (FutureBoxQueryVO) object;
            return selectList(new LambdaQuery<FutureBoxDO>().eqIfPresent(FutureBoxDO::getStartWarehouseId, vo.getStartWarehouseId()).eqIfPresent(FutureBoxDO::getDestWarehouseId, vo.getDestWarehouseId()).eqIfPresent(FutureBoxDO::getTransportType, vo.getTransportType()).eqIfPresent(FutureBoxDO::getReadyWarehousingNumber, vo.getReadyWarehousingNumber()).eqIfPresent(FutureBoxDO::getWarehousedNumber, vo.getWarehousedNumber()).eqIfPresent(FutureBoxDO::getWarehousingNumber, vo.getWarehousingNumber()).eqIfPresent(FutureBoxDO::getLoadingNumber, vo.getLoadingNumber()).eqIfPresent(FutureBoxDO::getFutureNumber, vo.getFutureNumber()).eqIfPresent(FutureBoxDO::getGpNumber, vo.getGpNumber()).eqIfPresent(FutureBoxDO::getHqNumber, vo.getHqNumber()).eqIfPresent(FutureBoxDO::getFuturePlanNumber, vo.getFuturePlanNumber()).eqIfPresent(FutureBoxDO::getActualNumber, vo.getActualNumber()).eqIfPresent(FutureBoxDO::getShippingChannelId, vo.getShippingChannelId()).eqIfPresent(FutureBoxDO::getStatus, vo.getStatus()).betweenIfPresent(FutureBoxDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime()).orderByDesc(FutureBoxDO::getId));
        }

        return null;
    }


    @ResultType(FutureBoxBackVO.class)
    @Select({
            "<script>",
            "select box.*,(select su.nickname from system_user su where su.id=box.creator) as create_name," +
                    " channel.name_zh as shipping_channel_name,channel.name_en as shipping_channel_name_en" +
                    " from ecw_future_box box left join ecw_channel channel on box.shipping_channel_id = channel.channel_id where box.deleted = 0 ",
            "<when test = 'query.transportType != null'>",
            " AND box.`transport_type` = #{query.transportType}",
            "</when>",
            "<when test = 'query.startWarehouseId != null'>",
            " AND box.`start_warehouse_id` = #{query.startWarehouseId}",
            "</when>",
            "<when test = 'query.destWarehouseId != null'>",
            " AND box.`dest_warehouse_id` = #{query.destWarehouseId}",
            "</when>",
            "</script>"
    })
    IPage<FutureBoxBackVO> selectBackPage(@Param("mPage") IPage<FutureBoxBackVO> mpPage, @Param("query") FutureBoxQueryVO query);


    @ResultType(BigDecimal.class)
    @Select({"<script>",
            "select  ROUND(sum(eo.cost->>'$.totalVolume'),2) from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 and eo.status = 2 ",
            "<when test = 'query.transportId != null'>",
            " AND eo.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",
            "</script>"})
    BigDecimal getToBeWareHousedVolume(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    @ResultType(BigDecimal.class)
    @Select({"<script>",
            "select  ROUND(sum(eo.sum_volume),2) from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 and eo.status = 5 ",
            "<when test = 'query.transportId != null'>",
            " AND eo.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",

            "</script>"})
    BigDecimal getWareHousedVolume(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    @ResultType(BigDecimal.class)
    @Select({"<script>",
//            "select  if((eo.`transport_id` = 1 or eo.`transport_id` = 2), ROUND(sum(eo.sum_volume),2), ROUND(sum(eo.sum_weight),2))",
            "select  ROUND(sum(eo.sum_volume),2)",
            "from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 ",
            " AND eo.`transport_id` = #{query.transportId}",
            "<when test = 'query.transportId == 1'>",
            "and eo.status = 12 ",
            "</when>",
            "<when test = 'query.transportId == 2'>",
            "and eo.status = 12 ",
            "</when>",
            "<when test = 'query.transportId == 3'>",
            "and eo.status = 32 ",
            "</when>",
            "<when test = 'query.transportId == 4'>",
            "and eo.status = 32 ",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",

            "</script>"})
    BigDecimal getBoxedVolume(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    @ResultType(BigDecimal.class)
    @Select({"<script>",
            "select ROUND(sum(eo.sum_weight),2)",
            "from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 ",
            " AND eo.`transport_id` = #{query.transportId}",
            "<when test = 'query.transportId == 1'>",
            "and eo.status = 12 ",
            "</when>",
            "<when test = 'query.transportId == 2'>",
            "and eo.status = 12 ",
            "</when>",
            "<when test = 'query.transportId == 3'>",
            "and eo.status = 32 ",
            "</when>",
            "<when test = 'query.transportId == 4'>",
            "and eo.status = 32 ",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",

            "</script>"})
    BigDecimal getBoxedWeight(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     *
     * @param futureBoxTotalQueryVo
     * @return
     */
    @ResultType(BigDecimal.class)
    @Select({"<script>",
            " select ROUND(SUM(eo.sum_volume),2) from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 and eo.status =5 ",
            " and eo.abnormal_state != 0 ",
            "<when test = 'query.transportId != null'>",
            " AND eo.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",
            "</script>"})
    BigDecimal getExceptionVolume(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    @ResultType(BigDecimal.class)
    @Select({"<script>",
            " select  ROUND(SUM(eo.sum_volume),2) from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 and eo.status = 5 ",
            " and eo.customs_type =2 ",
            "<when test = 'query.transportId != null'>",
            " AND eo.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",
            "</script>"})

    BigDecimal getOrderReportVolume(@Param("query") FutureBoxTotalQueryVo futureBoxTotalQueryVo);


    @ResultType(BigDecimal.class)
    @Select({"<script>",
            " select ROUND(SUM(eo.sum_volume),2) from ecw_order eo left join ecw_warehouse_line el on eo.line_id = el.id ",
            " where eo.deleted = 0 and eo.status = 5 ",
            " AND  eo.audit_type = 0 and eo.abnormal_state = 0 ",
            "<when test = 'query.transportId != null'>",
            " AND eo.`transport_id` = #{query.transportId}",
            "</when>",
            "<when test = 'query.channelId != null'>",
            " AND eo.`channel_id` = #{query.channelId}",
            "</when>",
            "<when test = 'query.lineId != null'>",
            " AND eo.`line_id` = #{query.lineId}",
            "</when>",
            "<when test = 'query.departureId != null'>",
            " AND el.`start_warehouse_id` = #{query.departureId}",
            "</when>",
            "<when test = 'query.objectiveId != null'>",
            " AND el.`dest_warehouse_id` = #{query.objectiveId}",
            "</when>",
            "</script>"})
    BigDecimal getWarehousingVolume(@Param("query")FutureBoxTotalQueryVo futureBoxTotalQueryVo);
}
