package cn.iocoder.yudao.module.shipment.listener;

import cn.iocoder.yudao.framework.apollo.core.event.BoxCheckOrderSchedulingEvent;
import cn.iocoder.yudao.framework.apollo.core.event.BoxOrderDynamicEstTimeEvent;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxShippingDO;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxBookSea.BoxBookSeaService;
import cn.iocoder.yudao.module.shipment.service.boxClearance.BoxClearanceService;
import cn.iocoder.yudao.module.shipment.service.boxShipping.BoxShippingService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.WAREHOUSE_IN_BOX_EXCEPTION;

/**
 * 查询订单动态的出货状态预计时间
 *
 * @author zhengYi
 */
@Component("BoxOrderDynamicEstTimeListener")
@AllArgsConstructor
@Slf4j
public class BoxOrderDynamicEstTimeListener {

    private final BoxService boxService;

    private final BoxBookSeaService boxBookSeaService;

    private final BoxShippingService boxShippingService;

    private final BoxClearanceService boxClearanceService;

    /**
     * 提供订单是否已经排单校验的接口，如果已经排单，则不能货物补充（会导致已装体积重量的不正确），需要先从预装单中移除才能继续补充，
     *
     * @param event 订单号
     */
    @EventListener(BoxOrderDynamicEstTimeEvent.class)
    public void checkOrderScheduling(BoxOrderDynamicEstTimeEvent event) {
        String selfNo = event.getContainerNumber();

        log.info("提供订单自编号 selfNo={}", selfNo);

        BoxDO boxDO = boxService.getOne(new LambdaQueryWrapper<BoxDO>().eq(BoxDO::getSelfNo, selfNo));
        if (Objects.nonNull(boxDO)) {
            Long shipmentId = boxDO.getId(); // 出货ID
           switch (event.getType()){
               case 10:
                   // 订舱 -> 预计开船时间
                   BoxBookSeaDO boxBookSeaDO = boxBookSeaService.getOne(new LambdaQueryWrapper<BoxBookSeaDO>().eq(BoxBookSeaDO::getShipmentId, shipmentId)
                           .orderByDesc(BoxBookSeaDO::getId).last("limit 1"));
                   if (Objects.nonNull(boxBookSeaDO) && Objects.nonNull(boxBookSeaDO.getSailTime())){
                       event.setEstTime(boxBookSeaDO.getSailTime());
                   }
                   break;
               case 11:
                   // 起运 -> 预计到港时间
                   BoxShippingDO boxShippingDO = boxShippingService.getOne(new LambdaQueryWrapper<BoxShippingDO>().eq(BoxShippingDO::getShipmentId, shipmentId)
                           .orderByDesc(BoxShippingDO::getId).last("limit 1"));
                   if (Objects.nonNull(boxShippingDO) && Objects.nonNull(boxShippingDO.getDtEstArrivalTime())){
                       event.setEstTime(boxShippingDO.getDtEstArrivalTime());
                   }
                   break;
               case 12:
                   // 清关 -> 预计清关时间
                   BoxClearanceDO boxClearanceDO = boxClearanceService.getOne(new LambdaQueryWrapper<BoxClearanceDO>().eq(BoxClearanceDO::getShipmentId, shipmentId)
                           .orderByDesc(BoxClearanceDO::getId).last("limit 1"));
                   if (Objects.nonNull(boxClearanceDO) && Objects.nonNull(boxClearanceDO.getClEstTime())){
                       event.setEstTime(boxClearanceDO.getClEstTime());
                   }
                   break;
               default:
                   log.error("未找到货柜【{}】的相关动态预计时间", selfNo);
           }
        }
    }


}
