package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentAirLoadExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.order.OrderWarehouseInDetailsVO;
import cn.iocoder.yudao.module.order.vo.orderItem.OrderItemBackVO;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemVO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductDO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.product.service.product.ProductService;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dto.AirLoadDto;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomAirLoadMergeStrategyNew;
import cn.iocoder.yudao.module.shipment.utils.CustomCellWriteHandler;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ShipmentAirLoadExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentAirLoadExcelExportListener {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private ProductBrankService productBrankService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderLocationService orderLocationService;

    @Resource
    private OrderGuanlianService orderGuanlianService;

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderService orderService;

    @Resource
    private ProductService productService;

    @Resource
    private ProductAttrService productAttrService;

    @Resource
    private BoxPkgOrderService boxPkgOrderService;

    @Resource
    private BoxMergePkgService boxMergePkgService;

    private static List<Integer> colorList = new ArrayList<>();
    static {
        colorList.add(22);
        colorList.add(41);
        colorList.add(42);
        colorList.add(43);
        colorList.add(44);
        colorList.add(45);
        colorList.add(46);
        colorList.add(47);
    }



    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentAirLoadExcelExportPushEvent.class)
    public void shipmentAirLoadExcelExportPushEvent(ShipmentAirLoadExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/load/excel/");
                File fileDir = new File(dir);
                if (!fileDir.exists()){
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_preload.xlsx");
                InputStream inputStream = null;
                inputStream = this.getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/airLoad.xlsx");
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");
                Map headMap = new HashMap();
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                }
                BoxDO box = boxService.getBox(shipmentId);
                BoxApprovalDO boxApprovalDO = boxApprovalService.getOne(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalType, 1).last("limit 1"));
                headMap.put("selfNo", box.getSelfNo());
                headMap.put("airChannel","");//TODO 航班渠道todo
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                if (boxApprovalDO != null) {
                    headMap.put("date", DateUtils.formatDate(boxApprovalDO.getUpdateTime()));
                }
                ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                excelWriter.fill(headMap, sheet);
                //以secId排序
                goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getPkgId).reversed().thenComparing(BoxPreloadGoodsBackVO::getTidanNum));
                List<Long> pkgIds = goodsItemList.stream().filter(item -> item.getPkgId() != null && item.getPkgId() > 0L).map(BoxPreloadGoodsBackVO::getPkgId).distinct().collect(Collectors.toList());
                Map<String,Integer> map = new HashMap<>();
                if (CollectionUtil.isNotEmpty(pkgIds)) {
                    map = boxPkgOrderService.getMergeOrderQualityByPkgIds(pkgIds);
                }
                Long disMergePkgCount = goodsItemList.stream().filter(item -> item.getPkgId() != null && item.getPkgId() > 0L).map(BoxPreloadGoodsBackVO::getPkgId).distinct().count();
                int sumMergeCount = disMergePkgCount.intValue();
//                goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getSecId).thenComparing(BoxPreloadGoodsBackVO::getTidanNum).thenComparing(BoxPreloadGoodsBackVO::getId));
                int allTotalNum = 0;
                BigDecimal allTotalVolume = new BigDecimal(0);
                BigDecimal allTotalWeight = new BigDecimal(0);
                Map<String,BigDecimal> mergeVolumnMap = new HashMap<>();
                Map<String,BigDecimal> mergeWeightMap = new HashMap<>();
                Map<String,Integer> mergeQualityMap = new HashMap<>();
                int totalMergeQuality = 0;
                BigDecimal totalMergeVolume = new BigDecimal("0");
                BigDecimal totalMergeWeight = new BigDecimal("0");
                List<Long> secCount = goodsItemList.stream().map(BoxPreloadGoodsBackVO::getSecId).distinct().sorted().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(secCount)) {
                    for (int i=0;i<secCount.size();i++){
                        List<String> list1 = new ArrayList();
                        List<List<String>> headList = new ArrayList<>();
                        String head = "";
                        if (i == 0) {
                            head = "第"+ NumberUtils.int2chineseNum(i+1) +"部分(Be loaded inside of the container)";
                        } else {
                            head = "第"+NumberUtils.int2chineseNum(i+1)+"部分(after "+ NumberUtils.format(i+1)+" parts)";
                        }
                        list1.add(head);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        List<PreloadDto> list = new ArrayList<>();

                        int totalNum = 0;
                        BigDecimal totalVolume = new BigDecimal(0);
                        BigDecimal totalWeight = new BigDecimal(0);

                        Map<String,Integer> numMap = new HashMap();
                        List<AirLoadDto> preloadDtoList = new ArrayList<>();

                        Long part = secCount.get(i);
                        List<Integer> mergeCountList = new ArrayList();

                        List<Integer> colorRowList = new ArrayList<>();
                        AtomicInteger colorRowNum = new AtomicInteger(5);
                        Map<String,Integer> colorMap = new HashMap<>();
                        Map<String,Integer> mergePkgCountMap = new LinkedHashMap<>();
                        int colorIndex = 0;
                        //以订单ID分组
                        Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream().filter(item -> item.getSecId().longValue()==part.longValue() && StringUtils.isNotBlank(item.getWarehouseInInfo()))
                                .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,LinkedHashMap::new,Collectors.toList()));
                        for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                            Long orderId = entry.getKey();
                            Long pkgId = entry.getValue().get(0).getPkgId();
                            OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                            OrderDO orderDO = orderService.selectOne(OrderDO::getOrderId, orderId);
                            List<OrderWarehouseInBackItemVO> orderWarehouseInBackItemVOS = orderWarehouseInService.getOrderWarehouseInListOrderByInTime(orderId);
                            for (OrderWarehouseInBackItemVO item : orderWarehouseInBackItemVOS) {
                                int andIncrement = colorRowNum.getAndIncrement();
                                if (StringUtils.isNotBlank(item.getOrderWarehouseInDetails())) {
                                    JSONArray jsonArray = JSONArray.parseArray(item.getOrderWarehouseInDetails());
                                    if (jsonArray.size() > 0) {
                                        colorMap.put(String.valueOf(andIncrement),colorList.get(colorIndex));
                                        colorRowList.add(andIncrement);//这里加1是为了设置颜色时过滤掉入仓记录那条
                                    }
                                }
                                int mergeCount = 1;
                                AirLoadDto preloadDto = new AirLoadDto();
                                if (numMap.get(orderInfo.getOrderNo()) != null) {
                                    preloadDto.setId(String.valueOf(numMap.get(orderInfo.getOrderNo())));
                                } else {
                                    preloadDto.setId(String.valueOf(numMap.size()+1));
                                    numMap.put(orderInfo.getOrderNo(),numMap.size()+1);
                                }
                                preloadDto.setOrderNo(orderInfo.getOrderNo());
                                preloadDto.setMarks(orderInfo.getMarks());
                                preloadDto.setProdTitleZh(item.getProdTitleZh()+ " " +item.getProdTitleEn());
                                preloadDto.setQuantity(item.getQuantityAll());
                                ProductBrankDO productBrank = productBrankService.getProductBrank(item.getBrand());
                                if(productBrank!=null && StringUtils.isNotBlank(productBrank.getTitleZh())){
                                    preloadDto.setBrand(productBrank.getTitleZh());
                                }
                                if(productBrank !=null && productBrank.getFiling()!=null){
                                    preloadDto.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                }
                                List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
                                Map<String, String> idNameMap = productAttrDOList.stream()
                                        .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
                                preloadDto.setProdAttr(getAttrNameByIds(item.getProdAttrIds(),idNameMap));
                                if(orderInfo.getCustomsType()!=null){
                                    preloadDto.setCustomsType(DictFrameworkUtils.getDictDataFromCache("customs_type",orderInfo.getCustomsType()+"").getLabel());
                                }
                                preloadDto.setSumNum(item.getCartonsNum());
                                preloadDto.setPackageType(DictFrameworkUtils.getDictDataFromCache("packaging_type",item.getUnit()).getLabel());
                                preloadDto.setBoxGauge(item.getBoxGauge());
                                preloadDto.setSumVolume(item.getVolume());
                                preloadDto.setSumWeight(item.getWeight());
                                List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(BoxPkgOrderDO::getOrderId, orderId);
                                if (CollectionUtil.isNotEmpty(boxPkgOrderDOS)) {
                                    preloadDto.setMergePkgNum(1);
                                    BoxMergePkgDO boxMergePkg = boxMergePkgService.getBoxMergePkg(boxPkgOrderDOS.get(0).getPkgId());
                                    preloadDto.setMergePkgBoxGauge(boxMergePkg.getPkgLength()+"*"+boxMergePkg.getPkgWidth()+"*"+boxMergePkg.getPkgHight());
                                    preloadDto.setMergePkgVolume(boxMergePkg.getSumVolume());
                                    preloadDto.setMergePkgWeight(boxMergePkg.getSumWeight());
                                    preloadDto.setMergePkgQua(map.get(String.valueOf(boxMergePkg.getId())));
                                    preloadDto.setMergePkgCode(boxMergePkg.getPkgNum());
                                    if (mergeVolumnMap.get(String.valueOf(boxMergePkg.getId())) == null) {
                                        mergeVolumnMap.put(String.valueOf(boxMergePkg.getId()),boxMergePkg.getSumVolume());
                                    }
                                    if (mergeWeightMap.get(String.valueOf(boxMergePkg.getId())) == null) {
                                        mergeWeightMap.put(String.valueOf(boxMergePkg.getId()),boxMergePkg.getSumWeight());
                                    }
                                    if (mergeQualityMap.get(String.valueOf(boxMergePkg.getId())) == null) {
                                        mergeQualityMap.put(String.valueOf(boxMergePkg.getId()),map.get(String.valueOf(boxMergePkg.getId())));
                                    }
                                } else {
                                    sumMergeCount = sumMergeCount + item.getCartonsNum();
                                    preloadDto.setMergePkgNum(item.getCartonsNum());
                                    preloadDto.setMergePkgBoxGauge(item.getBoxGauge());
                                    preloadDto.setMergePkgVolume(item.getVolume());
                                    preloadDto.setMergePkgWeight(item.getWeight());
                                    if (item.getQuantityAll() == null) {
                                        preloadDto.setMergePkgQua(null);
                                    } else {
                                        preloadDto.setMergePkgQua(item.getQuantityAll());
                                        totalMergeQuality = totalMergeQuality + item.getQuantityAll();
                                    }
                                    preloadDto.setMergePkgCode(orderInfo.getOrderNo());
                                    totalMergeVolume = totalMergeVolume.add(item.getVolume());
                                    totalMergeWeight = totalMergeWeight.add(item.getWeight());
                                }
                                if(StringUtils.isNotBlank(item.getMaterial())){
                                    preloadDto.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",item.getMaterial()+"").getLabel());
                                }
                                preloadDto.setUsageIds(getUsageMsg(item.getUsageIds()));
                                BigDecimal totalWorth = orderInfo.getOrderItemVOList().stream().map(OrderItemBackVO::getWorth).reduce(BigDecimal.ZERO, BigDecimal::add);
                                preloadDto.setWorth(totalWorth);
                                preloadDto.setTotalVolume(orderDO.getSumVolumeFinishedWarehouseIn().compareTo(BigDecimal.ZERO) == 0 ? orderInfo.getSumVolume():orderDO.getSumVolumeFinishedWarehouseIn());
                                preloadDto.setTotalWeight(orderDO.getSumWeightFinishedWarehouseIn().compareTo(BigDecimal.ZERO) == 0 ? orderInfo.getSumWeight():orderDO.getSumWeightFinishedWarehouseIn());
                                preloadDtoList.add(preloadDto);
                                if (StringUtils.isNotBlank(item.getOrderWarehouseInDetails())) {
                                    item.setOrderWarehouseInDetailsVOList(JsonUtils.parseArray(item.getOrderWarehouseInDetails(), OrderWarehouseInDetailsVO.class));
                                }
                                if (CollectionUtil.isNotEmpty(item.getOrderWarehouseInDetailsVOList())) {
                                    mergeCount = mergeCount + item.getOrderWarehouseInDetailsVOList().size();
                                    List<OrderWarehouseInDetailsVO> orderWarehouseInDetailsVOList = item.getOrderWarehouseInDetailsVOList();
                                    for (int j = 0; j < orderWarehouseInDetailsVOList.size(); j++) {
                                        OrderWarehouseInDetailsVO orderWarehouseInDetailsVO = orderWarehouseInDetailsVOList.get(j);
                                        int andIncrement1 = colorRowNum.getAndIncrement();
                                        colorMap.put(String.valueOf(andIncrement1),colorList.get(colorIndex));
                                        colorRowList.add(andIncrement1);
                                        AirLoadDto preloadDto1 = new AirLoadDto();
                                        BeanUtil.copyProperties(preloadDto,preloadDto1);
                                        ProductBrankDO productBrankDetail = productBrankService.getProductBrank(orderWarehouseInDetailsVO.getBrand());
                                        if(productBrankDetail!=null && StringUtils.isNotBlank(productBrankDetail.getTitleZh())){
                                            preloadDto1.setBrand(productBrankDetail.getTitleZh());
                                        }
                                        if(productBrank !=null && productBrank.getFiling()!=null){
                                            preloadDto1.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                        }
                                        preloadDto1.setQuantity(orderWarehouseInDetailsVO.getQuantityAll());
                                        if (StringUtils.isNotBlank(orderWarehouseInDetailsVO.getMaterial())) {
                                            preloadDto1.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",orderWarehouseInDetailsVO.getMaterial()+"").getLabel());
                                        }
                                        preloadDto1.setProdAttr(getAttrNameByIds(orderWarehouseInDetailsVO.getProdAttrIds(),idNameMap));
//                                        if (StringUtils.isNotBlank(orderWarehouseInDetailsVO.getUnit())) {
//                                            preloadDto1.setPackageType(DictFrameworkUtils.getDictDataFromCache("packaging_type",orderWarehouseInDetailsVO.getUnit()).getLabel());
//                                        }
                                        preloadDto1.setUsageIds(getUsageMsg(orderWarehouseInDetailsVO.getUsageIds()));
//                                        preloadDto1.setNumber(orderWarehouseInDetailsVO.getExpressNo());
                                        ProductDO product = productService.getProduct(orderWarehouseInDetailsVO.getProdId());
                                        if (product != null) {
                                            preloadDto1.setProdTitleZh(product.getTitleZh() + "" + product.getTitleEn());
                                        }
                                        preloadDtoList.add(preloadDto1);
                                    }
                                }
                                mergeCountList.add(mergeCount);
                                if (colorIndex == 7) {
                                    colorIndex = 0;
                                } else {
                                    colorIndex ++;
                                }
                                //合包那几个列合并单元格计算
                                if (pkgId.longValue() != 0L) {
                                    if (mergePkgCountMap.get(String.valueOf(pkgId)) == null) {
                                        mergePkgCountMap.put(String.valueOf(pkgId),mergeCount);
                                    } else {
                                        Integer sumMergePkgCount = mergePkgCountMap.get(String.valueOf(pkgId)) + mergeCount;
                                        mergePkgCountMap.put(String.valueOf(pkgId),sumMergePkgCount);
                                    }
                                } else {
                                    mergePkgCountMap.put(String.valueOf(item.getId()),mergeCount);
                                }
                            }
                            int sumNum = orderWarehouseInBackItemVOS.stream().mapToInt(j -> {
                                if (j.getOrderWarehouseInDetailsVOList() == null) {
                                    return 0;
                                } else {
                                    int sum = j.getOrderWarehouseInDetailsVOList().stream().mapToInt(vo -> Objects.isNull(vo.getCartonsNum()) ? 0 : vo.getCartonsNum()).sum();
                                    return sum;
                                }
                            }).sum();
                            BigDecimal sumVolume = orderWarehouseInBackItemVOS.stream().map(j -> {
                                if (j.getOrderWarehouseInDetailsVOList() == null) {
                                    return BigDecimal.ZERO;
                                } else {
                                    BigDecimal decimal = j.getOrderWarehouseInDetailsVOList().stream().map(vo -> Objects.isNull(vo.getVolume()) ? BigDecimal.ZERO : vo.getVolume()).reduce(BigDecimal.ZERO, BigDecimal::add);
                                    return decimal;
                                }
                            }).reduce(BigDecimal.ZERO, BigDecimal::add);
                            BigDecimal sumWeight = orderWarehouseInBackItemVOS.stream().map(j -> {
                                if (j.getOrderWarehouseInDetailsVOList() == null) {
                                    return BigDecimal.ZERO;
                                } else {
                                    BigDecimal decimal = j.getOrderWarehouseInDetailsVOList().stream().map(vo -> Objects.isNull(vo.getWeight()) ? BigDecimal.ZERO : vo.getWeight()).reduce(BigDecimal.ZERO, BigDecimal::add);
                                    return decimal;
                                }
                            }).reduce(BigDecimal.ZERO, BigDecimal::add);
                            totalNum = totalNum + sumNum;
                            totalVolume = totalVolume.add(sumVolume);
                            totalWeight = totalWeight.add(sumWeight);
                            totalNum = totalNum + orderInfo.getSumNum();
                            totalVolume = totalVolume.add(orderInfo.getSumVolume());
                            totalWeight = totalWeight.add(orderInfo.getSumWeight());
                        }
                        AirLoadDto preloadDto = new AirLoadDto();
                        BigDecimal volumeReduce = mergeVolumnMap.values().stream().reduce(BigDecimal.ZERO, BigDecimal::add);
                        BigDecimal weightReduce = mergeWeightMap.values().stream().reduce(BigDecimal.ZERO, BigDecimal::add);
                        totalMergeVolume = totalMergeVolume.add(volumeReduce);
                        totalMergeWeight = totalMergeWeight.add(weightReduce);
                        Integer qualityReduce = mergeQualityMap.values().stream().reduce(0, Integer::sum);
                        totalMergeQuality = totalMergeQuality + qualityReduce.intValue();
                        preloadDto.setId("TOTAL");
                        preloadDto.setSumNum(totalNum);
                        preloadDto.setSumVolume(totalVolume);
                        preloadDto.setSumWeight(totalWeight);
                        preloadDto.setMergePkgNum(sumMergeCount);
                        preloadDto.setMergePkgVolume(totalMergeVolume);
                        preloadDto.setMergePkgWeight(totalMergeWeight);
                        preloadDto.setMergePkgQua(totalMergeQuality);
                        preloadDtoList.add(preloadDto);
                        allTotalNum = allTotalNum + totalNum;
                        allTotalVolume = allTotalVolume.add(totalVolume);
                        allTotalWeight = allTotalWeight.add(totalWeight);
                        //2023-12-19 客户要求空运已装单取消一个总计，因为海运分部分有总计  空运不分部分
//                        if(i==secCount.size()-1){
//                            AirLoadDto preloadDto1 = new AirLoadDto();
//                            preloadDto1.setId("IN TOTAL");
//                            preloadDto1.setSumNum(allTotalNum);
//                            preloadDto1.setSumVolume(allTotalVolume);
//                            preloadDto1.setSumWeight(allTotalWeight);
//                            preloadDtoList.add(preloadDto1);
//                        }
                        List<Integer> mergePkgCountList = mergePkgCountMap.entrySet().stream().map(Map.Entry::getValue).collect(Collectors.toList());
                        WriteTable writeTable = EasyExcel.writerTable(i)
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,0,1))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,1,1))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,2,1))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,3, colorMap))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,4, colorMap))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,5, colorMap))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,6, colorMap))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,7, colorMap))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,8,1))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,9,2))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,10,2))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,11,2))
//                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,11, colorMap))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,12,2))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,13,2))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,14,3))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,15,3))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,16,3))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,17,3))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,18,3))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,mergePkgCountList,19,3))
//                                .registerWriteHandler(new CustomMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,20,1))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,20, colorMap))
                                .registerWriteHandler(new CustomCellWriteHandler(colorRowList,21, colorMap))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,22,1))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,23,1))
                                .registerWriteHandler(new CustomAirLoadMergeStrategyNew(preloadDtoList,mergeCountList,colorRowList,null,24,1))
                                .automaticMergeHead(true).head(headList).needHead(true).build();
                        excelWriter.write(preloadDtoList,sheet,writeTable);
                    }
                }
                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                log.info("预装单导出失败",e);
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }

    private String getUsageMsg(String usageIdsDetail) {
        if (StringUtils.isNotEmpty(usageIdsDetail)) {
            StringJoiner labelJoiner = new StringJoiner(",");
            for (String usageId : usageIdsDetail.split(",")) {
                DictDataRespDTO usageValue = DictFrameworkUtils.getDictDataFromCache("order_item_usage", usageId);
                if (usageValue != null) {
                    labelJoiner.add(usageValue.getLabel());
                }
            }
            return labelJoiner.toString();
        } else {
            return "";
        }
    }

    private String getAttrNameByIds(String attrIds, Map<String, String> idNameMap) {
        StringJoiner labelJoiner = new StringJoiner(",");
        if(StringUtils.isNotBlank(attrIds)) {
            String[] attrIdArray = attrIds.split(",");
            for (String attrId : attrIdArray) {
                //map一定包含数据，否则是数据错误
                labelJoiner.add(idNameMap.get(attrId));
            }
            return labelJoiner.toString();
        }
        return "";
    }
}
