package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentLoadExcelExportPushEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentReceivableExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomMergeStrategy;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableOrderPayedFeeTypeVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ShipmentReceivableExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentReceivableExcelExportListener {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private CurrencyService currencyService;

    @Resource
    private AdminUserService adminUserService;

    @Resource
    private ReceivableService receivableService;

//    @Value("${ue.templates-url}")
//    private String path;

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentReceivableExcelExportPushEvent.class)
    public void shipmentReceivableExcelExportPushEvent(ShipmentReceivableExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/receivable/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_receivable.xlsx");
                List<CurrencyDO> currencyList = currencyService.getCurrencyList();
                Map<Integer, String> currencyMap = currencyList.stream().collect(Collectors.toMap(CurrencyDO::getId, CurrencyDO::getTitleEn, (key1, key2) -> key2));
                InputStream inputStream = null;
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");
                inputStream = getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/nrlyReceivable.xlsx");
                Map headMap = new HashMap();
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                }
                BoxDO box = boxService.getBox(shipmentId);
                CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
                headMap.put("selfNo", box.getSelfNo());
                if (cabinet != null) {
                    headMap.put("cabinetName", cabinet.getName());
                }
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                headMap.put("dgDate", DateUtils.formatDate(box.getDgDate()));
                //以secId排序
                goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getTidanNum));

                List<Map<String, Object>> list = new ArrayList<>();

                int totalNum = 0;
                BigDecimal totalVolume = new BigDecimal(0);
                BigDecimal totalWeight = new BigDecimal(0);
                BigDecimal totalCollectionProxy = new BigDecimal(0);
                BigDecimal totalClearingCharge = new BigDecimal(0);
                BigDecimal totalSeaFreight = new BigDecimal(0);
                BigDecimal totalNrlyClearingCharge = new BigDecimal(0);
                BigDecimal totalNrlySeaFreight = new BigDecimal(0);
                BigDecimal totalZgClearingCharge = new BigDecimal(0);
                BigDecimal totalZgSeaFreight = new BigDecimal(0);
                BigDecimal totalJnClearingCharge = new BigDecimal(0);
                BigDecimal totalJnSeaFreight = new BigDecimal(0);
                //以订单ID分组
                Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream()
                        .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,LinkedHashMap::new,Collectors.toList()));
                for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                    Long orderId = entry.getKey();
                    OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                    Map<String, Object> map = new HashMap<>();
                    map.put("consigneeName", orderInfo.getConsigneeVO().getName());
                    map.put("mark", orderInfo.getMarks());
                    map.put("orderNo", orderInfo.getOrderNo());
                    map.put("consigneePhone", "+"+orderInfo.getConsigneeVO().getCountryCode()+orderInfo.getConsigneeVO().getPhone());
                    map.put("consigneeEmail", orderInfo.getConsigneeVO().getEmail());
                    map.put("consigneeAddress", orderInfo.getConsigneeVO().getAddress());
                    map.put("pickNum", boxPreloadGoodsService.getLadingBillNo(orderId));
                    //todo 计算应收明细的核销比例,待测试，未减优惠金额
                    List<ReceivableOrderPayedFeeTypeVO> payedFeeTypeVOList=receivableService.getPayedMoneyByFeeType(orderId);
                    BigDecimal zgyfyf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 1 && item.getFeeType() == 1 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("zgSeaFreight", zgyfyf);
                    BigDecimal zgyfqgf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 1 && item.getFeeType() == 2 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("zgClearingCharge", zgyfqgf);
                    BigDecimal jnyfyf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 3 && item.getFeeType() == 1 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("jnSeaFreight", jnyfyf);
                    BigDecimal jnyfqgf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 3 && item.getFeeType() == 2 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("jnClearingCharge", jnyfqgf);
                    BigDecimal nrlyyfyf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 2 && item.getFeeType() == 1 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("nrlySeaFreight", nrlyyfyf);
                    BigDecimal nrlyyfqgf = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 2 && item.getFeeType() == 2 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("nrlyClearingCharge", nrlyyfqgf);
                    BigDecimal ysyf = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 1).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal ysyfyhx = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 1 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    ysyf = ysyf.subtract(ysyfyhx);
                    map.put("seaFreight", ysyf);
                    BigDecimal ysqgf = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 2).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal ysqgfyhx = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 2 ).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    ysqgf = ysqgf.subtract(ysqgfyhx);
                    map.put("clearingCharge", ysqgf);
                    Map<String, BigDecimal> otherFee = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() != 1 && item.getFeeType() != 2).collect(Collectors.groupingBy(item -> item.getFeeType() + "," + item.getCurrencyId(), CollectorsUtil.summingBigDecimal(ReceivableOrderPayedFeeTypeVO::getTotalAmount)));
                    StringBuffer stringBuffer = new StringBuffer();
                    for (Map.Entry<String, BigDecimal> entry1 : otherFee.entrySet()) {
                        DictDataRespDTO data = DictFrameworkUtils.getDictDataFromCache("receivable_fee_type", entry1.getKey().split(",")[0]);
                        stringBuffer.append(data.getLabelEn()).append(":").append(entry1.getValue()).append(currencyMap.get(Integer.valueOf(entry1.getKey().split(",")[1]))).append("\n");
                    }
                    map.put("otherFee", stringBuffer.toString());
                    map.put("collectionProxy", orderInfo.getCollectionProxy());
                    map.put("sumNum", orderInfo.getSumNum());
                    map.put("sumVolume", orderInfo.getSumVolume());
                    map.put("sumWeight", orderInfo.getSumWeight());
                    map.put("isCargoControl", orderInfo.getIsCargoControl()? "YES":"NO");
                    map.put("objectiveName", orderInfo.getLogisticsInfoDto().getDestTitleEn());
                    map.put("consignorPhone", orderInfo.getConsignorVO().getPhone());
                    map.put("consignorEmail", orderInfo.getConsignorVO().getEmail());
                    if (orderInfo.getSalesmanId() !=null && orderInfo.getSalesmanId() > 0L) {
                        AdminUserDO user = adminUserService.getUser(orderInfo.getSalesmanId());
                        map.put("salesmanId", user.getNickname());
                    }
                    list.add(map);
                    totalNum = totalNum + orderInfo.getSumNum();
                    totalVolume = totalVolume.add(orderInfo.getSumVolume());
                    totalWeight = totalWeight.add(orderInfo.getSumWeight());
                    totalCollectionProxy = totalCollectionProxy.add(orderInfo.getCollectionProxy());
                    totalClearingCharge = totalClearingCharge.add(ysqgf);
                    totalSeaFreight = totalSeaFreight.add(ysyf);
                    totalNrlyClearingCharge = totalNrlyClearingCharge.add(nrlyyfqgf);
                    totalNrlySeaFreight = totalNrlySeaFreight.add(nrlyyfyf);
                    totalZgClearingCharge = totalZgClearingCharge.add(zgyfqgf);
                    totalZgSeaFreight = totalZgSeaFreight.add(zgyfyf);
                    totalJnClearingCharge = totalJnClearingCharge.add(jnyfqgf);
                    totalJnSeaFreight = totalJnSeaFreight.add(jnyfyf);
                }

                headMap.put("totalNum", totalNum);
                headMap.put("totalVolume", totalVolume.doubleValue());
                headMap.put("totalWeight", totalWeight.doubleValue());
                headMap.put("totalCollectionProxy", totalCollectionProxy.doubleValue());
                headMap.put("totalClearingCharge", totalClearingCharge.doubleValue());
                headMap.put("totalSeaFreight", totalSeaFreight.doubleValue());
                headMap.put("totalNrlyClearingCharge", totalNrlyClearingCharge.doubleValue());
                headMap.put("totalNrlySeaFreight", totalNrlySeaFreight.doubleValue());
                headMap.put("totalZgClearingCharge", totalZgClearingCharge.doubleValue());
                headMap.put("totalZgSeaFreight", totalZgSeaFreight.doubleValue());
                headMap.put("totalJnClearingCharge", totalJnClearingCharge.doubleValue());
                headMap.put("totalJnSeaFreight", totalJnSeaFreight.doubleValue());
                File fileDir = new File(dir);
                if (!fileDir.exists()){
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();
                excelWriter.fill(headMap, sheet);
                excelWriter.fill(list, fillConfig, sheet);
                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }
}
