package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ContainerNumberOrderExcelExportEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentPreloadExcelExportPushEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentSeaPreloadExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLocation.OrderLocationDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemDo;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.dto.SeaPreloadDto;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomMergeStrategy;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;


/**
 * 自编号订单excel导出监听
 *
 * @author zhengYi
 */
@Component("ShipmentSeaPreloadExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentSeaPreloadExcelExportListener {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private ProductBrankService productBrankService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderLocationService orderLocationService;

    @Resource
    private OrderGuanlianService orderGuanlianService;

    @Resource
    private OrderWarehouseInService orderWarehouseInService;

    @Resource
    private OrderService orderService;

//    @Value("${ue.templates-url}")
//    private String path;

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentSeaPreloadExcelExportPushEvent.class)
    public void shipmentPreloadExcelExportPushEvent(ShipmentSeaPreloadExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/preload/excel/");
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_preload.xlsx");
                InputStream inputStream = null;
                inputStream = this.getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/seaPreload.xlsx");
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");
                Map headMap = new HashMap();
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
//                    return error(HAVE_NOT_PRELOAD_GOODS);
                }
                BoxDO box = boxService.getBox(shipmentId);
                CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
                BoxApprovalDO boxApprovalDO = boxApprovalService.getOne(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalType, 1).last("limit 1"));
                headMap.put("selfNo", box.getSelfNo());
                if(cabinet != null) {
                    headMap.put("cabinetName", cabinet.getName());
                }
                headMap.put("exportDate", DateUtils.formatDate(new Date()));
                if (boxApprovalDO != null) {
                    headMap.put("date", DateUtils.formatDate(boxApprovalDO.getUpdateTime()));
                }
                File fileDir = new File(dir);
                if (!fileDir.exists()){
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                excelWriter.fill(headMap, sheet);
                //以secId排序
                goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getSecId).thenComparing(BoxPreloadGoodsBackVO::getTidanNum).thenComparing(BoxPreloadGoodsBackVO::getId));
                int allTotalNum = 0;
                BigDecimal allTotalVolume = new BigDecimal(0);
                BigDecimal allTotalWeight = new BigDecimal(0);
                List<Long> secCount = goodsItemList.stream().map(BoxPreloadGoodsBackVO::getSecId).distinct().sorted().collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(secCount)) {
                    for (int i=0;i<secCount.size();i++){
                        List<String> list1 = new ArrayList();
                        List<List<String>> headList = new ArrayList<>();
                        String head = "";
                        if (i == 0) {
                            head = "第"+ NumberUtils.int2chineseNum(i+1) +"部分(Be loaded inside of the container)";
                        } else {
                            head = "第"+NumberUtils.int2chineseNum(i+1)+"部分(after "+ NumberUtils.format(i+1)+" parts)";
                        }
                        list1.add(head);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);
                        headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);headList.add(list1);

                        List<SeaPreloadDto> list = new ArrayList<>();

                        int totalNum = 0;
                        BigDecimal totalVolume = new BigDecimal(0);
                        BigDecimal totalWeight = new BigDecimal(0);

                        Map<String,Integer> numMap = new HashMap();

                        Long part = secCount.get(i);

                        //以订单ID分组
                        Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream().filter(item -> item.getSecId().longValue()==part.longValue() && StringUtils.isNotBlank(item.getWarehouseInInfo()))
                                .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId,LinkedHashMap::new,Collectors.toList()));
                        WriteTable writeTable = EasyExcel.writerTable(i)
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,0,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,1,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,2,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,3,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,4,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,5,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,6,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,7,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,8,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,9,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,10,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,11,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,15,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,16,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,17,true))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,18,false))
                                .registerWriteHandler(new CustomMergeStrategy(orderMap,orderWarehouseInService,orderItemService,20,false))
                                .automaticMergeHead(true).head(headList).needHead(true).build();
                        for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                            Long orderId = entry.getKey();
                            OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                            List<BoxPreloadGoodsBackVO> boxPreload = entry.getValue();
                            List<OrderWarehouseInBackVO> orderWarehouseInList = orderWarehouseInService.getOrderWarehouseInList(orderId);
                            List<OrderItemDO> orderItems = boxPreload.stream().map(item -> orderItemService.getOrderItem(item.getOrderItemId())).collect(Collectors.toList());
                            orderItems = orderItems.stream().sorted(Comparator.comparing(OrderItemDO::getOrderItemId)).collect(Collectors.toList());
                            for(OrderItemDO item : orderItems) {
                                List<OrderWarehouseInBackVO> warehouseInBackVOS = orderWarehouseInList.stream().filter(it -> it.getOrderItemId().longValue() == item.getOrderItemId().longValue()).collect(Collectors.toList());
                                List<OrderWarehouseInBackItemDo> orderWarehouseInBackItemDoList = warehouseInBackVOS.get(0).getOrderWarehouseInBackItemDoList();
                                Map<Date, List<OrderWarehouseInBackItemDo>> collect = orderWarehouseInBackItemDoList.stream().collect(Collectors.groupingBy(OrderWarehouseInBackItemDo::getInTime));
                                Map<Date, Map<String, Map<String, List<OrderWarehouseInBackItemDo>>>> collect1 = orderWarehouseInBackItemDoList.stream().collect(Collectors.groupingBy(OrderWarehouseInBackItemDo::getInTime, Collectors.groupingBy(OrderWarehouseInBackItemDo::getUnit, Collectors.groupingBy(OrderWarehouseInBackItemDo::getOrderLocationStr4Merge))));
                                for (OrderWarehouseInBackItemDo warehouse : orderWarehouseInBackItemDoList) {
                                    SeaPreloadDto preloadDto = new SeaPreloadDto();
                                    preloadDto.setOrderNo(orderInfo.getOrderNo());
                                    if (numMap.get(orderInfo.getOrderNo()) != null) {
                                        preloadDto.setId(String.valueOf(numMap.get(orderInfo.getOrderNo())));
                                    } else {
                                        preloadDto.setId(String.valueOf(numMap.size()+1));
                                        numMap.put(orderInfo.getOrderNo(),numMap.size()+1);
                                    }
                                    preloadDto.setRucangTime(warehouse.getInTime());
                                    preloadDto.setMarks(orderInfo.getMarks());
                                    List<OrderGuanlianDO> guanlianList = orderGuanlianService.getGuanlianListByOrderId(orderInfo.getOrderId());
                                    //rIdList为此订单的所有关联订单
                                    List<Long> rIdList = new ArrayList<>();
                                    for (OrderGuanlianDO orderGuanlianDO : guanlianList) {
                                        Long oId = orderGuanlianDO.getOrderId();
                                        Long rId = orderGuanlianDO.getRelateOrderId();
                                        if(oId.equals(orderId)) {
                                            rIdList.add((rId));
                                        } else if(rId.equals(orderId)) {
                                            rIdList.add(oId);
                                        }
                                    }
                                    if (CollectionUtil.isNotEmpty(rIdList)) {
                                        List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, rIdList);
                                        if (CollectionUtil.isNotEmpty(orderDOS)) {
                                            preloadDto.setRelateOrder(orderDOS.stream().map(OrderDO::getOrderNo).collect(Collectors.joining(",")));
                                        }
                                    }
                                    preloadDto.setProdTitleZh(item.getProdTitleZh()+ " " +item.getProdTitleEn());
                                    preloadDto.setMerge(orderInfo.getOldNumbers());
                                    ProductBrankDO productBrank = productBrankService.getProductBrank(item.getBrand());
                                    if(productBrank!=null && StringUtils.isNotBlank(productBrank.getTitleZh())){
                                        preloadDto.setBrand(productBrank.getTitleZh());
                                    }
                                    if(StringUtils.isNotBlank(item.getMaterial())){
                                        preloadDto.setMaterial(DictFrameworkUtils.getDictDataFromCache("product_material",item.getMaterial()+"").getLabel());
                                    }
                                    if(productBrank !=null && productBrank.getFiling()!=null){
                                        preloadDto.setProductRecord(productBrank.getFiling().equals("0")? "N":"Y");
                                    }
                                    if (warehouse != null) {
                                        String warehousingSpecificationType = collect.get(warehouse.getInTime()).stream().map(j -> {
                                            return DictFrameworkUtils.getDictDataFromCache("warehousing_specification_type", j.getSpecificationType() + "").getLabel();
                                        }).distinct().collect(Collectors.joining("/"));
//                                String packagingType = orderWarehouseInBackItemDoList.stream().map(j -> {
//                                    return DictFrameworkUtils.getDictDataFromCache("packaging_type", j.getUnit()).getLabel();
//                                }).distinct().collect(Collectors.joining("/"));
                                        preloadDto.setPackageType(DictFrameworkUtils.getDictDataFromCache("packaging_type",warehouse.getUnit()).getLabel());
                                        preloadDto.setUnit(warehousingSpecificationType);
                                    }
                                    List<OrderLocationDO> orderLocationListByWarehouseInId = orderLocationService.getOrderLocationListByWarehouseInId(warehouse.getId());
                                    StringBuffer stringBuffer = new StringBuffer("");
                                    if (orderLocationListByWarehouseInId != null) {
                                        orderLocationListByWarehouseInId.forEach(s -> stringBuffer.append(StringUtils.isBlank(s.getAreaName()) ? "":s.getAreaName()).append(StringUtils.isBlank(s.getLocationName()) ? "":s.getLocationName()).append(","));
                                        if (stringBuffer.length() > 0) {
                                            preloadDto.setPrincess(stringBuffer.deleteCharAt(stringBuffer.length()-1).toString());
                                        }
                                    }
                                    preloadDto.setNumber(warehouse.getExpressNo());
                                    preloadDto.setSumNum(collect1.get(warehouse.getInTime()).get(warehouse.getUnit()).get(warehouse.getOrderLocationStr4Merge()).stream().mapToInt(OrderWarehouseInBackItemDo::getCartonsNum).sum());
                                    preloadDto.setSumVolume(collect1.get(warehouse.getInTime()).get(warehouse.getUnit()).get(warehouse.getOrderLocationStr4Merge()).stream().map(OrderWarehouseInBackItemDo::getVolume).reduce(BigDecimal.ZERO,BigDecimal::add));
                                    preloadDto.setSumWeight(collect1.get(warehouse.getInTime()).get(warehouse.getUnit()).get(warehouse.getOrderLocationStr4Merge()).stream().map(OrderWarehouseInBackItemDo::getWeight).reduce(BigDecimal.ZERO,BigDecimal::add));
                                    if(orderInfo.getCustomsType()!=null){
                                        preloadDto.setCustomsType(DictFrameworkUtils.getDictDataFromCache("customs_type",orderInfo.getCustomsType()+"").getLabel());
                                    }
                                    preloadDto.setWorth(item.getWorth());
//                                    preloadDto.setQuantity(collect1.get(warehouse.getInTime()).get(warehouse.getUnit()).get(warehouse.getOrderLocationStr4Merge()).stream().mapToInt(OrderWarehouseInBackItemDo::getQuantityAll).sum());
                                    preloadDto.setQuantity(collect1.get(warehouse.getInTime()).get(warehouse.getUnit()).get(warehouse.getOrderLocationStr4Merge()).stream().mapToInt(s -> Objects.isNull(s.getQuantityAll()) ? 0:s.getQuantityAll()).sum());
//                                    preloadDto.setQuantity(collect.get(warehouse.getInTime()).stream().filter(jj -> jj.getQuantityAll() !=null ).mapToInt(OrderWarehouseInBackItemDo::getQuantityAll).sum());
                                    list.add(preloadDto);
                                }
                                totalNum = totalNum + item.getWarehouseInInfoVO().getCartonsNum();
                                totalVolume = totalVolume.add(item.getWarehouseInInfoVO().getVolume());
                                totalWeight = totalWeight.add(item.getWarehouseInInfoVO().getWeight());
                            }
                        }
                        SeaPreloadDto preloadDto = new SeaPreloadDto();
                        preloadDto.setId("TOTAL");
                        preloadDto.setSumNum(totalNum);
                        preloadDto.setSumVolume(totalVolume);
                        preloadDto.setSumWeight(totalWeight);
                        list.add(preloadDto);
                        allTotalNum = allTotalNum + totalNum;
                        allTotalVolume = allTotalVolume.add(totalVolume);
                        allTotalWeight = allTotalWeight.add(totalWeight);
                        if(i==secCount.size()-1){
                            SeaPreloadDto preloadDto1 = new SeaPreloadDto();
                            preloadDto1.setId("IN TOTAL");
                            preloadDto1.setSumNum(allTotalNum);
                            preloadDto1.setSumVolume(allTotalVolume);
                            preloadDto1.setSumWeight(allTotalWeight);
                            list.add(preloadDto1);
                        }
                        excelWriter.write(list,sheet,writeTable);
                    }
                }
                excelWriter.finish();
                inputStream.close();
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }
}

