package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentSummeryExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dto.ContainerNumberSummaryDto;
import cn.iocoder.yudao.module.shipment.enums.SlStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxFinanceBackVO;
import cn.iocoder.yudao.module.shipment.vo.box.BoxSettlementQueryVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;

@Component("ShipmentSummeryExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentSummeryExcelExportListener {

    @Resource
    private BoxService boxService;

    private UeProperties ueProperties;

    private FileService fileService;

    @EventListener(ShipmentSummeryExcelExportPushEvent.class)
    public void airBillReceivableExcelExportPushEvent(ShipmentSummeryExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                PageVO page = new PageVO();
                page.setRows(10000);
                page.setPage(1);

                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/shipmentSummery/excel/");
                File fileDir = new File(dir);
                if(!fileDir.exists()) {
                    fileDir.mkdirs();
                }
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_summery.xlsx");
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                String queryString = jsonObject.getString("query");
                BoxSettlementQueryVO query = JSONObject.parseObject(queryString,BoxSettlementQueryVO.class);
                PageResult<BoxFinanceBackVO> boxPage = boxService.getBoxSettlementPage(query, page);
                List<BoxFinanceBackVO> records = boxPage.getList();

                if (CollectionUtil.isNotEmpty(records)) {
                    InputStream inputStream = getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/box_finance.xlsx");

                    BoxFinanceBackVO data = records.get(0);
                    RegionDO regionDO = data.getDestRegionDO();

                    String nameZh = "自编号报表汇总统计";
                    String nameEn = "Summary Statistics of Shipment";
                    String sheetName = I18nMessage.getLang() == 0 ? nameZh : nameEn;

                    //应收款单位
                    String yskUnit = "";
                    //应收款运费单位
                    String yfUnit = "";
                    //应收款清关费单位
                    String qgfUnit = "";
                    //应收款额外费用单位
                    String ewUnit = "";
                    //目的地国家中文名称
                    String dZhName = regionDO.getTitleZh();
                    //目的地国家英文名称
                    String dEnName = regionDO.getTitleEn();
                    //目的国运费单位
                    String myfUnit = "";
                    //目的国清关费单位
                    String mqgfUnit = "";
                    //目的国额外费用单位
                    String mewUnit = "";
                    //始发国运费单位
                    String syfUnit = "";
                    //始发国清关费单位
                    String sqgfUnit = "";
                    //始发国额外费用单位
                    String sewUnit = "";
                    //折扣运费单位
                    String zyfUnit = "";
                    //折扣清关费单位
                    String zqgfUnit = "";
                    //折扣额外费用单位
                    String zewUnit = "";

                    Map headMap = new HashMap();

                    ContainerNumberSummaryDto summaryInfo = data.getSummaryInfo();
                    if(summaryInfo != null) {
                        yskUnit = summaryInfo.getTotalFeeCurrency();
                        yfUnit = summaryInfo.getTotalFreightFeeCurrency();
                        qgfUnit = summaryInfo.getTotalClearanceFeeCurrency();
                        ewUnit = summaryInfo.getTotalOtherFeeCurrency();
                        myfUnit = summaryInfo.getNetReceiptsCollectFreightFeeCurrency();
                        mqgfUnit = summaryInfo.getNetReceiptsCollectClearanceFeeCurrency();
                        mewUnit = summaryInfo.getNetReceiptsCollectOtherFeeCurrency();
                        syfUnit = summaryInfo.getNetReceiptsAdvanceFreightFeeCurrency();
                        sqgfUnit = summaryInfo.getNetReceiptsAdvanceClearanceFeeCurrency();
                        sewUnit = summaryInfo.getNetReceiptsAdvanceOtherFeeCurrency();
                        zyfUnit = myfUnit;
                        zqgfUnit = mqgfUnit;
                        zewUnit = mewUnit;
                    }
                    headMap.put("yskUnit", yskUnit);
                    headMap.put("yfUnit", yfUnit);
                    headMap.put("qgfUnit", qgfUnit);
                    headMap.put("ewUnit", ewUnit);
                    headMap.put("myfUnit", myfUnit);
                    headMap.put("mqgfUnit", mqgfUnit);
                    headMap.put("mewUnit", mewUnit);
                    headMap.put("syfUnit", syfUnit);
                    headMap.put("sqgfUnit", sqgfUnit);
                    headMap.put("sewUnit", sewUnit);
                    headMap.put("zyfUnit", zyfUnit);
                    headMap.put("zqgfUnit", zqgfUnit);
                    headMap.put("zewUnit", zewUnit);
                    headMap.put("dZhName", dZhName);
                    headMap.put("dEnName", dEnName);

                    List<Map<String, Object>> list = new ArrayList<>();

                    BigDecimal totalYskFee = BigDecimal.ZERO;
                    BigDecimal totalYfFee = BigDecimal.ZERO;
                    BigDecimal totalQgfFee = BigDecimal.ZERO;
                    BigDecimal totalEwFee = BigDecimal.ZERO;
                    BigDecimal totalMfyFee = BigDecimal.ZERO;
                    BigDecimal totalMqgfFee = BigDecimal.ZERO;
                    BigDecimal totalMewFee = BigDecimal.ZERO;
                    BigDecimal totalSyfFee = BigDecimal.ZERO;
                    BigDecimal totalSqgfFee = BigDecimal.ZERO;
                    BigDecimal totalSewFee = BigDecimal.ZERO;
                    BigDecimal totalZyfFee = BigDecimal.ZERO;
                    BigDecimal totalZqgfFee = BigDecimal.ZERO;
                    BigDecimal totalZewFee = BigDecimal.ZERO;

                    for (BoxFinanceBackVO item : records) {
                        String jsName = SlStatueEnum.getTextName(item.getSlStatus());

                        Map<String, Object> map = new HashMap<>();
                        map.put("jsName", jsName);
                        map.put("selfNo", item.getSelfNo());
                        String yskFee = "";
                        String yfFee = "";
                        String qgfFee = "";
                        String ewFee = "";
                        String myfFee = "";
                        String mqgfFee = "";
                        String mewFee = "";
                        String syfFee = "";
                        String sqgfFee = "";
                        String sewFee = "";
                        String zyfFee = "";
                        String zqgfFee = "";
                        String zewFee = "";
                        ContainerNumberSummaryDto containerNumberSummary = item.getSummaryInfo();
                        if(containerNumberSummary != null) {
                            yskFee = containerNumberSummary.getReceivableTotalFee() + yskUnit;
                            totalYskFee = totalYskFee.add(containerNumberSummary.getReceivableTotalFee());

                            yfFee = containerNumberSummary.getTotalReceivableFreightFee() + yfUnit;
                            totalYfFee = totalYfFee.add(containerNumberSummary.getTotalReceivableFreightFee());

                            qgfFee = containerNumberSummary.getTotalReceivableClearanceFee() + qgfUnit;
                            totalQgfFee = totalQgfFee.add(containerNumberSummary.getTotalReceivableClearanceFee());

                            ewFee = containerNumberSummary.getTotalReceivableOtherFee() + ewUnit;
                            totalEwFee = totalEwFee.add(containerNumberSummary.getTotalReceivableOtherFee());

                            myfFee = containerNumberSummary.getTotalNetReceiptsCollectFreightFee() + myfUnit;
                            totalMfyFee = totalMfyFee.add(containerNumberSummary.getTotalNetReceiptsCollectFreightFee());

                            mqgfFee = containerNumberSummary.getTotalNetReceiptsCollectClearanceFee() + mqgfUnit;
                            totalMqgfFee = totalMqgfFee.add(containerNumberSummary.getTotalNetReceiptsCollectClearanceFee());

                            mewFee = containerNumberSummary.getTotalNetReceiptsCollectOtherFee() + mewUnit;
                            totalMewFee = totalMewFee.add(containerNumberSummary.getTotalNetReceiptsCollectOtherFee());

                            syfFee = containerNumberSummary.getTotalNetReceiptsAdvanceFreightFee() + syfUnit;
                            totalSyfFee = totalSyfFee.add(containerNumberSummary.getTotalNetReceiptsAdvanceFreightFee());

                            sqgfFee = containerNumberSummary.getTotalNetReceiptsAdvanceClearanceFee() + sqgfUnit;
                            totalSqgfFee = totalSqgfFee.add(containerNumberSummary.getTotalNetReceiptsAdvanceClearanceFee());

                            sewFee = containerNumberSummary.getTotalNetReceiptsAdvanceOtherFee() + sewUnit;
                            totalSewFee = totalSewFee.add(containerNumberSummary.getTotalNetReceiptsAdvanceOtherFee());

                            zyfFee = containerNumberSummary.getDiscountFreightFee() + zyfUnit;
                            totalZyfFee = totalZyfFee.add(containerNumberSummary.getDiscountFreightFee());

                            zqgfFee = containerNumberSummary.getDiscountClearanceFee() + zqgfUnit;
                            totalZqgfFee = totalZqgfFee.add(containerNumberSummary.getDiscountClearanceFee());

                            zewFee = containerNumberSummary.getDiscountOtherFee() + zewUnit;
                            totalZewFee = totalZewFee.add(containerNumberSummary.getDiscountOtherFee());

                        }
                        map.put("yskFee", yskFee);
                        map.put("yfFee", yfFee);
                        map.put("qgfFee", qgfFee);
                        map.put("ewFee", ewFee);
                        map.put("myfFee", myfFee);
                        map.put("mqgfFee", mqgfFee);
                        map.put("mewFee", mewFee);
                        map.put("syfFee", syfFee);
                        map.put("sqgfFee", sqgfFee);
                        map.put("sewFee", sewFee);
                        map.put("zyfFee", zyfFee);
                        map.put("zqgfFee", zqgfFee);
                        map.put("zewFee", zewFee);

                        list.add(map);
                    }

                    headMap.put("totalYskFee", totalYskFee);
                    headMap.put("totalYfFee", totalYfFee);
                    headMap.put("totalQgfFee", totalQgfFee);
                    headMap.put("totalEwFee", totalEwFee);
                    headMap.put("totalMfyFee", totalMfyFee);
                    headMap.put("totalMqgfFee", totalMqgfFee);
                    headMap.put("totalMewFee", totalMewFee);
                    headMap.put("totalSyfFee", totalSyfFee);
                    headMap.put("totalSqgfFee", totalSqgfFee);
                    headMap.put("totalSewFee", totalSewFee);
                    headMap.put("totalZyfFee", totalZyfFee);
                    headMap.put("totalZqgfFee", totalZqgfFee);
                    headMap.put("totalZewFee", totalZewFee);

                    ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                    WriteSheet sheet = EasyExcel.writerSheet(0).build();
                    sheet.setSheetName(sheetName);
                    FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();
                    excelWriter.fill(headMap, sheet);
                    excelWriter.fill(list, fillConfig, sheet);
                    excelWriter.finish();
                    inputStream.close();
                } else {
                    ExcelWriter excelWriter = EasyExcel.write(dir + fileName).build();
                    WriteSheet sheet = EasyExcel.writerSheet(0).build();
                    excelWriter.write(null,sheet);
                    excelWriter.finish();
                }
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }
    }
}
