package cn.iocoder.yudao.module.shipment.service.boxAbnormal;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.api.internalMessage.InternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.order.enums.OrderAirTimeEnum;
import cn.iocoder.yudao.module.order.enums.OrderSeaTimeEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxAbnormalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.*;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxAbnormalMapper;
import cn.iocoder.yudao.module.shipment.enums.AbnormalStepAirEnum;
import cn.iocoder.yudao.module.shipment.enums.AbnormalStepEnum;
import cn.iocoder.yudao.module.shipment.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxArrival.BoxArrivalService;
import cn.iocoder.yudao.module.shipment.service.boxCustoms.BoxCustomsService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.service.boxShipping.BoxShippingService;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalUpdateReqVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_ABNORMAL_NOT_EXISTS;

/**
 * 出货异常记录 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxAbnormalServiceImpl extends AbstractService<BoxAbnormalMapper, BoxAbnormalDO> implements BoxAbnormalService {

    @Resource
    private BoxAbnormalMapper boxAbnormalMapper;

    @Resource
    @Lazy
    BoxService boxService;

    @Resource
    @Lazy
    private BoxPreloadGoodsService boxPreloadGoodsService;
    @Resource
    private BoxCustomsService boxCustomsService;
    @Resource
    private BoxShippingService boxShippingService;
    @Resource
    private BoxArrivalService boxArrivalService;

    @Resource
    private InternalMessageApi internalMessageApi;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxAbnormal(BoxAbnormalCreateReqVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();

        List<BoxAbnormalDO> abnormalList = boxService.getAbnormalList(shipmentId);
        if (CollectionUtil.isNotEmpty(abnormalList)) {
            throw new ServiceException(500, "当前有未处理的异常");
        }
        Integer opStep = createReqVO.getOpStep();
        if (opStep.equals(AbnormalStepEnum.CUSTOMS.getStep())) {
//            List<BoxCustomsDO> customList = boxCustomsService.list(new LambdaQueryWrapperX<BoxCustomsDO>()
//                .eq(BoxCustomsDO::getShipmentId, shipmentId)
//            );
//            if(CollectionUtil.isEmpty(customList)) {
//                throw new ServiceException(500, "报关不存在");
//            }
        } else if (opStep.equals(AbnormalStepEnum.SHIPPING.getStep())) {
//            List<BoxShippingDO> shippingList = boxShippingService.list(new LambdaQueryWrapperX<BoxShippingDO>()
//                .eq(BoxShippingDO::getShipmentId, shipmentId)
//            );
//            if(CollectionUtil.isEmpty(shippingList)) {
//                throw new ServiceException(500, "起运不存在");
//            }
        } else if (opStep.equals(AbnormalStepEnum.ARRIVAL.getStep())) {
//            List<BoxArrivalDO> arrivalList = boxArrivalService.list(new LambdaQueryWrapperX<BoxArrivalDO>()
//                .eq(BoxArrivalDO::getShipmentId, shipmentId)
//            );
//            if(CollectionUtil.isEmpty(arrivalList)) {
//                throw new ServiceException(500, "到港不存在");
//            }
        }

        // 插入
        BoxAbnormalDO boxAbnormal = BoxAbnormalConvert.INSTANCE.convert(createReqVO);
        boxAbnormalMapper.insert(boxAbnormal);

        BoxDO boxDO = boxService.getBox(boxAbnormal.getShipmentId());
        String transportType = boxDO.getTransportType();
        String billId = createReqVO.getBillAbnId().toString();
        String type;
        String errorType = "shipping_ticket_exception";
        if (transportType.equals(TransportTypeEnum.SEA_AIR.getType())) {
            type = "shipping_process_sea_air";
        } else if (transportType.equals(TransportTypeEnum.AIR.getType())) {
            type = "air_shipment_process";
        } else {
            type = "shipping_process";
        }
        DictDataRespDTO opStepDict = DictFrameworkUtils.getDictDataFromCache(type, opStep.toString());
        if (transportType.equals(TransportTypeEnum.AIR.getType())) {
            // 空运的异常说明类型根据操作步骤判断
            errorType = opStep == 10 ? "flying_ticket_exception" : (opStep == 12 ? "arrival_ticket_exception" : (opStep == 13 ? "clearance_ticket_exception" : errorType));
        }
        DictDataRespDTO errorDict = DictFrameworkUtils.getDictDataFromCache(errorType, billId);

        String notifyUsers = createReqVO.getNotifyUsers();
        if (StringUtils.isNotEmpty(notifyUsers)) {
            Date startDate = createReqVO.getAbnStartTime();
            String start = "";
            if (startDate != null) {
                start = DateUtil.format(startDate, "yyyy-MM-dd");
            }
            Date endDate = createReqVO.getAbnEndTime();
            String end = "";
            if (endDate != null) {
                end = DateUtil.format(endDate, "yyyy-MM-dd");
            }


            String title = boxDO.getSelfNo() + "发生异常，请注意查看";

            String content = "自编号：" + boxDO.getSelfNo() + ",在" + opStepDict.getLabel() + "时发生异常，异常内容:" +
                    errorDict.getLabel() + ",异常情况：" + createReqVO.getAbnDetail() + "，异常时间" + start + "~" + end
                    + "，请注意查看";

            String[] notifyUserArray = notifyUsers.split(",");
            List<Long> idList = new ArrayList<>();
            for (String notifyUser : notifyUserArray) {
                Long userId = Long.valueOf(notifyUser);
                idList.add(userId);
            }
            sendMessage(title, content, idList);
        }
        Integer updateStatus = createReqVO.getUpdateStatus();
        boolean updateOrderStatus = Objects.nonNull(updateStatus) && updateStatus == 1;
        if (updateOrderStatus) {
            // 异常登记补充订单动态
            Set<Long> orderIdList = boxPreloadGoodsService.getShipOrderIdList(shipmentId);
            if (transportType.equals(TransportTypeEnum.AIR.getType())) {
                //订单不存在
                if (CollectionUtil.isNotEmpty(orderIdList)) {
                    OrderAirTimeEnum orderAirTimeEnum = null;
                    switch (opStep) {
                        case 8:
                            orderAirTimeEnum = OrderAirTimeEnum.AIR_CUSTOMS_IN_PROGRESS;
                            break;
                        case 10:
                            orderAirTimeEnum = OrderAirTimeEnum.AIR_HAS_TAKEN_OFF_DELAY;
                            break;
                        case 12:
                            orderAirTimeEnum = OrderAirTimeEnum.AIR_ARRIVED_DELAY;
                            break;
                        case 13:
                            orderAirTimeEnum = OrderAirTimeEnum.AIR_CLEARED_DELAY;
                            break;
                        case 14:
                            orderAirTimeEnum = OrderAirTimeEnum.AIR_UNLOADED_CABINET_DELAY;
                            break;
                        default:
                    }

                    if (Objects.nonNull(orderAirTimeEnum)) {
                        boxService.addOrderAirTimeLog(orderIdList, orderAirTimeEnum, errorDict.getLabel(), errorDict.getLabelEn(), null, null, null);
                    }
                } else if (transportType.equals(TransportTypeEnum.SEA_COMBINED_CABINET.getType())) {

                    //订单不存在
                    if (CollectionUtil.isNotEmpty(orderIdList)) {
                        OrderSeaTimeEnum orderSeaTimeEnum = null;
                        switch (opStep) {
                            case 8:
                                orderSeaTimeEnum = OrderSeaTimeEnum.SEA_CUSTOMS_IN_PROGRESS;
                                break;
                            case 10:
                                orderSeaTimeEnum = OrderSeaTimeEnum.SEA_HAS_TAKEN_OFF_DELAY;
                                break;
                            case 12:
                                orderSeaTimeEnum = OrderSeaTimeEnum.SEA_ARRIVED_DELAY;
                                break;
                            case 13:
                                orderSeaTimeEnum = OrderSeaTimeEnum.SEA_CLEARED_DELAY;
                                break;
                            case 14:
                                orderSeaTimeEnum = OrderSeaTimeEnum.SEA_UNLOADED_CABINET_DELAY;
                                break;
                            default:
                        }

                        if (Objects.nonNull(orderSeaTimeEnum)) {
                            boxService.addOrderSeaTimeLog(orderIdList, orderSeaTimeEnum, errorDict.getLabel(), errorDict.getLabelEn(), null, null, null);
                        }
                    } else {
                        // TODO 后续业务再补充
                    }
                }
            }
        }

        // 返回
        return boxAbnormal.getId();
    }

    private void sendMessage(String title, String content, List<Long> idList) {
        InternalMessageCreateDto dto = InternalMessageCreateDto
                .builder().toIdList(idList)
                .fromId("")
                .sendTime(new Date())
                .title(title)
                .content(content)
                .type(3)
                .build();
        internalMessageApi.createInternalMessage(dto);
    }

    @Override
    public void updateBoxAbnormal(BoxAbnormalUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAbnormalExists(updateReqVO.getId());
        // 更新
        BoxAbnormalDO updateObj = BoxAbnormalConvert.INSTANCE.convert(updateReqVO);
        boxAbnormalMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxAbnormal(Long id) {
        // 校验存在
        this.validateBoxAbnormalExists(id);
        // 删除
        boxAbnormalMapper.deleteById(id);
    }

    private void validateBoxAbnormalExists(Long id) {
        if (boxAbnormalMapper.selectById(id) == null) {
            throw exception(BOX_ABNORMAL_NOT_EXISTS);
        }
    }

    @Override
    public BoxAbnormalDO getBoxAbnormal(Long id) {
        return boxAbnormalMapper.selectById(id);
    }

    @Override
    public List<BoxAbnormalDO> getBoxAbnormalList(Collection<Long> ids) {
        return boxAbnormalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxAbnormalDO> getBoxAbnormalPage(BoxAbnormalQueryVO query, PageVO page) {
        return boxAbnormalMapper.selectPage(page, query);
    }

    @Override
    public List<BoxAbnormalDO> getBoxAbnormalList(BoxAbnormalQueryVO query) {
        return boxAbnormalMapper.selectList(query);
    }
}
