package cn.iocoder.yudao.module.shipment.service.boxAirCheckout;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirCheckoutDO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 空运出仓 Service 接口
 *
 * @author 江楠
 */
public interface BoxAirCheckoutService extends IService<BoxAirCheckoutDO> {

    /**
     * 创建空运出仓
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxAirCheckout(@Valid BoxAirCheckoutCreateReqVO createReqVO);

    /**
     * 更新空运出仓
     * @param updateReqVO 更新信息
     */
    void updateBoxAirCheckout(@Valid BoxAirCheckoutUpdateReqVO updateReqVO);

    /**
     * 删除空运出仓
     * @param id 编号
     */
    void deleteBoxAirCheckout(Long id);

    /**
     * 获得空运出仓
     * @param id 编号
     * @return 空运出仓
     */
    BoxAirCheckoutDO getBoxAirCheckout(Long id);

    /**
     * 获得空运出仓列表
     * @param ids 编号
     * @return 空运出仓列表
     */
    List<BoxAirCheckoutDO> getBoxAirCheckoutList(Collection<Long> ids);

    /**
     * 获得空运出仓分页
     * @param page 分页查询
     * @param query 查询
     * @return 空运出仓分页
     */
    PageResult<BoxAirCheckoutDO> getBoxAirCheckoutPage(BoxAirCheckoutQueryVO query, PageVO page);

    /**
     * 获得空运出仓列表, 用于 Excel 导出
     * @param query 查询
     * @return 空运出仓列表
     */
    List<BoxAirCheckoutDO> getBoxAirCheckoutList(BoxAirCheckoutQueryVO query);

    PageResult<BoxAirCheckoutPDAVO> getBoxAirCheckout4PDAPage(String selfNo, PageVO page);

    BoxAirCheckoutDO getAirCheckoutByShipmentId(Long shipmentId);

    Integer getPlanNum(Long shipmentId);

    Boolean confirmAirCheckout(Long shipmentId);

    Integer getRealNum(Long shipmentId);
}
