package cn.iocoder.yudao.module.shipment.service.boxAirCheckout;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.DeptWarehouseDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseBaseVO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.enums.TargetLogEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxAirCheckoutConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirCheckoutDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookAirDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxAirCheckoutMapper;
import cn.iocoder.yudao.module.shipment.enums.AirCheckoutStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxAirStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.OrderShipmentLog;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxBookAir.BoxBookAirService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutUpdateReqVO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.DEPT_NOT_FOUND;

/**
 * 空运出仓 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxAirCheckoutServiceImpl extends AbstractService<BoxAirCheckoutMapper, BoxAirCheckoutDO> implements BoxAirCheckoutService {

    @Resource
    private BoxAirCheckoutMapper boxAirCheckoutMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Resource
    private WarehouseService warehouseService;
    
    @Resource
    @Lazy
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    @Lazy
    private BoxBookAirService boxBookAirService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxAirCheckout(BoxAirCheckoutCreateReqVO createReqVO) {
        // 插入
        Long shipmentId = createReqVO.getShipmentId();
        checkBoxApproval(shipmentId);
        //出仓，校验已装箱数量是否符合
        if (createReqVO.getOperateType() != 1) {
            boxService.checkInstallNum(shipmentId);
        }
        BoxAirCheckoutDO dbCheckOut = getOne(new LambdaQueryWrapper<BoxAirCheckoutDO>()
            .eq(BoxAirCheckoutDO::getShipmentId, shipmentId)
        );

        BoxAirCheckoutDO boxAirCheckout = BoxAirCheckoutConvert.INSTANCE.convert(createReqVO);
        if(dbCheckOut != null) {
            boxAirCheckout.setId(dbCheckOut.getId());
            boxAirCheckoutMapper.updateById(boxAirCheckout);
        } else {
            boxAirCheckoutMapper.insert(boxAirCheckout);
        }

        updateBoxStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), createReqVO.getCheckoutTime());
        return boxAirCheckout.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxAirCheckout(BoxAirCheckoutUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAirCheckoutExists(updateReqVO.getId());
        // 更新
        //出仓，校验已装箱数量是否符合
        if (updateReqVO.getOperateType() != 1) {
            boxService.checkInstallNum(updateReqVO.getShipmentId());
        }
        BoxAirCheckoutDO updateObj = BoxAirCheckoutConvert.INSTANCE.convert(updateReqVO);
        boxAirCheckoutMapper.updateById(updateObj);
        updateBoxStatus(updateObj.getShipmentId(), updateReqVO.getOperateType(), updateObj.getCheckoutTime());
    }

    private void updateBoxStatus(Long shipmentId, Integer operateType, Date checkoutTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            int checkoutStatus = boxDO.getCheckoutStatus();
            AirCheckoutStatusEnum checkout = AirCheckoutStatusEnum.CHECKOUT;
            if(checkoutStatus != checkout.getStatus()) {

                BoxBookAirDO boxBookAirDO = boxBookAirService.getOne(new LambdaQueryWrapper<BoxBookAirDO>()
                    .eq(BoxBookAirDO::getShipmentId, shipmentId)
                        .last("limit 1")
                );

                //修改为已订舱,如果是已订舱的状态，则不用再修改
                boxDO.setCheckoutStatus(checkout.getStatus());

                //修改订单状态
                boxService.updateOrderStatusByShipmentIdAndTransportType(boxDO.getId(), OrderStatusEnum.SHIPMENT.getValue(),
                        null, BoxAirStatusEnum.WAREHOUSED.getStatus(), null, null, checkoutTime, null, Integer.parseInt(boxDO.getTransportType()));

                boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.AIR_OUT_WAREHOUSED, "");

                //完成装柜
//                boxService.airLoadInfo(boxDO.getId());

                boxDO.setCkDate(checkoutTime);

//                boxService.airLoadInfo(boxDO.getId());
                boxService.updateById(boxDO);
                // TODO 预计起飞时间参数待确认
                Date flyTime = boxBookAirDO.getFlyTime();
                String estFlyingTime = DateUtil.format(flyTime, "yyyy-MM-dd HH:mm:ss");
                // 站内信
                boxService.shipmentAirInternalMessage(shipmentId, boxDO.getSelfNo(), estFlyingTime, BoxAirStatusEnum.WAREHOUSED, Integer.parseInt(boxDO.getTransportType()));

                //出仓添加业绩日志
                boxService.addTargetLog(shipmentId, TargetLogEnum.LOADING,checkoutTime);
                // 出仓短信通知
                boxService.sendSms(shipmentId, SmsSceneEnum.AIR_SHIPMENT_WAREHOUSE);
                //插入操作日志
                boxOpLogService.createBoxOpLog(boxDO.getId(), checkout.getStatus(), checkout.getText(), checkout.getTextEn(), null);
            }

        }
    }



    @Override
    public void deleteBoxAirCheckout(Long id) {
        // 校验存在
        this.validateBoxAirCheckoutExists(id);
        // 删除
        boxAirCheckoutMapper.deleteById(id);
    }

    private void validateBoxAirCheckoutExists(Long id) {
        if (boxAirCheckoutMapper.selectById(id) == null) {
            throw exception(BOX_AIR_CHECKOUT_NOT_EXISTS);
        }
    }

    @Override
    public BoxAirCheckoutDO getBoxAirCheckout(Long id) {
        return boxAirCheckoutMapper.selectById(id);
    }

    @Override
    public List<BoxAirCheckoutDO> getBoxAirCheckoutList(Collection<Long> ids) {
        return boxAirCheckoutMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxAirCheckoutDO> getBoxAirCheckoutPage(BoxAirCheckoutQueryVO query, PageVO page) {
        return boxAirCheckoutMapper.selectPage(page, query);
    }

    @Override
    public List<BoxAirCheckoutDO> getBoxAirCheckoutList(BoxAirCheckoutQueryVO query) {
        return boxAirCheckoutMapper.selectList(query);
    }

    @Override
    public PageResult<BoxAirCheckoutPDAVO> getBoxAirCheckout4PDAPage(String selfNo, PageVO page) {
        Long deptId = SecurityFrameworkUtils.getLoginUser().getDeptId();
        DeptWarehouseBaseVO deptWarehouseBaseVO = new DeptWarehouseBaseVO();
        deptWarehouseBaseVO.setDeptId(deptId);
        List<DeptWarehouseDO> deptWarehouseDOS = warehouseService.deptList(deptWarehouseBaseVO);

        if (CollectionUtil.isEmpty(deptWarehouseDOS)) {
            throw exception(DEPT_NOT_FOUND);
        }

        IPage<BoxAirCheckoutPDAVO> mpPage = MyBatisUtils.buildPage(page);

        Set<Long> warehouseIds = deptWarehouseDOS.stream()
                .map(DeptWarehouseDO::getWarehouseId)
                .collect(Collectors.toSet());

        IPage<BoxAirCheckoutPDAVO> result = boxAirCheckoutMapper.getBoxAirCheckout4PDAPage(mpPage,warehouseIds,selfNo);
        result.getRecords().stream().forEach(item ->{
            BoxGoodsStatistics boxStatistics = boxPreloadGoodsService.getBoxStatistics(item.getId());
            BeanUtils.copyProperties(boxStatistics,item);
            item.setPlanNum(boxService.getPlanNum(item.getId()));
            item.setRealNum(boxService.getRealNum(item.getId()));
        });
        return PageResult.of(result);
    }

    @Override
    public BoxAirCheckoutDO getAirCheckoutByShipmentId(Long shipmentId) {
        return selectOne(new LambdaQueryWrapperX<BoxAirCheckoutDO>().eq(BoxAirCheckoutDO::getShipmentId,shipmentId)
                .orderByDesc(BoxAirCheckoutDO::getId)
                .last("limit 1")
        );
    }

    @Override
    public Integer getPlanNum(Long shipmentId) {
        return boxService.getPlanNum(shipmentId);
    }

    @Override
    public Boolean confirmAirCheckout(Long shipmentId) {
        this.checkInstallNum(shipmentId);
        return true;
    }

    @Override
    public Integer getRealNum(Long shipmentId) {
        return boxService.getRealNum(shipmentId);
    }

    public void checkInstallNum(Long shipmentId) {
        BoxGoodsStatistics boxStatistics = boxPreloadGoodsService.getBoxStatistics(shipmentId);
        long numSum = boxStatistics.getNum();
        long installNum = boxStatistics.getLoadNum();
        if (numSum != installNum) {
            throw exception(AIR_CHECKOUT_NOT_MATCH);
        }
    }

    private void checkBoxApproval(Long shipmentId){
        Long aLong = boxApprovalService.selectCount(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalStatus, 0));
        if (aLong != null && aLong > 0L) {
            throw exception(BOX_APPROVAL_EXIST);
        }
    }
}
