package cn.iocoder.yudao.module.shipment.service.boxAirFly;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.boxAirFly.BoxAirFlyConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxAirFly.BoxAirFlyDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.boxAirFly.BoxAirFlyMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxAirFly.*;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 起飞 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxAirFlyServiceImpl extends AbstractService<BoxAirFlyMapper, BoxAirFlyDO> implements BoxAirFlyService {

    @Resource
    private BoxAirFlyMapper boxAirFlyMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Override
    public Long createBoxAirFly(BoxAirFlyCreateReqVO createReqVO) {
        // 插入
        BoxAirFlyDO boxAirFly = BoxAirFlyConvert.INSTANCE.convert(createReqVO);
        boxAirFlyMapper.insert(boxAirFly);
        // 返回
        updateBoxDtStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), createReqVO.getDtRealFlyTime(), createReqVO.getEstTime());
        return boxAirFly.getId();
    }

    @Override
    public void updateBoxAirFly(BoxAirFlyUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAirFlyExists(updateReqVO.getId());
        // 更新
        BoxAirFlyDO updateObj = BoxAirFlyConvert.INSTANCE.convert(updateReqVO);
        boxAirFlyMapper.updateById(updateObj);
        updateBoxDtStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateObj.getDtRealFlyTime(), updateObj.getEstTime());
    }

    private void updateBoxDtStatus(Long shipmentId, Integer operateType, Date dtEstArrivalTime, Date estTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }


        if (operateType == OperateTypeEnum.COMMIT.getType()) {
            FlyStatueEnum shipping = FlyStatueEnum.FLYING;
            //已经起运，不执行下面的业务
            if (boxDO.getFlyStatus() == shipping.getDtStatus()) {
                boxService.updateOrderArriveTime(boxDO.getSelfNo(), Integer.parseInt(boxDO.getTransportType()), estTime);
                return;
            }

            boxDO.setFlyStatus(shipping.getDtStatus());
            boxDO.setQyDate(dtEstArrivalTime);

            //更新日志，修改为起飞中
            boxOpLogService.createBoxOpLog(shipmentId, shipping.getDtStatus(), shipping.getStatusNameZh(), shipping.getStatusNameEn(), null);

            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.IN_SHIPPING.getValue(), null,
                    BoxAirStatusEnum.FLYING.getStatus(), null, null, dtEstArrivalTime, estTime, Integer.parseInt(boxDO.getTransportType()));

            String date = DateUtils.formatDate(dtEstArrivalTime);
            // TODO 待添加订单动态
            boxService.shipmentAirInternalMessage(shipmentId, boxDO.getSelfNo(), date, BoxAirStatusEnum.FLYING, Integer.parseInt(boxDO.getTransportType()));
        }
        boxService.updateById(boxDO);
    }

    @Override
    public void deleteBoxAirFly(Long id) {
        // 校验存在
        this.validateBoxAirFlyExists(id);
        // 删除
        boxAirFlyMapper.deleteById(id);
    }

    private void validateBoxAirFlyExists(Long id) {
        if (boxAirFlyMapper.selectById(id) == null) {
            throw exception(BOX_AIR_FLY_NOT_EXISTS);
        }
    }

    @Override
    public BoxAirFlyDO getBoxAirFly(Long id) {
        return boxAirFlyMapper.selectById(id);
    }

    @Override
    public List<BoxAirFlyDO> getBoxAirFlyList(Collection<Long> ids) {
        return boxAirFlyMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxAirFlyDO> getBoxAirFlyPage(BoxAirFlyQueryVO query, PageVO page) {
        return boxAirFlyMapper.selectPage(page, query);
    }

    @Override
    public List<BoxAirFlyDO> getBoxAirFlyList(BoxAirFlyQueryVO query) {
        return boxAirFlyMapper.selectList(query);
    }
}
