package cn.iocoder.yudao.module.shipment.service.boxAirShipment;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirShipmentDO;
import cn.iocoder.yudao.module.shipment.vo.boxAirShipment.*;

/**
 * 空运出货 Service 接口
 *
 * @author 江楠
 */
public interface BoxAirShipmentService extends IService<BoxAirShipmentDO> {

    /**
     * 创建空运出货
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxAirShipment(@Valid BoxAirShipmentCreateReqVO createReqVO);

    /**
     * 更新空运出货
     * @param updateReqVO 更新信息
     */
    void updateBoxAirShipment(@Valid BoxAirShipmentUpdateReqVO updateReqVO);

    /**
     * 删除空运出货
     * @param id 编号
     */
    void deleteBoxAirShipment(Long id);

    /**
     * 获得空运出货
     * @param id 编号
     * @return 空运出货
     */
    BoxAirShipmentDO getBoxAirShipment(Long id);

    /**
     * 获得空运出货列表
     * @param ids 编号
     * @return 空运出货列表
     */
    List<BoxAirShipmentDO> getBoxAirShipmentList(Collection<Long> ids);

    /**
     * 获得空运出货分页
     * @param page 分页查询
     * @param query 查询
     * @return 空运出货分页
     */
    PageResult<BoxAirShipmentDO> getBoxAirShipmentPage(BoxAirShipmentQueryVO query, PageVO page);

    /**
     * 获得空运出货列表, 用于 Excel 导出
     * @param query 查询
     * @return 空运出货列表
     */
    List<BoxAirShipmentDO> getBoxAirShipmentList(BoxAirShipmentQueryVO query);

    BoxAirShipmentDO getAirShipmentByShipmentId(Long shipmentId);
}
