package cn.iocoder.yudao.module.shipment.service.boxAirShipment;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxAirShipmentConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAirShipmentDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxAirShipmentMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxAirShipment.*;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalCreateReqVO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 空运出货 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxAirShipmentServiceImpl extends AbstractService<BoxAirShipmentMapper, BoxAirShipmentDO> implements BoxAirShipmentService {

    @Resource
    private BoxAirShipmentMapper boxAirShipmentMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Resource
    private BoxApprovalService approvalService;
    @Override
    public Long createBoxAirShipment(BoxAirShipmentCreateReqVO createReqVO) {
        List<BoxAirShipmentDO> boxAirShipmentDOS = boxAirShipmentMapper.selectList(BoxAirShipmentDO::getShipmentId, createReqVO.getShipmentId());
        if (CollectionUtil.isNotEmpty(boxAirShipmentDOS)) {
            boxAirShipmentMapper.deleteBatchIds(boxAirShipmentDOS.stream().map(BoxAirShipmentDO::getId).collect(Collectors.toList()));
        }
        // 插入
        BoxAirShipmentDO boxAirShipment = BoxAirShipmentConvert.INSTANCE.convert(createReqVO);
        boxAirShipmentMapper.insert(boxAirShipment);
        // 返回
        commitUpdate(createReqVO.getOperateType(),createReqVO.getShipmentId(), createReqVO.getCopyUserId());
        return boxAirShipment.getId();
    }

    @Override
    public void updateBoxAirShipment(BoxAirShipmentUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxAirShipmentExists(updateReqVO.getId());
        // 更新
        BoxAirShipmentDO updateObj = BoxAirShipmentConvert.INSTANCE.convert(updateReqVO);
        commitUpdate(updateReqVO.getOperateType(),updateReqVO.getShipmentId(), updateReqVO.getCopyUserId());
        boxAirShipmentMapper.updateById(updateObj);
    }

    private void commitUpdate(Integer operateType, Long shipmentId, String[] copyUserId) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            int airShipmentStatus = boxDO.getAirShipmentStatus();
            AirShipmentStatusEnum shipment = AirShipmentStatusEnum.SHIPMENT;

            if(airShipmentStatus != shipment.getStatus()) {
                createApproval(shipmentId, copyUserId);
            }
        }
        boxService.updateById(boxDO);
    }

    private void createApproval(Long shipmentId, String[] copyUserId) {
        BoxApprovalCreateReqVO boxApprovalCreateReqVO = new BoxApprovalCreateReqVO();
        boxApprovalCreateReqVO.setShipmentId(shipmentId);
        boxApprovalCreateReqVO.setApplyReason("出货审核");
        boxApprovalCreateReqVO.setCopyUserId(copyUserId);
        boxApprovalCreateReqVO.setApprovalStatus(BpmProcessInstanceResultEnum.PROCESS.getResult());
        boxApprovalCreateReqVO.setApprovalType(BoxApprovalTypeEnum.AIR_SHIPMENT.getType());
        approvalService.createBoxApproval(boxApprovalCreateReqVO);
    }

    @Override
    public void deleteBoxAirShipment(Long id) {
        // 校验存在
        this.validateBoxAirShipmentExists(id);
        // 删除
        boxAirShipmentMapper.deleteById(id);
    }

    private void validateBoxAirShipmentExists(Long id) {
        if (boxAirShipmentMapper.selectById(id) == null) {
            throw exception(BOX_AIR_SHIPMENT_NOT_EXISTS);
        }
    }

    @Override
    public BoxAirShipmentDO getBoxAirShipment(Long id) {
        return boxAirShipmentMapper.selectById(id);
    }

    @Override
    public List<BoxAirShipmentDO> getBoxAirShipmentList(Collection<Long> ids) {
        return boxAirShipmentMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxAirShipmentDO> getBoxAirShipmentPage(BoxAirShipmentQueryVO query, PageVO page) {
        return boxAirShipmentMapper.selectPage(page, query);
    }

    @Override
    public List<BoxAirShipmentDO> getBoxAirShipmentList(BoxAirShipmentQueryVO query) {
        return boxAirShipmentMapper.selectList(query);
    }

    @Override
    public BoxAirShipmentDO getAirShipmentByShipmentId(Long shipmentId) {
        return boxAirShipmentMapper.selectOne(BoxAirShipmentDO::getShipmentId,shipmentId);
    }

    private void checkBoxApproval(Long shipmentId){
        Long aLong = approvalService.selectCount(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getShipmentId, shipmentId).eq(BoxApprovalDO::getApprovalStatus, 0));
        if (aLong != null && aLong > 0L) {
            throw exception(BOX_APPROVAL_EXIST);
        }
    }
}
