package cn.iocoder.yudao.module.shipment.service.boxApproval;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxLoadDetailVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalUpdateReqVO;

/**
 * 出货审核 Service 接口
 *
 * @author wlh
 */
public interface BoxApprovalService extends IService<BoxApprovalDO> {

    /**
     * 创建出货审核
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxApproval(@Valid BoxApprovalCreateReqVO createReqVO);

    /**
     * 更新出货审核
     * @param updateReqVO 更新信息
     */
    void updateBoxApproval(@Valid BoxApprovalUpdateReqVO updateReqVO);

    /**
     * 删除出货审核
     * @param id 编号
     */
    void deleteBoxApproval(Long id);

    /**
     * 获得出货审核
     * @param id 编号
     * @return 出货审核
     */
    BoxApprovalDO getBoxApproval(Long id);

    /**
     * 获得出货审核列表
     * @param ids 编号
     * @return 出货审核列表
     */
    List<BoxApprovalDO> getBoxApprovalList(Collection<Long> ids);

    /**
     * 获得出货审核分页
     * @param page 分页查询
     * @param query 查询
     * @return 出货审核分页
     */
    PageResult<BoxApprovalDO> getBoxApprovalPage(BoxApprovalQueryVO query, PageVO page);

    /**
     * 获得出货审核列表, 用于 Excel 导出
     * @param query 查询
     * @return 出货审核列表
     */
    List<BoxApprovalDO> getBoxApprovalList(BoxApprovalQueryVO query);

    /**
     * 审核通知回调方法
     * @param flowKey flowKey
     * @param id id
     * @param result result
     */
    void updateBoxApproveResult(String flowKey, String id,int result);

    /**
     * 审核页面查看详情
     * @param approvalId approvalId
     */
    BoxLoadDetailVO approvalDetail(Long approvalId);

    /**
     * 取消审核
     * @param updateReqV updateReqV
     */
    void cancelApproval(BoxApprovalUpdateReqVO updateReqV);

    /**
     * 校验出货单是否在审核中
     * @param shipmentId  出货单ID
     * @param approvalType  审核类型
     */
    void checkApproval(Long shipmentId, Integer approvalType);

    /**
     * 查询出货正在反审的文案
     * @param shipmentId  出货单ID
     * @return
     */
    Integer getBackApprovalType(Long shipmentId);

    List<BoxApprovalDO> getBackApprovalList(List<Long> idList);
}
