package cn.iocoder.yudao.module.shipment.service.boxArrival;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxArrivalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxArrivalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxArrivalMapper;
import cn.iocoder.yudao.module.shipment.enums.ApStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalUpdateReqVO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 到港 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class BoxArrivalServiceImpl extends AbstractService<BoxArrivalMapper, BoxArrivalDO> implements BoxArrivalService {

    @Resource
    private BoxArrivalMapper boxArrivalMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    BoxOpLogService boxOpLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxArrival(BoxArrivalCreateReqVO createReqVO) {
        // 插入
        BoxArrivalDO boxArrival = BoxArrivalConvert.INSTANCE.convert(createReqVO);
        boxArrivalMapper.insert(boxArrival);
        // 返回
        updateBoxApStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(),
                createReqVO.getApUnloadPortTime(), createReqVO.getApConfirmTime(), createReqVO.getApRealTime());
        return boxArrival.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxArrival(BoxArrivalUpdateReqVO updateReqVO) {
        // 更新
        BoxArrivalDO updateObj = BoxArrivalConvert.INSTANCE.convert(updateReqVO);
        boxArrivalMapper.updateById(updateObj);
        updateBoxApStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(),
                updateReqVO.getApUnloadPortTime(), updateReqVO.getApConfirmTime(), updateObj.getApRealTime());
    }

    private void updateBoxApStatus(Long shipmentId, Integer operateType, Date apUnloadPortTime, Date apConfirmTime, Date apRelTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if (operateType == OperateTypeEnum.COMMIT.getType()) {
            int apStatus = boxDO.getApStatus();

            boxDO.setApStatus(ApStatueEnum.HAS_ARRIVE.getApStatus());
            boxDO.setDgDate(apRelTime);

            //新增操作日志
            String opText;

            Integer shipmentState;

            if (apUnloadPortTime == null) {
                opText = "已到港，待卸港";
                shipmentState = BoxStatusEnum.ARRIVAL_WAIT_DISCHARGE.getStatus();
            } else {
                opText = "已卸港，待清关";
                shipmentState = BoxStatusEnum.DISCHARGE_WAIT_CLEARANCE.getStatus();
            }
            boxDO.setShipmentStatus(shipmentState);

            //已到港，下面的逻辑已经执行过
            if (apStatus != ApStatueEnum.HAS_ARRIVE.getApStatus()) {
                boxOpLogService.createBoxOpLog(shipmentId, ApStatueEnum.HAS_ARRIVE.getApStatus(), opText, opText, null);

                //确认到港
                //发送短信通知
                if (StringUtils.equals("3", boxDO.getTransportType())) {
                    boxService.sendSms(shipmentId, SmsSceneEnum.AIR_SHIPMENT_ARRIVAL);
                } else {
                    boxService.sendSms(shipmentId, SmsSceneEnum.SHIPMENT_ARRIVAL);
                }


                //修改订单状态为已到港
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.ARRIVED.getValue(), null,
                        shipmentState, null, null, apRelTime, null, Integer.parseInt(boxDO.getTransportType()));

                //发送站内信
                boxService.shipmentInternalMessage(shipmentId, null, null, BoxStatusEnum.ARRIVAL_WAIT_DISCHARGE, Integer.parseInt(boxDO.getTransportType()));
            }
        }
        boxService.updateById(boxDO);
    }


    @Override
    public BoxArrivalDO getBoxArrival(Long id) {
        return boxArrivalMapper.selectById(id);
    }

    @Override
    public List<BoxArrivalDO> getBoxArrivalList(Collection<Long> ids) {
        return boxArrivalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxArrivalDO> getBoxArrivalPage(BoxArrivalQueryVO query, PageVO page) {
        return boxArrivalMapper.selectPage(page, query);
    }

    @Override
    public List<BoxArrivalDO> getBoxArrivalList(BoxArrivalQueryVO query) {
        return boxArrivalMapper.selectList(query);
    }
}
