package cn.iocoder.yudao.module.shipment.service.boxArrivalAir;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.shipment.convert.BoxArrivalAirConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxArrivalAirDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxArrivalOrder.BoxArrivalOrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxClearanceOrder.BoxClearanceOrderDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxArrivalAirMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxArrivalOrder.BoxArrivalOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirBaseVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalOrder.BoxArrivalOrderAllCreateVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalOrder.BoxArrivalOrderCreateVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 空运到港 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxArrivalAirServiceImpl extends AbstractService<BoxArrivalAirMapper, BoxArrivalAirDO> implements BoxArrivalAirService {

    @Resource
    private BoxArrivalAirMapper boxArrivalAirMapper;
    @Resource
    @Lazy
    private BoxService boxService;
    @Resource
    @Lazy
    private BoxPreloadGoodsService preloadGoodsService;
    @Resource
    @Lazy
    private BoxArrivalOrderService arrivalOrderService;
    @Resource
    @Lazy
    private OrderQueryService orderQueryService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxArrivalAir(BoxArrivalAirCreateReqVO createReqVO) {
        BoxArrivalAirDO dbDO = getByShipmentId(createReqVO.getShipmentId());
        if(dbDO != null) {
            BoxArrivalAirUpdateReqVO updateReqVO = new BoxArrivalAirUpdateReqVO();
            BeanUtils.copyProperties(createReqVO, updateReqVO);
            updateReqVO.setId(dbDO.getId());
            updateBoxArrivalAir(updateReqVO);
            return dbDO.getId();
        }

        // 插入
        BoxArrivalAirDO boxArrivalAir = BoxArrivalAirConvert.INSTANCE.convert(createReqVO);
        clearTime(boxArrivalAir);
        boxArrivalAirMapper.insert(boxArrivalAir);
        // 返回
        updateBoxSapStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(),
                createReqVO, createReqVO.getArriveOrderIdList());
        return boxArrivalAir.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxArrivalAir(BoxArrivalAirUpdateReqVO updateReqVO) {
        // 更新
        BoxArrivalAirDO updateObj = BoxArrivalAirConvert.INSTANCE.convert(updateReqVO);
        clearTime(updateObj);
        boxArrivalAirMapper.updateById(updateObj);

        updateBoxSapStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(),
                updateReqVO, updateReqVO.getArriveOrderIdList());
    }

    private void clearTime(BoxArrivalAirDO boxArrivalAir) {
        if(boxArrivalAir.getArriveType() == 1) {
            boxArrivalAir.setActTime(null);
            boxArrivalAir.setEstTime(null);
            boxArrivalAir.setActSecondTime(null);
        }
    }

    private void updateBoxSapStatus(Long shipmentId, Integer operateType, BoxArrivalAirBaseVO arrivalAirBaseVO, List<Long> arriveOrderIdList) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            //已到港
            if (boxDO.getSapStatus() == SapStatueEnum.HAS_ARRIVE.getSapStatus()) return;

            if (boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
                Collection<Long> arrivalOrders = null;
                //一次到港
                if (arrivalAirBaseVO.getArriveType() == 0) {
                    arrivalOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
                } else if (arrivalAirBaseVO.getArriveType() == 1) {
                    //多次到港
                    if (CollectionUtils.isEmpty(arriveOrderIdList)) {
                        throw new ServiceException(500, "未选择订单");
                    } else {
                        arrivalOrders = arriveOrderIdList;
                    }
                }
                updateOrderArrival(shipmentId, arrivalOrders, arrivalAirBaseVO.getActTime(),
                        arrivalAirBaseVO.getEstTime(), arrivalAirBaseVO.getActSecondTime());
                boxDO.setDgDate(arrivalAirBaseVO.getActTime());
                updateOrderStatus(boxDO, arrivalOrders);
                updateBoxStatus(boxDO, arrivalAirBaseVO.getActTime());
            } else {
                SapStatueEnum sapStatueEnum = SapStatueEnum.HAS_ARRIVE;
                boxDO.setSapStatus(sapStatueEnum.getSapStatus());
                boxDO.setDgDate(arrivalAirBaseVO.getActTime());
                boxService.updateOrderStatusByShipmentIdAndTransportType(boxDO.getId(), OrderStatusEnum.TWP_WAY_ARRIVAL.getValue(),
                        0, BoxStatusEnum.TWO_WAY_ARRIVAL.getStatus(),
                        null, null, boxDO.getDgDate(), null, Integer.parseInt(boxDO.getTransportType()));
                boxService.updateById(boxDO);
            }
        }
    }

    private void updateBoxStatus(BoxDO boxDO, Date actTime) {
        Long shipmentId = boxDO.getId();
        Collection<Long> arrivalOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
        long arrivalCount = arrivalOrderService.count(new LambdaQueryWrapper<BoxArrivalOrderDO>()
                .eq(BoxArrivalOrderDO::getShipmentId, shipmentId)
        );
        if (arrivalCount == arrivalOrders.size()) {
            SapStatueEnum sapStatueEnum = SapStatueEnum.HAS_ARRIVE;
            boxDO.setSapStatus(sapStatueEnum.getSapStatus());
            boxDO.setDgDate(actTime);
            boxService.updateById(boxDO);
        }
    }

    @Override
    public BoxArrivalAirDO getBoxArrivalAir(Long id) {
        return boxArrivalAirMapper.selectById(id);
    }

    @Override
    public List<BoxArrivalAirDO> getBoxArrivalAirList(Collection<Long> ids) {
        return boxArrivalAirMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxArrivalAirDO> getBoxArrivalAirPage(BoxArrivalAirQueryVO query, PageVO page) {
        return boxArrivalAirMapper.selectPage(page, query);
    }

    @Override
    public List<BoxArrivalAirDO> getBoxArrivalAirList(BoxArrivalAirQueryVO query) {
        return boxArrivalAirMapper.selectList(query);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateOrderArrival(BoxArrivalOrderCreateVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();
        Date actTime = createReqVO.getActTime();
        List<Long> orderIdList = createReqVO.getOrderIdList();
        updateOrderArrival(shipmentId, createReqVO.getOrderIdList(), actTime,
                createReqVO.getEstTime(), createReqVO.getActSecondTime());
        BoxDO boxDO = boxService.getBox(shipmentId);
        updateOrderStatus(boxDO, orderIdList);
        updateBoxStatus(boxDO, actTime);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateOrderAllArrival(BoxArrivalOrderAllCreateVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();
        Date actTime = createReqVO.getActTime();
        Collection<Long> arrivalOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
        updateOrderArrival(shipmentId, arrivalOrders, actTime,
                createReqVO.getEstTime(), createReqVO.getActSecondTime());
        BoxDO boxDO = boxService.getBox(shipmentId);
        updateOrderStatus(boxDO, arrivalOrders);
        updateBoxStatus(boxDO, actTime);
    }

    private void updateOrderStatus(BoxDO boxDO, Collection<Long> arrivalOrders) {
        if(CollectionUtils.isEmpty(arrivalOrders)) return;
        boxService.updateOrderStatusByOrderList(arrivalOrders, OrderStatusEnum.TWP_WAY_ARRIVAL.getValue(),
                0, BoxAirStatusEnum.ARRIVAL_WAIT_DISCHARGE.getStatus(),
                null, null, null, boxDO.getDgDate());
        // 空运站内信
        boxService.shipmentAirOrderInternalMessage(arrivalOrders, boxDO.getSelfNo(), null, BoxAirStatusEnum.ARRIVAL_WAIT_DISCHARGE, Integer.parseInt(boxDO.getTransportType()));
    }

    private void updateOrderArrival(Long shipmentId, Collection<Long> arrivalOrders, Date actTime, Date estTime, Date actSecondTime) {
        List<BoxArrivalOrderDO> arrivalOrderDOList = arrivalOrderService.list(new LambdaQueryWrapper<BoxArrivalOrderDO>()
                .eq(BoxArrivalOrderDO::getShipmentId, shipmentId)
        );
        Map<Long, Long> orderMap = arrivalOrderDOList.stream()
                .collect(Collectors.toMap(BoxArrivalOrderDO::getOrderId, BoxArrivalOrderDO::getId));

        if (CollectionUtils.isNotEmpty(arrivalOrders)) {
            List<OrderDO> orderDOList = orderQueryService.getOrderList(arrivalOrders);
            List<BoxArrivalOrderDO> saveOrUpdateList = new ArrayList<>();
            for (OrderDO orderDO : orderDOList) {
                Long orderId = orderDO.getOrderId();
                BoxArrivalOrderDO boxArrivalOrderDO = new BoxArrivalOrderDO();
                boxArrivalOrderDO.setShipmentId(shipmentId);
                boxArrivalOrderDO.setOrderId(orderId);
                if (orderMap.containsKey(orderId)) {
                    boxArrivalOrderDO.setId(orderMap.get(orderId));
                    orderMap.remove(orderId);
                }
                boxArrivalOrderDO.setOrderNo(orderDO.getOrderNo());
                boxArrivalOrderDO.setActTime(actTime);
                boxArrivalOrderDO.setEstTime(estTime);
                boxArrivalOrderDO.setActSecondTime(actSecondTime);
                saveOrUpdateList.add(boxArrivalOrderDO);
            }
            arrivalOrderService.saveOrUpdateBatch(saveOrUpdateList);
        }
    }

    public BoxArrivalAirDO getByShipmentId(Long shipmentId) {
        return getOne(new LambdaQueryWrapperX<BoxArrivalAirDO>()
                .eq(BoxArrivalAirDO::getShipmentId, shipmentId)
                .orderByDesc(BoxArrivalAirDO::getId)
                .last(" limit 1")
        );
    }
}
