package cn.iocoder.yudao.module.shipment.service.boxBarge;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.shipment.convert.BoxBargeConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBargeDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBargeMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.BgStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.BlStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 驳船 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxBargeServiceImpl extends AbstractService<BoxBargeMapper, BoxBargeDO> implements BoxBargeService {

    @Resource
    private BoxBargeMapper boxBargeMapper;
    @Resource
    private BoxMapper boxMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxBarge(BoxBargeCreateReqVO createReqVO) {
        // 插入
        BoxBargeDO boxBarge = BoxBargeConvert.INSTANCE.convert(createReqVO);
        boxBargeMapper.insert(boxBarge);
        // 返回
        updateBoxBgStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), createReqVO.getNotifyUser());
        return boxBarge.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxBarge(BoxBargeUpdateReqVO updateReqVO) {
        // 更新
        BoxBargeDO updateObj = BoxBargeConvert.INSTANCE.convert(updateReqVO);
        boxBargeMapper.updateById(updateObj);
        updateBoxBgStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateReqVO.getNotifyUser());
    }

    private void updateBoxBgStatus(Long shipmentId, Integer operateType, Long notifyId) {
        BoxDO boxDO = boxMapper.selectById(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            boxDO.setBgStatus(BgStatueEnum.HAS_BARGE.getBgStatus());
        }
        boxMapper.updateById(boxDO);

        //TODO  通知用户
    }


    @Override
    public BoxBargeDO getBoxBarge(Long id) {
        return boxBargeMapper.selectById(id);
    }

    @Override
    public List<BoxBargeDO> getBoxBargeList(Collection<Long> ids) {
        return boxBargeMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxBargeDO> getBoxBargePage(BoxBargeQueryVO query, PageVO page) {
        return boxBargeMapper.selectPage(page, query);
    }

    @Override
    public List<BoxBargeDO> getBoxBargeList(BoxBargeQueryVO query) {
        return boxBargeMapper.selectList(query);
    }
}
