package cn.iocoder.yudao.module.shipment.service.boxBookAir;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.BookStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.PrStatusEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.shipment.vo.boxBookAir.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookAirDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.convert.BoxBookAirConvert;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBookAirMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 订舱-空运 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class BoxBookAirServiceImpl extends AbstractService<BoxBookAirMapper, BoxBookAirDO> implements BoxBookAirService {

    @Resource
    private BoxBookAirMapper boxBookAirMapper;

    @Lazy
    @Resource
    private BoxService boxService;

    @Resource
    BoxOpLogService boxOpLogService;

    @Override
    public Long createBoxBookAir(BoxBookAirCreateReqVO createReqVO) {
        // 插入
        BoxBookAirDO boxBookAir = BoxBookAirConvert.INSTANCE.convert(createReqVO);
        boxBookAirMapper.insert(boxBookAir);

//        if(createReqVO.getFlyTime() != null) {
//            boxService.updateOrderSailTime(createReqVO.getShipmentId(), createReqVO.getFlyTime());
//        }
        commitUpdate(createReqVO.getOperateType(), createReqVO.getShipmentId());
        // 返回
        return boxBookAir.getId();
    }

    @Override
    public void updateBoxBookAir(BoxBookAirUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxBookAirExists(updateReqVO.getId());

//        if(updateReqVO.getFlyTime() != null) {
//            boxService.updateOrderSailTime(updateReqVO.getShipmentId(), updateReqVO.getFlyTime());
//        }

        // 更新
        BoxBookAirDO updateObj = BoxBookAirConvert.INSTANCE.convert(updateReqVO);
        boxBookAirMapper.updateById(updateObj);
        //提交
        commitUpdate(updateReqVO.getOperateType(), updateReqVO.getShipmentId());
    }

    private void commitUpdate(Integer operateType, Long shipmentId) {
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            //修改出货表的状态
            BoxDO boxDO = boxService.getBox(shipmentId);
            if(boxDO == null) {
                throw exception(BOX_NOT_EXISTS);
            }

            int bkStatus = boxDO.getBkStatus();
            BookStatusEnum booked = BookStatusEnum.BOOKED;
            if(bkStatus != booked.getBkStatus()) {
                //修改为已订舱,如果是已订舱的状态，则不用再修改
                boxDO.setBkStatus(booked.getBkStatus());
            }
            boxService.updateById(boxDO);

            //插入操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), booked.getBkStatus(), booked.getStatusNameZh(), booked.getStatusNameEn(), null);
        }
    }

    @Override
    public void deleteBoxBookAir(Long id) {
        // 校验存在
        this.validateBoxBookAirExists(id);
        // 删除
        boxBookAirMapper.deleteById(id);
    }

    private void validateBoxBookAirExists(Long id) {
        if (boxBookAirMapper.selectById(id) == null) {
            throw exception(BOX_BOOK_AIR_NOT_EXISTS);
        }
    }

    @Override
    public BoxBookAirDO getBoxBookAir(Long id) {
        return boxBookAirMapper.selectById(id);
    }

    @Override
    public List<BoxBookAirDO> getBoxBookAirList(Collection<Long> ids) {
        return boxBookAirMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxBookAirDO> getBoxBookAirPage(BoxBookAirQueryVO query, PageVO page) {
        return boxBookAirMapper.selectPage(page, query);
    }

    @Override
    public List<BoxBookAirDO> getBoxBookAirList(BoxBookAirQueryVO query) {
        return boxBookAirMapper.selectList(query);
    }

    @Override
    public List<BoxBookAirDO> getNoticeBookList() {
        return boxBookAirMapper.getNoticeBookList();
    }
}
