package cn.iocoder.yudao.module.shipment.service.boxBookSea;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.shipment.convert.BoxBookSeaConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBookSeaMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.BookStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaUpdateReqVO;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_BOOK_SEA_NOT_EXISTS;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 订舱 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxBookSeaServiceImpl extends AbstractService<BoxBookSeaMapper, BoxBookSeaDO> implements BoxBookSeaService {

    @Resource
    private BoxBookSeaMapper boxBookSeaMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    BoxOpLogService boxOpLogService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long createBoxBookSea(BoxBookSeaCreateReqVO createReqVO) {
        // 插入
        BoxBookSeaDO boxBookSea = BoxBookSeaConvert.INSTANCE.convert(createReqVO);
        boxBookSeaMapper.insert(boxBookSea);

        if(createReqVO.getSailTime() != null) {
            boxService.updateOrderSailTime(createReqVO.getShipmentId(), 1, createReqVO.getSailTime());
        }

        //提交
        commitUpdate(createReqVO.getOperateType(), createReqVO.getShipmentId());
        return boxBookSea.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxBookSea(BoxBookSeaUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxBookSeaExists(updateReqVO.getId());

        if(updateReqVO.getSailTime() != null) {
            boxService.updateOrderSailTime(updateReqVO.getShipmentId(),1, updateReqVO.getSailTime());
        }

        // 更新
        BoxBookSeaDO updateObj = BoxBookSeaConvert.INSTANCE.convert(updateReqVO);
        boxBookSeaMapper.updateById(updateObj);
        //提交
        commitUpdate(updateReqVO.getOperateType(), updateReqVO.getShipmentId());
    }

    private void commitUpdate(Integer operateType, Long shipmentId) {
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            //修改出货表的状态
            BoxDO boxDO = boxService.getBox(shipmentId);
            if(boxDO == null) {
                throw exception(BOX_NOT_EXISTS);
            }

            int bkStatus = boxDO.getBkStatus();
            BookStatusEnum booked = BookStatusEnum.BOOKED;
            if(bkStatus != booked.getBkStatus()) {
                //修改为已订舱,如果是已订舱的状态，则不用再修改
                boxDO.setBkStatus(booked.getBkStatus());
            }
            boxService.updateById(boxDO);

            //插入操作日志
            boxOpLogService.createBoxOpLog(boxDO.getId(), booked.getBkStatus(), booked.getStatusNameZh(), booked.getStatusNameEn(), null);
        }
    }

    @Override
    public void deleteBoxBookSea(Long id) {
        // 校验存在
        this.validateBoxBookSeaExists(id);
        // 删除
        boxBookSeaMapper.deleteById(id);
    }

    private void validateBoxBookSeaExists(Long id) {
        if (boxBookSeaMapper.selectById(id) == null) {
            throw exception(BOX_BOOK_SEA_NOT_EXISTS);
        }
    }

    @Override
    public BoxBookSeaDO getBoxBookSea(Long id) {
        return boxBookSeaMapper.selectById(id);
    }

    @Override
    public List<BoxBookSeaDO> getBoxBookSeaList(Collection<Long> ids) {
        return boxBookSeaMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxBookSeaDO> getBoxBookSeaPage(BoxBookSeaQueryVO query, PageVO page) {
        return boxBookSeaMapper.selectPage(page, query);
    }

    @Override
    public List<BoxBookSeaDO> getBoxBookSeaList(BoxBookSeaQueryVO query) {
        return boxBookSeaMapper.selectList(query);
    }

    @Override
    public List<BoxBookSeaDO> getNoticeBookList() {
        return boxBookSeaMapper.getNoticeBookList();
    }
}
