package cn.iocoder.yudao.module.shipment.service.boxCabinet;

import java.util.*;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.order.enums.TargetLogEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadGoodsDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBookSeaMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCabinetMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsQueryVO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 装柜 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxCabinetServiceImpl extends AbstractService<BoxCabinetMapper, BoxCabinetDO> implements BoxCabinetService {

    @Resource
    private BoxCabinetMapper boxCabinetMapper;
    @Resource
    private BoxBookSeaMapper boxBookSeaMapper;

    @Resource
    @Lazy
    private BoxService boxService;

    @Resource
    BoxOpLogService boxOpLogService;

    public static void main(String[] args) {
        String ldPictures = "[{\"type\":\"image\",\"url\":\"\"}]";
        JSONArray jsonArray = JSON.parseArray(ldPictures);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            System.out.println(jsonObject.getString("type"));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxCabinet(BoxCabinetCreateReqVO createReqVO) {
        BoxCabinetDO dbCabinet = getByShipmentId(createReqVO.getShipmentId());

        if(dbCabinet != null) {
            BoxCabinetUpdateReqVO updateReqVO = new BoxCabinetUpdateReqVO();
            BeanUtils.copyProperties(createReqVO, updateReqVO);
            updateReqVO.setId(dbCabinet.getId());
            updateBoxCabinet(updateReqVO);
            return dbCabinet.getId();
        }

        // 插入
        BoxCabinetDO boxCabinet = BoxCabinetConvert.INSTANCE.convert(createReqVO);

        checkResourceUrl(boxCabinet.getLdPictures());

        boxCabinetMapper.insert(boxCabinet);
        // 返回
        updateBoxLdStatus(createReqVO.getShipmentId(), createReqVO.getOperateType());
        return boxCabinet.getId();
    }

    private void checkResourceUrl(String recs) {
        if(StringUtils.isEmpty(recs)) return;
        JSONArray jsonArray = JSON.parseArray(recs);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String url = jsonObject.getString("url");
            if(StringUtils.isEmpty(url)) {
                throw new ServiceException(500, "地址不能为空");
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxCabinet(BoxCabinetUpdateReqVO updateReqVO) {
        // 更新
        BoxCabinetDO updateObj = BoxCabinetConvert.INSTANCE.convert(updateReqVO);

        checkResourceUrl(updateObj.getLdPictures());

        boxCabinetMapper.updateById(updateObj);
        updateBoxLdStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType());
    }

    private void updateBoxLdStatus(Long shipmentId, Integer operateType) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            //封柜，
            BoxCabinetDO cabinetDO = getOne(new LambdaQueryWrapper<BoxCabinetDO>()
                .eq(BoxCabinetDO::getShipmentId, shipmentId)
                    .last(" limit 1")
            );
            Date ldBoxTime = new Date();
            if(cabinetDO != null && cabinetDO.getLdBoxTime() != null) {
                ldBoxTime = cabinetDO.getLdBoxTime();
            }

            int currentStatus = boxDO.getLdStatus();
            if(currentStatus != LdStatusEnum.CABINET_EXAMINE_SUCCESS.getLdStatus()) {
                throw exception(ErrorCodeConstants.BOX_CABINET_TIP);
            } else {
                //如果已成功，修改状态
                LdStatusEnum ldStatusEnum = LdStatusEnum.CABINET_SEALED;
                boxDO.setLdStatus(ldStatusEnum.getLdStatus());
                boxDO.setZgDate(ldBoxTime);
                boxService.updateById(boxDO);

                //如果已经是封柜状态，则不执行下面的业务
                if(currentStatus == LdStatusEnum.CABINET_SEALED.getLdStatus()) return;

                //新增日志
                boxOpLogService.createBoxOpLog(shipmentId, ldStatusEnum.getLdStatus(),
                        ldStatusEnum.getStatusNameZh(), ldStatusEnum.getStatusNameEn(), null);

                boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.CABINET_SETTLED, "");
                Date sailTime = null;
                BoxBookSeaDO boxBookSeaDO = boxBookSeaMapper.selectOne(new LambdaQueryWrapper<BoxBookSeaDO>().eq(BoxBookSeaDO::getShipmentId, shipmentId).last("limit 1"));
                if (Objects.nonNull(boxBookSeaDO)){
                    sailTime = boxBookSeaDO.getSailTime();
                }
                //修改订单状态
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, null, BoxStatusEnum.SEALED.getStatus(), null, null, ldBoxTime, sailTime, Integer.parseInt(boxDO.getTransportType()));

                //生成订单提货单
                //修改为提单审核后生成
//                boxService.generateBillLadingNo(shipmentId);

                //业绩日志
                boxService.addTargetLog(shipmentId, TargetLogEnum.LOADING, ldBoxTime);
                //发送短信
                try {
                    boxService.sendSms(shipmentId, SmsSceneEnum.SHIPMENT_CABINET);
                } catch (Exception e) {
                    log.error(e.getMessage());
                }
            }
        } else {
            //保存,修改为装柜中
            Integer ldStatus = boxDO.getLdStatus();
            if(ldStatus == LdStatusEnum.NO_CABINET.getLdStatus()) {
                boxDO.setLdStatus(LdStatusEnum.IN_CABINET.getLdStatus());
                boxService.updateById(boxDO);
            }

        }

    }


    @Override
    public BoxCabinetDO getBoxCabinet(Long id) {
        return boxCabinetMapper.selectById(id);
    }

    @Override
    public List<BoxCabinetDO> getBoxCabinetList(Collection<Long> ids) {
        return boxCabinetMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCabinetDO> getBoxCabinetPage(BoxCabinetQueryVO query, PageVO page) {
        return boxCabinetMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCabinetDO> getBoxCabinetList(BoxCabinetQueryVO query) {
        return boxCabinetMapper.selectList(query);
    }

    @Override
    public BoxCabinetDO getByShipmentId(Long shipmentId) {
        return getOne(new LambdaQueryWrapperX<BoxCabinetDO>()
            .eq(BoxCabinetDO::getShipmentId, shipmentId)
                .orderByDesc(BoxCabinetDO::getId)
                .last(" limit 1")
        );
    }
}
