package cn.iocoder.yudao.module.shipment.service.boxCabinetUnload;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.DeptWarehouseDO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseBaseVO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusMsgEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetUnloadConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetUnloadDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCabinetUnloadMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxCustoms.BoxCustomsService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxCabinetUnloadUpdateReqVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.DEPT_NOT_FOUND;

/**
 * 卸柜 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxCabinetUnloadServiceImpl extends AbstractService<BoxCabinetUnloadMapper, BoxCabinetUnloadDO> implements BoxCabinetUnloadService {

    @Resource
    private BoxCabinetUnloadMapper boxCabinetUnloadMapper;
    @Resource
    @Lazy
    private BoxService boxService;
    @Resource
    private BoxOpLogService boxOpLogService;
    @Resource
    private WarehouseService warehouseService;
    @Resource
    @Lazy
    private BoxPreloadGoodsService boxPreloadGoodsService;
    @Resource
    private BoxCustomsService boxCustomsService;
    @Resource
    private BoxApprovalService boxApprovalService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxCabinetUnload(BoxCabinetUnloadCreateReqVO createReqVO) {
        // 插入
        BoxCabinetUnloadDO boxCabinetUnload = BoxCabinetUnloadConvert.INSTANCE.convert(createReqVO);
        boxCabinetUnloadMapper.insert(boxCabinetUnload);
        // 返回
        updateBoxUlStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), createReqVO.getUlBoxTime());
        return boxCabinetUnload.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxCabinetUnload(BoxCabinetUnloadUpdateReqVO updateReqVO) {
        // 更新
        BoxCabinetUnloadDO updateObj = BoxCabinetUnloadConvert.INSTANCE.convert(updateReqVO);
        boxCabinetUnloadMapper.updateById(updateObj);
        updateBoxUlStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateReqVO.getUlBoxTime());
    }

    private void updateBoxUlStatus(Long shipmentId, Integer operateType, Date ulBoxTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            boxService.checkUnLoadNum(shipmentId);
            //校验是否审核中
            boxApprovalService.checkApproval(shipmentId, null);
            if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
                //校验是否是审核完成
                Integer toWarehouseStatus = boxDO.getToWarehouseStatus();
                if(toWarehouseStatus != TWStatusEnum.TO_WAREHOUSE_EXAMINE_SUCCESS.getTwStatus()) {
                    throw new ServiceException(500, "未完成到仓审核");
                }

                int currentStatus = boxDO.getToWarehouseStatus();
                TWStatusEnum twStatusEnum = TWStatusEnum.TO_WAREHOUSED_SEALED;
                boxDO.setToWarehouseStatus(twStatusEnum.getTwStatus());
                //已到仓日志
                boxOpLogService.createBoxOpLog(shipmentId, twStatusEnum.getTwStatus(),
                        twStatusEnum.getStatusNameZh(), twStatusEnum.getStatusNameEn(), null);
                //已到仓，则不执行下面业务
                if(currentStatus == TWStatusEnum.TO_WAREHOUSED_SEALED.getTwStatus()) return;
            } else {
                int currentStatus = boxDO.getUlStatus();
                ULStatusEnum ulStatusEnum = ULStatusEnum.UNLOADING_SEALED;
                boxDO.setUlStatus(ulStatusEnum.getUlStatus());
                //已卸柜日志
                boxOpLogService.createBoxOpLog(shipmentId, ulStatusEnum.getUlStatus(),
                        ulStatusEnum.getStatusNameZh(), ulStatusEnum.getStatusNameEn(), null);
                //已卸柜，则不执行下面业务
                if(currentStatus == ULStatusEnum.UNLOADING_SEALED.getUlStatus()) return;
            }

            if(ulBoxTime == null) {
                ulBoxTime = new Date();
            }

            boxDO.setThDate(ulBoxTime);
            boxDO.setThDate(ulBoxTime);

            if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
                //修改订单状态为已到仓
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.UNLOADED_CABINET.getValue(),
                        OrderStatusMsgEnum.UNLOADED_CABINET.getInWarehouseState(),
                        BoxAirStatusEnum.TO_WAREHOUSED.getStatus(), null, null, ulBoxTime, null, Integer.parseInt(boxDO.getTransportType()));

                boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.CABINET_WAREHOUSE, "");
            } else {
                //修改订单状态为已卸柜
                boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.UNLOADED_CABINET.getValue(),
                        OrderStatusMsgEnum.UNLOADED_CABINET.getInWarehouseState(),
                        BoxStatusEnum.UNLOADED.getStatus(), null, null, ulBoxTime, null, Integer.parseInt(boxDO.getTransportType()));

                boxService.addOrderLogByShipmentId(shipmentId, OrderShipmentLog.CABINET_UNLOAD, "");
            }

            //创建提货异常
            boxService.createPickUpException(shipmentId);
            //创建代收货款异常
            boxService.createPaymentException(shipmentId);
            if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
                // 空运站内信
                boxService.shipmentAirInternalMessage(shipmentId, boxDO.getSelfNo(), null, BoxAirStatusEnum.TO_WAREHOUSED, Integer.parseInt(boxDO.getTransportType()));
            }else {
                //发送站内信
                boxService.shipmentInternalMessage(shipmentId, null, null, BoxStatusEnum.UNLOADED, Integer.parseInt(boxDO.getTransportType()));
            }
        } else {
            if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
                Integer twStatus = boxDO.getToWarehouseStatus();
                if(twStatus == TWStatusEnum.NO_TO_WAREHOUSE.getTwStatus()) {
                    boxDO.setToWarehouseStatus(TWStatusEnum.TO_WAREHOUSING.getTwStatus());
                }
            } else {
                Integer ulStatus = boxDO.getUlStatus();
                if(ulStatus == ULStatusEnum.NO_CABINET_UNLOAD.getUlStatus()) {
                    boxDO.setUlStatus(ULStatusEnum.UNLOADING_CABINET.getUlStatus());
                }
            }

        }
        boxService.updateById(boxDO);
    }

    @Override
    public BoxCabinetUnloadDO getBoxCabinetUnload(Long id) {
        return boxCabinetUnloadMapper.selectById(id);
    }

    @Override
    public List<BoxCabinetUnloadDO> getBoxCabinetUnloadList(Collection<Long> ids) {
        return boxCabinetUnloadMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCabinetUnloadDO> getBoxCabinetUnloadPage(BoxCabinetUnloadQueryVO query, PageVO page) {
        return boxCabinetUnloadMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCabinetUnloadDO> getBoxCabinetUnloadList(BoxCabinetUnloadQueryVO query) {
        return boxCabinetUnloadMapper.selectList(query);
    }

    @Override
    public BoxCabinetUnloadDO getByShipmentId(Long shipmentId) {
        return getOne(new LambdaQueryWrapperX<BoxCabinetUnloadDO>()
            .eq(BoxCabinetUnloadDO::getShipmentId, shipmentId)
                .last(" limit 1")
        );
    }

    @Override
    public PageResult<BoxCabinetUnloadPDAVO> getBoxCabinetUnload4PDAPage(String selfNo,Integer ulStatus, PageVO page) {
        Long deptId = SecurityFrameworkUtils.getLoginUser().getDeptId();
        DeptWarehouseBaseVO deptWarehouseBaseVO = new DeptWarehouseBaseVO();
        deptWarehouseBaseVO.setDeptId(deptId);
        List<DeptWarehouseDO> deptWarehouseDOS = warehouseService.deptList(deptWarehouseBaseVO);

        if (CollectionUtil.isEmpty(deptWarehouseDOS)) {
            throw exception(DEPT_NOT_FOUND);
        }

        IPage<BoxCabinetUnloadPDAVO> mpPage = MyBatisUtils.buildPage(page);

        Set<Long> warehouseIds = deptWarehouseDOS.stream()
                .map(DeptWarehouseDO::getWarehouseId)
                .collect(Collectors.toSet());

        IPage<BoxCabinetUnloadPDAVO> result = boxCabinetUnloadMapper.getBoxCabinetUnload4PDAPage(mpPage,warehouseIds,selfNo,ulStatus);
        result.getRecords().stream().forEach(item ->{
            item.setBoxCustomsExceptionFlag(false);
            BoxCustomsDO boxCustomsDO = boxCustomsService.selectOne(new LambdaQueryWrapperX<BoxCustomsDO>().eq(BoxCustomsDO::getShipmentId, item.getId()).orderByDesc(BaseDO::getCreateTime).last("limit 1"));
            if (boxCustomsDO != null) {
                if (boxCustomsDO.getOverMachineStatus() == 2 && boxCustomsDO.getOverMachineAbnormalStatus() == 2) {
                    if (boxCustomsDO.getWeightMisreport() == 1||boxCustomsDO.getWeightMisreport() == 2) {
                        item.setBoxCustomsExceptionFlag(true);
                    }
                }
                if (boxCustomsDO.getOverWeight() != null && boxCustomsDO.getOverWeight().compareTo(BigDecimal.ZERO) > 0){
                    item.setBoxCustomsExceptionFlag(true);
                }
            }
            BoxGoodsStatistics boxStatistics = boxPreloadGoodsService.getBoxStatistics(item.getId());
            BeanUtils.copyProperties(boxStatistics,item);
        });
        return PageResult.of(result);
    }
}
