package cn.iocoder.yudao.module.shipment.service.boxCabinetUnloadAbnormal;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.enums.OrderExceptionEnum;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderException.OrderExceptionService;
import cn.iocoder.yudao.module.order.vo.orderException.OrderExceptionCreateReqVO;
import cn.iocoder.yudao.module.order.vo.orderException.OrderExceptionDescVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetUnloadAbnormalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetUnloadAbnormalDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCabinetUnloadAbnormalMapper;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 卸柜异常 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxCabinetUnloadAbnormalServiceImpl extends AbstractService<BoxCabinetUnloadAbnormalMapper, BoxCabinetUnloadAbnormalDO> implements BoxCabinetUnloadAbnormalService {

    @Resource
    private BoxCabinetUnloadAbnormalMapper boxCabinetUnloadAbnormalMapper;
    @Resource
    private OrderExceptionService orderExceptionService;
    @Resource
    private OrderService orderService;
    @Resource
    private OrderItemService orderItemService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxCabinetUnloadAbnormal(BoxCabinetUnloadAbnormalCreateReqVO createReqVO) {
        // 插入
        BoxCabinetUnloadAbnormalDO boxCabinetUnloadAbnormal = BoxCabinetUnloadAbnormalConvert.INSTANCE.convert(createReqVO);
        boxCabinetUnloadAbnormalMapper.insert(boxCabinetUnloadAbnormal);

        Long orderId = createReqVO.getOrderId();
        OrderDO orderDO = orderService.getById(orderId);

        Long orderItemId = createReqVO.getProductId();
        OrderItemDO orderItemDO = orderItemService.getOrderItem(orderItemId);


        //创建订单异常
        OrderExceptionCreateReqVO orderExceptionCreateReqVO = new OrderExceptionCreateReqVO();
        orderExceptionCreateReqVO.setOrderId(orderId);
        orderExceptionCreateReqVO.setOrderItemId(createReqVO.getProductId());
        orderExceptionCreateReqVO.setOrderNumber(orderDO.getOrderNo());

        OrderExceptionEnum orderExceptionType = null;
        Integer exceptionType = createReqVO.getExceptionType();
        if(exceptionType == 1) {
            orderExceptionType = OrderExceptionEnum.ORDER_MISS_EXCEPTION;
        } else if(exceptionType == 2) {
            orderExceptionType = OrderExceptionEnum.ORDER_SUPERFLUOUS_GOODS_EXCEPTION;
        } else if(exceptionType == 3) {
            orderExceptionType = OrderExceptionEnum.ORDER_DAMAGE_EXCEPTION;
        } else if(exceptionType == 4) {
            orderExceptionType = OrderExceptionEnum.ORDER_IN_WATER_EXCEPTION;
        } else if(exceptionType == 5) {
            orderExceptionType = OrderExceptionEnum.ORDER_ABNORMAL_WEIGHT_EXCEPTION;
        }
        orderExceptionCreateReqVO.setOrderExceptionType(orderExceptionType.getKey());

        DictDataRespDTO unitDto = DictFrameworkUtils.getDictDataFromCache("packaging_type", orderItemDO.getUnit());
        if(unitDto == null) {
            //默认取箱
            unitDto = DictFrameworkUtils.getDictDataFromCache("packaging_type", "1");
        }
        String descZh = new StringJoiner(", ").add(orderExceptionType.getZhValueDesc()).add(orderItemDO.getProdTitleZh())
                .add(createReqVO.getProductNum() + unitDto.getLabel()).toString();
        String descEn = new StringJoiner(", ").add(orderExceptionType.getEnValueDesc()).add(orderItemDO.getProdTitleEn())
                .add(createReqVO.getProductNum() + " " + unitDto.getLabelEn()).toString();

        orderExceptionCreateReqVO.setOrderExceptionDescVO(OrderExceptionDescVO.builder().descZh(descZh).descEn(descEn).build());

        orderExceptionService.createOrderException(orderExceptionCreateReqVO);

        // 返回
        return boxCabinetUnloadAbnormal.getId();
    }

    @Override
    public void updateBoxCabinetUnloadAbnormal(BoxCabinetUnloadAbnormalUpdateReqVO updateReqVO) {
        // 更新
        BoxCabinetUnloadAbnormalDO updateObj = BoxCabinetUnloadAbnormalConvert.INSTANCE.convert(updateReqVO);
        boxCabinetUnloadAbnormalMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxCabinetUnloadAbnormal(Long id) {
        // 删除
        boxCabinetUnloadAbnormalMapper.deleteById(id);
    }


    @Override
    public BoxCabinetUnloadAbnormalDO getBoxCabinetUnloadAbnormal(Long id) {
        return boxCabinetUnloadAbnormalMapper.selectById(id);
    }

    @Override
    public List<BoxCabinetUnloadAbnormalDO> getBoxCabinetUnloadAbnormalList(Collection<Long> ids) {
        return boxCabinetUnloadAbnormalMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCabinetUnloadAbnormalDO> getBoxCabinetUnloadAbnormalPage(BoxCabinetUnloadAbnormalQueryVO query, PageVO page) {
        return boxCabinetUnloadAbnormalMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCabinetUnloadAbnormalDO> getBoxCabinetUnloadAbnormalList(BoxCabinetUnloadAbnormalQueryVO query) {
        return boxCabinetUnloadAbnormalMapper.selectList(query);
    }
}
