package cn.iocoder.yudao.module.shipment.service.boxClearance;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.shipment.convert.BoxClearanceConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxClearanceOrder.BoxClearanceOrderDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxClearanceMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxClearanceOrder.BoxClearanceOrderService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceBaseVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceOrder.BoxClearanceOrderAllCreateVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceOrder.BoxClearanceOrderCreateVO;
import cn.iocoder.yudao.module.system.enums.sms.SmsSceneEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.base.Joiner;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 清关 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxClearanceServiceImpl extends AbstractService<BoxClearanceMapper, BoxClearanceDO> implements BoxClearanceService {

    @Resource
    private BoxClearanceMapper boxClearanceMapper;
    @Resource
    @Lazy
    private BoxService boxService;
    @Resource
    private BoxOpLogService boxOpLogService;
    @Resource
    @Lazy
    private BoxPreloadGoodsService preloadGoodsService;
    @Resource
    private BoxClearanceOrderService boxClearanceOrderService;
    @Resource
    @Lazy
    private OrderQueryService orderQueryService;
    @Resource
    @Lazy
    private BoxApprovalService boxApprovalService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxClearance(BoxClearanceCreateReqVO createReqVO) {
        // 插入
        BoxClearanceDO boxClearance = BoxClearanceConvert.INSTANCE.convert(createReqVO);
        boxClearanceMapper.insert(boxClearance);
        // 返回
        updateBoxClStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), createReqVO, createReqVO.getClearanceOrderIdList());
        return boxClearance.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxClearance(BoxClearanceUpdateReqVO updateReqVO) {
        // 更新
        BoxClearanceDO updateObj = BoxClearanceConvert.INSTANCE.convert(updateReqVO);
        boxClearanceMapper.updateById(updateObj);
        updateBoxClStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateReqVO, updateReqVO.getClearanceOrderIdList());
    }

    private void updateBoxClStatus(Long shipmentId, Integer operateType, BoxClearanceBaseVO clearanceBaseVO, List<Long> clearanceOrderIdList) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }

        if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
            updateBoxClStatusAir(boxDO, shipmentId, operateType, clearanceBaseVO, clearanceOrderIdList);
        } else {
            updateBoxClStatus(boxDO, shipmentId, operateType, clearanceBaseVO.getClClearTime());
        }
        boxService.updateById(boxDO);
    }

    private void updateBoxClStatus(BoxDO boxDO, Long shipmentId, Integer operateType, Date clClearTime) {
        if (operateType == OperateTypeEnum.COMMIT.getType()) {

            //已清关
            if (boxDO.getClStatus() == ClStatueEnum.HAS_CLEARANCE.getClStatus()) return;

            boxDO.setClStatus(ClStatueEnum.HAS_CLEARANCE.getClStatus());
            boxDO.setQgDate(clClearTime);

            //操作日志
            if (clClearTime != null) {
                boxOpLogService.createBoxOpLog(shipmentId, ClStatueEnum.HAS_CLEARANCE.getClStatus(), "已清关，待到仓", "", null);
                //发送清关短信
                boxService.sendSms(shipmentId, SmsSceneEnum.SHIPMENT_CUSTOMS_CLEARANCE);
            }

            //修改订单状态为已清关
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.CLEARED.getValue(), 0,
                    BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus(), null, null, clClearTime,null, Integer.parseInt(boxDO.getTransportType()));
            //发送站内信
            boxService.shipmentInternalMessage(shipmentId, null, null, BoxStatusEnum.CLEARANCE_WAIT_UNLOAD, Integer.parseInt(boxDO.getTransportType()));
        }
    }

    private void updateBoxClStatusAir(BoxDO boxDO, Long shipmentId, Integer operateType, BoxClearanceBaseVO boxClearanceBaseVO, List<Long> clearanceOrderIdList) {
        if (operateType == OperateTypeEnum.COMMIT.getType()) {
            if (boxDO.getClStatus() == ClStatueEnum.HAS_CLEARANCE.getClStatus()) return;
            //已清关

            Date clClearTime = boxClearanceBaseVO.getClClearTime();

            Collection<Long> clearanceOrders = null;
            //一次到港
            if(boxClearanceBaseVO.getClearanceType() == 0) {
                clearanceOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
            } else if(boxClearanceBaseVO.getClearanceType() == 1) {
                //多次到港
                if(CollectionUtils.isEmpty(clearanceOrderIdList)) {
                    throw new ServiceException(500, "未选择订单");
                } else {
                    clearanceOrders = clearanceOrderIdList;

                    //判断是否有撤销清关审核中的订单
                    List<BoxApprovalDO> approvalDOList = boxApprovalService.list(new LambdaQueryWrapper<BoxApprovalDO>()
                        .eq(BoxApprovalDO::getApprovalType, BoxApprovalTypeEnum.CLEARANCE_BACK.getType())
                        .eq(BoxApprovalDO::getShipmentId, shipmentId)
                        .eq(BoxApprovalDO::getApprovalStatus, BpmProcessInstanceResultEnum.PROCESS.getResult())
                    );
                    if(CollectionUtil.isNotEmpty(approvalDOList)) {
                        List<Long> orderIdList = approvalDOList.stream()
                                .map(BoxApprovalDO::getOrderId)
                                .collect(Collectors.toList());

                        List<Long> resultList = clearanceOrders.stream()
                                .filter(orderIdList::contains)
                                .distinct()
                                .collect(Collectors.toList());

                        if(CollectionUtil.isNotEmpty(resultList)) {
                            List<OrderDO> orderDOList = orderQueryService.getOrderList(resultList);
                            List<String> orderNoList = orderDOList.stream()
                                    .map(OrderDO::getOrderNo)
                                    .collect(Collectors.toList());
                            throw new ServiceException(500, "当前出货订单:" + Joiner.on(",").join(orderNoList) + "正在撤销清关审核中");
                        }
                    }

                }
            }

            updateOrderClearance(shipmentId, clearanceOrders, boxClearanceBaseVO.getClEstTime(), boxClearanceBaseVO.getClClearTime(), boxDO.getDgDate());

            updateOrderStatus(boxDO, clearanceOrders, clClearTime);
            updateBoxStatus(boxDO, clClearTime);
        }
    }

    @Override
    public BoxClearanceDO getBoxClearance(Long id) {
        return boxClearanceMapper.selectById(id);
    }

    @Override
    public List<BoxClearanceDO> getBoxClearanceList(Collection<Long> ids) {
        return boxClearanceMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxClearanceDO> getBoxClearancePage(BoxClearanceQueryVO query, PageVO page) {
        return boxClearanceMapper.selectPage(page, query);
    }

    @Override
    public List<BoxClearanceDO> getBoxClearanceList(BoxClearanceQueryVO query) {
        return boxClearanceMapper.selectList(query);
    }

    private void updateOrderClearance(Long shipmentId, Collection<Long> clearanceOrders, Date estTime, Date clearTime, Date dgTime) {
        List<BoxClearanceOrderDO> clearanceOrderDOList = boxClearanceOrderService.list(new LambdaQueryWrapper<BoxClearanceOrderDO>()
                .eq(BoxClearanceOrderDO::getShipmentId, shipmentId)
        );
        Map<Long, Long> orderMap = clearanceOrderDOList.stream()
                .collect(Collectors.toMap(BoxClearanceOrderDO::getOrderId, BoxClearanceOrderDO::getId));

        if(CollectionUtils.isNotEmpty(clearanceOrders)) {
            List<OrderDO> orderDOList = orderQueryService.getOrderList(clearanceOrders);

            List<BoxClearanceOrderDO> saveOrUpdateList = new ArrayList<>();
            for (OrderDO orderDO : orderDOList) {
                Long orderId = orderDO.getOrderId();
                BoxClearanceOrderDO boxClearanceOrderDO = new BoxClearanceOrderDO();
                boxClearanceOrderDO.setShipmentId(shipmentId);
                boxClearanceOrderDO.setOrderId(orderId);
                if(orderMap.containsKey(orderId)) {
                    boxClearanceOrderDO.setId(orderMap.get(orderId));
                    orderMap.remove(orderId);
                }
                boxClearanceOrderDO.setOrderNo(orderDO.getOrderNo());
                boxClearanceOrderDO.setClClearTime(clearTime);
                boxClearanceOrderDO.setClEstTime(estTime);
                saveOrUpdateList.add(boxClearanceOrderDO);
            }
            boxClearanceOrderService.saveOrUpdateBatch(saveOrUpdateList);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateOrderClearance(BoxClearanceOrderCreateVO createReqVO) {
        Date clearTime = createReqVO.getClClearTime();
        Long shipmentId = createReqVO.getShipmentId();
        List<Long> orderIdLIst = createReqVO.getOrderIdList();
        BoxDO boxDO = boxService.getBox(shipmentId);
        updateOrderClearance(shipmentId, orderIdLIst, createReqVO.getClEstTime(), clearTime, boxDO.getDgDate());
        updateOrderStatus(boxDO, orderIdLIst, clearTime);
        updateBoxStatus(boxDO, clearTime);
    }

    @Override
    public void updateAllOrderClearance(BoxClearanceOrderAllCreateVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();
        Date clearTime = createReqVO.getClClearTime();
        Collection<Long> clearanceOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
        BoxDO boxDO = boxService.getBox(shipmentId);
        updateOrderClearance(shipmentId, clearanceOrders, createReqVO.getClEstTime(), clearTime, boxDO.getDgDate());
        updateOrderStatus(boxDO, clearanceOrders, clearTime);
        updateBoxStatus(boxDO, clearTime);
    }

    private void updateBoxStatus(BoxDO boxDO, Date cleaTime) {
        Long shipmentId = boxDO.getId();
        Collection<Long> clearanceOrders = preloadGoodsService.getShipOrderIdList(shipmentId);
        long clearanceCount = boxClearanceOrderService.count(new LambdaQueryWrapper<BoxClearanceOrderDO>()
                .eq(BoxClearanceOrderDO::getShipmentId, shipmentId)
        );
        if(clearanceCount == clearanceOrders.size()) {
            boxDO.setClStatus(ClStatueEnum.HAS_CLEARANCE.getClStatus());
            boxDO.setQgDate(cleaTime);
            boxService.updateById(boxDO);
        }
    }

    private void updateOrderStatus(BoxDO boxDO, Collection<Long> clearanceOrders, Date clearTime) {
        if(CollectionUtils.isEmpty(clearanceOrders)) return;
        Long shipmentId = boxDO.getId();
        //操作日志
        if (clearTime != null) {
            boxOpLogService.createBoxOpLog(shipmentId, ClStatueEnum.HAS_CLEARANCE.getClStatus(), "已清关，待到仓", "", null);
            //发送清关短信
            boxService.sendSms(shipmentId, clearanceOrders, SmsSceneEnum.AIR_SHIPMENT_CUSTOMS_CLEARANCE);
        }

        //修改订单状态为已清关
        boxService.updateOrderStatusByOrderIdAndTransportType(clearanceOrders, OrderStatusEnum.CLEARED.getValue(), 0,
                BoxStatusEnum.CLEARANCE_WAIT_UNLOAD.getStatus(), null, null, clearTime,null, Integer.parseInt(boxDO.getTransportType()));
        // 空运站内信
        boxService.shipmentAirOrderInternalMessage(clearanceOrders, boxDO.getSelfNo(), null, BoxAirStatusEnum.CLEARANCE_WAIT_WAREHOUSE, Integer.parseInt(boxDO.getTransportType()));
    }
}
