package cn.iocoder.yudao.module.shipment.service.boxClearanceDoc;

import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;

import cn.hutool.core.util.ZipUtil;
import cn.iocoder.yudao.module.shipment.convert.BoxClearanceDocConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDocDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxClearanceDocMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.CdStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.CpStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocUpdateReqVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 清关文件 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxClearanceDocServiceImpl extends AbstractService<BoxClearanceDocMapper, BoxClearanceDocDO> implements BoxClearanceDocService {

    @Resource
    private BoxClearanceDocMapper boxClearanceDocMapper;

    @Resource
    BoxMapper boxMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxClearanceDoc(BoxClearanceDocCreateReqVO createReqVO) {
        // 插入
        BoxClearanceDocDO boxClearanceDoc = BoxClearanceDocConvert.INSTANCE.convert(createReqVO);
        boxClearanceDocMapper.insert(boxClearanceDoc);
        // 返回
        updateBoxCdStatus(createReqVO.getShipmentId(), createReqVO.getOperateType());
        return boxClearanceDoc.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxClearanceDoc(BoxClearanceDocUpdateReqVO updateReqVO) {
        // 更新
        BoxClearanceDocDO updateObj = BoxClearanceDocConvert.INSTANCE.convert(updateReqVO);
        boxClearanceDocMapper.updateById(updateObj);
        updateBoxCdStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType());
    }


    private void updateBoxCdStatus(Long shipmentId, Integer operateType) {
        BoxDO boxDO = boxMapper.selectById(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            CdStatueEnum cdStatueEnum = CdStatueEnum.HAS_CLEARANCE_DOCUMENTS;
            boxDO.setCdStatus(cdStatueEnum.getCdStatus());
        }
        boxMapper.updateById(boxDO);
    }


    @Override
    public BoxClearanceDocDO getBoxClearanceDoc(Long id) {
        return boxClearanceDocMapper.selectById(id);
    }

    @Override
    public List<BoxClearanceDocDO> getBoxClearanceDocList(Collection<Long> ids) {
        return boxClearanceDocMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxClearanceDocDO> getBoxClearanceDocPage(BoxClearanceDocQueryVO query, PageVO page) {
        return boxClearanceDocMapper.selectPage(page, query);
    }

    @Override
    public List<BoxClearanceDocDO> getBoxClearanceDocList(BoxClearanceDocQueryVO query) {
        return boxClearanceDocMapper.selectList(query);
    }

    @Override
    public void getDocZip(Long shipmentId, ServletOutputStream outputStream) throws Exception {
        BoxClearanceDocDO boxClearanceDocDO = boxClearanceDocMapper.selectOne(BoxClearanceDocDO::getShipmentId, shipmentId);
        if (boxClearanceDocDO != null) {
            String docFiles = boxClearanceDocDO.getDocFiles();
            if (StringUtils.isNotBlank(docFiles)) {
                String[] files = docFiles.split(",");
                List<InputStream> inputStreams = new ArrayList<>();
                for (String file : files) {
                    URL url = new URL(file);
                    inputStreams.add(url.openStream());
                }
                ZipUtil.zip(outputStream,files,inputStreams.stream().toArray(InputStream[]::new));
            }
        }
    }
}
