package cn.iocoder.yudao.module.shipment.service.boxCost;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCostDO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostUpdateReqVO;

/**
 * 费用登记 Service 接口
 *
 * @author 系统管理员
 */
public interface BoxCostService extends IService<BoxCostDO> {

    /**
     * 创建费用登记
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxCost(@Valid BoxCostCreateReqVO createReqVO);

    /**
     * 更新费用登记
     * @param updateReqVO 更新信息
     */
    void updateBoxCost(@Valid BoxCostUpdateReqVO updateReqVO);

    /**
     * 删除费用登记
     * @param id 编号
     */
    void deleteBoxCost(Long id);

    /**
     * 获得费用登记
     * @param id 编号
     * @return 费用登记
     */
    BoxCostDO getBoxCost(Long id);

    /**
     * 获得费用登记列表
     * @param ids 编号
     * @return 费用登记列表
     */
    List<BoxCostDO> getBoxCostList(Collection<Long> ids);

    /**
     * 获得费用登记分页
     * @param page 分页查询
     * @param query 查询
     * @return 费用登记分页
     */
    PageResult<BoxCostDO> getBoxCostPage(BoxCostQueryVO query, PageVO page);

    /**
     * 获得费用登记列表, 用于 Excel 导出
     * @param query 查询
     * @return 费用登记列表
     */
    List<BoxCostDO> getBoxCostList(BoxCostQueryVO query);
}
