package cn.iocoder.yudao.module.shipment.service.boxCustoms;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.module.bpm.enums.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxCustomsConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxBookSeaMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCustomsMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 报关 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxCustomsServiceImpl extends AbstractService<BoxCustomsMapper, BoxCustomsDO> implements BoxCustomsService {

    @Resource
    private BoxCustomsMapper boxCustomsMapper;
    @Lazy
    @Resource
    private BoxService boxService;

    @Resource
    private BoxBookSeaMapper boxBookSeaMapper;
    @Resource
    private BoxOpLogService boxOpLogService;
    @Resource
    private BoxApprovalService boxApprovalService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxCustoms(BoxCustomsCreateReqVO createReqVO) {
        // 插入
        BoxCustomsDO boxCustoms = BoxCustomsConvert.INSTANCE.convert(createReqVO);
        Integer dcCustomsStatus = createReqVO.getDcCustomsStatus();
        boxCustomsMapper.insert(boxCustoms);
        // 返回
        updateBoxDcStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(),
                dcCustomsStatus, createReqVO.getDcCheckStatus(), boxCustoms.getDcDecTime(),
                createReqVO.getOverMachineStatus(), createReqVO.getOverMachineAbnormalStatus(), createReqVO.getWeightMisreport());
        return boxCustoms.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxCustoms(BoxCustomsUpdateReqVO updateReqVO) {
        // 更新
        BoxCustomsDO updateObj = BoxCustomsConvert.INSTANCE.convert(updateReqVO);
        Integer dcCustomsStatus = updateReqVO.getDcCustomsStatus();
        boxCustomsMapper.updateById(updateObj);
        updateBoxDcStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(),
                dcCustomsStatus, updateReqVO.getDcCheckStatus(), updateObj.getDcDecTime(),
                updateObj.getOverMachineStatus(), updateObj.getOverMachineAbnormalStatus(), updateObj.getWeightMisreport());
    }

    private void updateBoxDcStatus(BoxDO boxDO, Integer operateType, Integer dcCustomsStatus, Date dcDecTime, Integer dcCheckStatus) {
        //创建退场审核
        Long shipmentId = boxDO.getId();
        Integer checkExamineStatus = null;

        //已报关，则不执行下面的业务，防止多次修改订单状态
        if(boxDO.getDcStatus() == DcStatueEnum.DECLARED.getDcStatus()) throw new ServiceException(500, "出货已报关");

        String opText = "";
        Date estTime = null;
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            int dcStatus = DcStatueEnum.CUSTOMING.getDcStatus();

            //1：报关中，2：放行，3：查验
            //查验状态，1：退场，2：部分退场，3：查验后放行
            //新增操作日志

            if(dcCustomsStatus == 1) {
                opText = "报关中";
            } else if(dcCustomsStatus == 2) {
                opText = "报关已放行";
                //已报关
                dcStatus = DcStatueEnum.DECLARED.getDcStatus();
                BoxBookSeaDO boxBookSea = boxBookSeaMapper.selectOne(new LambdaQueryWrapper<BoxBookSeaDO>().eq(BoxBookSeaDO::getShipmentId, shipmentId).last("limit 1"));
                if (Objects.nonNull(boxBookSea)) {
                    estTime = boxBookSea.getSailTime();
                }
            } else if(dcCustomsStatus == 3) {
                if(dcCheckStatus == 1) {
                    opText = "报关全退场";
                    checkExamineStatus = BpmProcessInstanceResultEnum.PROCESS.getResult();
                } else if(dcCheckStatus == 2) {
                    opText = "报关部分退场";
                    checkExamineStatus = BpmProcessInstanceResultEnum.PROCESS.getResult();
                } else if(dcCheckStatus == 3) {
                    opText = "报关查验后放行";
                    //已报关
                    dcStatus = DcStatueEnum.DECLARED.getDcStatus();
                }
            }

            boxDO.setDcStatus(dcStatus);

            boxOpLogService.createBoxOpLog(shipmentId, dcCustomsStatus, opText, opText, null);
        }
        boxDO.setDcCustomsStatus(dcCustomsStatus);
        boxDO.setDcCheckStatus(dcCheckStatus);
        boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, null, 0,
                boxDO.getShipmentStatus(), null, null, dcDecTime, estTime, Integer.parseInt(boxDO.getTransportType()));

        if(checkExamineStatus != null) {
            boxDO.setCheckExamineStatus(checkExamineStatus);

            BoxApprovalCreateReqVO boxApprovalCreateReqVO = new BoxApprovalCreateReqVO();
            boxApprovalCreateReqVO.setShipmentId(shipmentId);
            boxApprovalCreateReqVO.setApplyReason(opText);
            boxApprovalCreateReqVO.setApprovalStatus(checkExamineStatus);

            int approvalType;
            if(dcCheckStatus == 1) {
                approvalType = BoxApprovalTypeEnum.CUSTOMS_FULL_REFUND.getType();
            } else {
                approvalType = BoxApprovalTypeEnum.CUSTOMS_PART_REFUND.getType();
            }
            boxApprovalCreateReqVO.setApprovalType(approvalType);

            boxApprovalService.createBoxApproval(boxApprovalCreateReqVO);
        }

        boxService.updateById(boxDO);
    }

    private void updateBoxDcStatus(Long shipmentId, Integer operateType, Integer dcCustomsStatus, Integer dcCheckStatus, Date dcDecTime, Integer overMachineStatus, Integer overMachineAbnormalStatus, Integer weightMisreport) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(boxDO.getTransportType().equals(TransportTypeEnum.AIR.getType())) {
            updateBoxDcStatusAir(boxDO, operateType, dcCustomsStatus, dcCheckStatus, dcDecTime, overMachineStatus, overMachineAbnormalStatus, weightMisreport);
        } else {
            updateBoxDcStatus(boxDO, operateType, dcCustomsStatus, dcDecTime, dcCheckStatus);
        }
    }

    private void updateBoxDcStatusAir(BoxDO boxDO, Integer operateType, Integer dcCustomsStatus, Integer dcCheckStatus, Date dcDecTime, Integer overMachineStatus, Integer overMachineAbnormalStatus, Integer weightMisreport) {
        Long shipmentId = boxDO.getId();

        //创建退场审核
        Integer checkExamineStatus = null;

        //已报关，则不执行下面的业务，防止多次修改订单状态
        Integer flyStatus = boxDO.getFlyStatus();
        if(boxDO.getDcStatus() == DcStatueEnum.DECLARED.getDcStatus() &&
                flyStatus == FlyStatueEnum.FLYING.getDtStatus()) {
            //已报关并且已完成下一步，则不允许再次报关
            throw new ServiceException(500, "出货已报关");
        }

        String opText = "";
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            //1：报关中，2：放行，3：查验
            //查验状态，1：退场，2：部分退场，3：查验后放行
            //新增操作日志

            if(dcCustomsStatus == 1) {
                opText = "报关中";
            } else if(dcCustomsStatus == 2) {
                opText = "报关已放行";
            } else if(dcCustomsStatus == 3) {
                if(dcCheckStatus == 1) {
                    opText = "报关全退场";
                    checkExamineStatus = BpmProcessInstanceResultEnum.PROCESS.getResult();
                } else if(dcCheckStatus == 2) {
                    opText = "报关部分退场";
                    checkExamineStatus = BpmProcessInstanceResultEnum.PROCESS.getResult();
                } else if(dcCheckStatus == 3) {
                    opText = "报关查验后放行";
                }
            }
            boxOpLogService.createBoxOpLog(shipmentId, dcCustomsStatus, opText, opText, null);

            boxDO.setDcCustomsStatus(dcCustomsStatus);
            boxDO.setDcCheckStatus(dcCheckStatus);
            boxDO.setOverMachineStatus(overMachineStatus);
            boxDO.setOverMachineAbnormalStatus(overMachineAbnormalStatus);

            if(checkExamineStatus != null) {
                boxDO.setCheckExamineStatus(checkExamineStatus);

                BoxApprovalCreateReqVO boxApprovalCreateReqVO = new BoxApprovalCreateReqVO();
                boxApprovalCreateReqVO.setShipmentId(shipmentId);
                boxApprovalCreateReqVO.setApplyReason(opText);
                boxApprovalCreateReqVO.setApprovalStatus(checkExamineStatus);

                int approvalType;
                if(dcCheckStatus == 1) {
                    approvalType = BoxApprovalTypeEnum.CUSTOMS_FULL_REFUND.getType();
                } else {
                    approvalType = BoxApprovalTypeEnum.CUSTOMS_PART_REFUND.getType();
                }
                boxApprovalCreateReqVO.setApprovalType(approvalType);

                boxApprovalService.createBoxApproval(boxApprovalCreateReqVO);
            } else {
                //查验&查验后放行
                boolean checkPass = (dcCheckStatus != null && dcCheckStatus == 3);
                //&放行
                boolean customsPass = checkPass || dcCustomsStatus == 2;

                //重量误报已找到
                boolean overPass = weightMisreport != null && (weightMisreport == 3 || weightMisreport == 4);
                //&过机顺利
                boolean overMachinePass = overPass || overMachineStatus == 1;
                if(customsPass && overMachinePass) {
                     boxDO.setDcStatus(DcStatueEnum.DECLARED.getDcStatus());
                }
            }
            int shipmentStatus = boxDO.getShipmentStatusAir();
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.INSTALLED_CABINET.getValue(), null,
                    shipmentStatus, null, null, dcDecTime, null, Integer.parseInt(boxDO.getTransportType()));

            boxService.updateById(boxDO);
        }
    }

    @Override
    public BoxCustomsDO getBoxCustoms(Long id) {
        return boxCustomsMapper.selectById(id);
    }

    @Override
    public List<BoxCustomsDO> getBoxCustomsList(Collection<Long> ids) {
        return boxCustomsMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCustomsDO> getBoxCustomsPage(BoxCustomsQueryVO query, PageVO page) {
        return boxCustomsMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCustomsDO> getBoxCustomsList(BoxCustomsQueryVO query) {
        return boxCustomsMapper.selectList(query);
    }
}
