package cn.iocoder.yudao.module.shipment.service.boxCustomsExtraCost;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsExtraCostDO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostUpdateReqVO;

/**
 * 额外报关费用 Service 接口
 *
 * @author 系统管理员
 */
public interface BoxCustomsExtraCostService extends IService<BoxCustomsExtraCostDO> {

    /**
     * 创建额外报关费用
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBoxCustomsExtraCost(@Valid BoxCustomsExtraCostCreateReqVO createReqVO);

    /**
     * 更新额外报关费用
     * @param updateReqVO 更新信息
     */
    void updateBoxCustomsExtraCost(@Valid BoxCustomsExtraCostUpdateReqVO updateReqVO);

    /**
     * 删除额外报关费用
     * @param id 编号
     */
    void deleteBoxCustomsExtraCost(Long id);

    /**
     * 获得额外报关费用
     * @param id 编号
     * @return 额外报关费用
     */
    BoxCustomsExtraCostDO getBoxCustomsExtraCost(Long id);

    /**
     * 获得额外报关费用列表
     * @param ids 编号
     * @return 额外报关费用列表
     */
    List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostList(Collection<Long> ids);

    /**
     * 获得额外报关费用分页
     * @param page 分页查询
     * @param query 查询
     * @return 额外报关费用分页
     */
    PageResult<BoxCustomsExtraCostDO> getBoxCustomsExtraCostPage(BoxCustomsExtraCostQueryVO query, PageVO page);

    /**
     * 获得额外报关费用列表, 用于 Excel 导出
     * @param query 查询
     * @return 额外报关费用列表
     */
    List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostList(BoxCustomsExtraCostQueryVO query);

    Long searchByOrderNo(BoxCustomsExtraCostCreateReqVO createReqVO);

    List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostByOrderIdList(List<Long> orderIdList);

    void resetById(Long id);

    void resetByShipmentId(Long id);
}
