package cn.iocoder.yudao.module.shipment.service.boxCustomsExtraCost;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.shipment.convert.BoxCustomsExtraCostConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsExtraCostDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxCustomsExtraCostMapper;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 额外报关费用 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class BoxCustomsExtraCostServiceImpl extends AbstractService<BoxCustomsExtraCostMapper, BoxCustomsExtraCostDO> implements BoxCustomsExtraCostService {

    @Resource
    private BoxCustomsExtraCostMapper boxCustomsExtraCostMapper;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Override
    public Long searchByOrderNo(BoxCustomsExtraCostCreateReqVO createReqVO) {
        return boxPreloadGoodsService.selectByShipmentIdAndOrderNo(createReqVO.getShipmentId(), createReqVO.getOrderNo());
    }

    @Override
    public List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostByOrderIdList(List<Long> orderIdList) {
        return boxCustomsExtraCostMapper.selectList(new LambdaQueryWrapper<BoxCustomsExtraCostDO>()
            .in(BoxCustomsExtraCostDO::getOrderId, orderIdList)
        );
    }

    @Override
    public void resetById(Long id) {
        BoxCustomsExtraCostDO boxCustomsExtraCostDO = getById(id);
        boxCustomsExtraCostDO.setFee(boxCustomsExtraCostDO.getInitFee());
        updateById(boxCustomsExtraCostDO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void resetByShipmentId(Long id) {
        boxCustomsExtraCostMapper.updateSetInitFee(id);
    }

    @Override
    public Long createBoxCustomsExtraCost(BoxCustomsExtraCostCreateReqVO createReqVO) {
        Long orderId = createReqVO.getOrderId();
        Long shipmentId = createReqVO.getShipmentId();
        Long count = boxCustomsExtraCostMapper.selectCount(new LambdaQueryWrapperX<BoxCustomsExtraCostDO>()
            .eq(BoxCustomsExtraCostDO::getShipmentId, shipmentId)
                .eq(BoxCustomsExtraCostDO::getOrderId, orderId)
        );

        if(count > 0) {
            throw new ServiceException(500, "此订单已添加额外费用");
        }

        // 插入
        BoxCustomsExtraCostDO boxCustomsExtraCost = BoxCustomsExtraCostConvert.INSTANCE.convert(createReqVO);
        boxCustomsExtraCost.setInitFee(createReqVO.getFee());
        boxCustomsExtraCostMapper.insert(boxCustomsExtraCost);
        // 返回
        return boxCustomsExtraCost.getId();
    }

    @Override
    public void updateBoxCustomsExtraCost(BoxCustomsExtraCostUpdateReqVO updateReqVO) {
        // 更新
        BoxCustomsExtraCostDO updateObj = BoxCustomsExtraCostConvert.INSTANCE.convert(updateReqVO);
        boxCustomsExtraCostMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxCustomsExtraCost(Long id) {
        // 删除
        boxCustomsExtraCostMapper.deleteById(id);
    }

    @Override
    public BoxCustomsExtraCostDO getBoxCustomsExtraCost(Long id) {
        return boxCustomsExtraCostMapper.selectById(id);
    }

    @Override
    public List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostList(Collection<Long> ids) {
        return boxCustomsExtraCostMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxCustomsExtraCostDO> getBoxCustomsExtraCostPage(BoxCustomsExtraCostQueryVO query, PageVO page) {
        return boxCustomsExtraCostMapper.selectPage(page, query);
    }

    @Override
    public List<BoxCustomsExtraCostDO> getBoxCustomsExtraCostList(BoxCustomsExtraCostQueryVO query) {
        return boxCustomsExtraCostMapper.selectList(query);
    }
}
