package cn.iocoder.yudao.module.shipment.service.boxLadingBill;

import java.io.*;
import java.util.*;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.clearanceFileNo.ClearanceFileNoDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.supplier.SupplierDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.supplier.SupplierExternalDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.ecw.service.clearanceFileNo.ClearanceFileNoService;
import cn.iocoder.yudao.module.ecw.service.dock.DockService;
import cn.iocoder.yudao.module.ecw.service.supplier.SupplierExternalService;
import cn.iocoder.yudao.module.ecw.service.supplier.SupplierService;
import cn.iocoder.yudao.module.ecw.service.unit.UnitService;
import cn.iocoder.yudao.module.shipment.convert.BoxLadingBillConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAgentDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLadingBillDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxShippingDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxLadingBillMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.BlStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.TlStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxAgent.BoxAgentService;
import cn.iocoder.yudao.module.shipment.service.boxShipping.BoxShippingService;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillUpdateReqVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.itextpdf.text.pdf.AcroFields;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.MAKE_BILL_OF_LADING_NOT_EXISTS;

/**
 * 提单补料 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxLadingBillServiceImpl extends AbstractService<BoxLadingBillMapper, BoxLadingBillDO> implements BoxLadingBillService {

    @Resource
    private BoxLadingBillMapper boxLadingBillMapper;
    @Resource
    private BoxMapper boxMapper;

    @Resource
    private BoxShippingService boxShippingService;

    @Resource
    private SupplierService supplierService;

    @Resource
    private DockService dockService;

    @Resource
    private UnitService unitService;

    @Resource
    private CabinetService cabinetService;

    @Resource
    private SupplierExternalService supplierExternalService;

    @Resource
    private ClearanceFileNoService clearanceFileNoService;

    @Resource
    @Lazy
    private BoxAgentService boxAgentService;

    @Value("${ue.templates-url}")
    private String path;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxLadingBill(BoxLadingBillCreateReqVO createReqVO) {
        // 插入
        BoxLadingBillDO boxLadingBill = BoxLadingBillConvert.INSTANCE.convert(createReqVO);
        boxLadingBillMapper.insert(boxLadingBill);
        updateBoxBlStatus(createReqVO.getShipmentId(), createReqVO.getOperateType());
        // 返回
        return boxLadingBill.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxLadingBill(BoxLadingBillUpdateReqVO updateReqVO) {
        // 更新
        BoxLadingBillDO updateObj = BoxLadingBillConvert.INSTANCE.convert(updateReqVO);
        boxLadingBillMapper.updateById(updateObj);
        updateBoxBlStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType());
        boxAgentService.update(new LambdaUpdateWrapper<BoxAgentDO>().set(BoxAgentDO::getAgentId,updateObj.getAgentId()).eq(BoxAgentDO::getShipmentId,updateObj.getShipmentId()));
    }

    private void updateBoxBlStatus(Long shipmentId, Integer operateType) {
        BoxDO boxDO = boxMapper.selectById(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            boxDO.setBlStatus(BlStatueEnum.HAS_LADING_BILL.getBlStatus());
        }
        boxMapper.updateById(boxDO);
    }

    @Override
    public BoxLadingBillDO getBoxLadingBill(Long id) {
        return boxLadingBillMapper.selectById(id);
    }

    @Override
    public List<BoxLadingBillDO> getBoxLadingBillList(Collection<Long> ids) {
        return boxLadingBillMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxLadingBillDO> getBoxLadingBillPage(BoxLadingBillQueryVO query, PageVO page) {
        return boxLadingBillMapper.selectPage(page, query);
    }

    @Override
    public List<BoxLadingBillDO> getBoxLadingBillList(BoxLadingBillQueryVO query) {
        return boxLadingBillMapper.selectList(query);
    }

    @Override
    public Map<String,String> getBoxLadingBillPdf(Long shipmentId) throws Exception {
        BoxLadingBillDO boxLadingBillDO = boxLadingBillMapper.selectOne(BoxLadingBillDO::getShipmentId, shipmentId);
        if (boxLadingBillDO == null) {
            throw exception(MAKE_BILL_OF_LADING_NOT_EXISTS);
        }
        BoxShippingDO boxShippingDO = boxShippingService.getOne(new LambdaQueryWrapperX<BoxShippingDO>().eq(BoxShippingDO::getShipmentId, shipmentId));
        BoxDO box = boxMapper.selectById(shipmentId);
        CabinetDO cabinet = cabinetService.getCabinet(box.getCabinetId());
        Map<String,String> headMap = new HashMap<>();
        String shipType = "X1";
        if (cabinet != null) {
            shipType = cabinet.getName()+"X1";
        }
        headMap.put("shipperInfo",buildMsg(boxLadingBillDO.getShipperId(),0));
        headMap.put("consigneeInfo",buildMsg(boxLadingBillDO.getAgentId(),1));
        headMap.put("notifyInfo",buildMsg(boxLadingBillDO.getAgentId(),2));
        headMap.put("mblNo",boxLadingBillDO.getBlMblNo());
        headMap.put("soNo",boxLadingBillDO.getSoNo());
        if (boxShippingDO != null) {
            headMap.put("date", boxShippingDO.getDtRealShipTime() == null ? "" : DateUtil.formatDateTime(boxShippingDO.getDtRealShipTime()));
        }
        if (boxLadingBillDO.getShipCompanyId() != null && boxLadingBillDO.getShipCompanyId() > 0L) {
            headMap.put("shipCompany",supplierService.getSupplier(boxLadingBillDO.getShipCompanyId()).getCompanyEn());
        }
        headMap.put("shipType",shipType);
        headMap.put("freight","FREIGHT PREPAID");
        StringBuffer remarks = new StringBuffer();
        Integer issueType = boxLadingBillDO.getIssueType();
        if (StringUtils.isNotBlank(boxLadingBillDO.getRemarks())) {
            remarks.append(boxLadingBillDO.getRemarks());
        }
        if (issueType == 1) {
            remarks.append("\r\n").append("正本");
        } else if (issueType == 2) {
            remarks.append("\r\n").append("电放");
        }
        headMap.put("remark",remarks.toString());
        if (boxLadingBillDO.getStartPortId() != null && boxLadingBillDO.getStartPortId() > 0L) {
            headMap.put("loadingPort",dockService.getDock(boxLadingBillDO.getStartPortId()).getTitleEn());
        }
        if (boxLadingBillDO.getDestPortId() != null && boxLadingBillDO.getDestPortId() > 0L) {
            headMap.put("dischargingPort",dockService.getDock(boxLadingBillDO.getDestPortId()).getTitleEn());
        }
        headMap.put("markNo",boxLadingBillDO.getMarkNo());
        if (StringUtils.isEmpty(boxLadingBillDO.getPackageUnit())) {
            headMap.put("pkgs",boxLadingBillDO.getPackageNum() == null ? "" : boxLadingBillDO.getPackageNum() + "");
        } else {
            DictDataRespDTO dictDataFromCache = DictFrameworkUtils.getDictDataFromCache("goods_package_type", boxLadingBillDO.getPackageUnit());
            if (dictDataFromCache != null) {
                headMap.put("pkgs",boxLadingBillDO.getPackageNum() == null ? "" : boxLadingBillDO.getPackageNum() + dictDataFromCache.getLabelEn());
            } else {
                headMap.put("pkgs",boxLadingBillDO.getPackageNum() == null ? "" : boxLadingBillDO.getPackageNum() + "");
            }
        }
//        StringBuffer stringBuffer = new StringBuffer();
//        SupplierExternalDO supplierExternal = supplierExternalService.selectOne(SupplierExternalDO::getSupplierId,boxLadingBillDO.getAgentId());
//        if (supplierExternal != null) {
//            if (StringUtils.isNotBlank(supplierExternal.getFromNo())) {
//                stringBuffer.append("FROM M NO.:").append(supplierExternal.getFromNo()+"\r\n");
//            }
//            if (StringUtils.isNotBlank(supplierExternal.getBaNo())) {
//                stringBuffer.append("BA NO.:").append(supplierExternal.getBaNo()+"\r\n");
//            }
//            if (StringUtils.isNotBlank(supplierExternal.getCtnNo())) {
//                stringBuffer.append("CTN NO.:").append(supplierExternal.getCtnNo()+"\r\n");
//            }
//        }
//        headMap.put("description",boxLadingBillDO.getGoodsDesc()+"\r\n"+stringBuffer.toString());
        headMap.put("description",boxLadingBillDO.getGoodsDesc());
        headMap.put("weight",boxLadingBillDO.getKgs() == null ? "" : String.valueOf(boxLadingBillDO.getKgs().doubleValue()));
        headMap.put("volume",boxLadingBillDO.getCbm() == null ? "" : String.valueOf(boxLadingBillDO.getCbm().doubleValue()));
        headMap.put("boxNo",boxLadingBillDO.getContainerSealNo());
        headMap.put("totalNum","SAY("+shipType+")ONLY ");
        return headMap;
    }

    private String buildMsg(Long id,int i){
        StringBuffer stringBuffer = new StringBuffer("");
        if (id == null || id == 0L) {
            return stringBuffer.toString();
        } else {
            if (i == 0) {
                SupplierDO supplier = supplierService.getSupplier(id);
                if (StringUtils.isNotBlank(supplier.getCompanyEn())) {
                    stringBuffer.append(supplier.getCompanyEn()).append("\r\n");
                }
                if (StringUtils.isNotBlank(supplier.getEmail())) {
                    stringBuffer.append(supplier.getEmail()).append("\r\n");
                }
                if (StringUtils.isNotBlank(supplier.getAddress())) {
                    stringBuffer.append(supplier.getAddress()).append("\r\n");
                }
                if (StringUtils.isNotBlank(supplier.getTell())) {
                    stringBuffer.append(supplier.getTell()).append("\r\n");
                }
            } else if (i == 1) {
                SupplierExternalDO supplierExternal = supplierExternalService.selectOne(SupplierExternalDO::getSupplierId,id);
                if (supplierExternal != null) {
                    if (StringUtils.isNotBlank(supplierExternal.getReceiveCompanyName())) {
                        stringBuffer.append(supplierExternal.getReceiveCompanyName()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getReceiveEmail())) {
                        stringBuffer.append(supplierExternal.getReceiveEmail()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getReceiveAddress())) {
                        stringBuffer.append(supplierExternal.getReceiveAddress()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getReceiveTel())) {
                        stringBuffer.append(supplierExternal.getReceiveTel()).append("\r\n");
                    }
                }
            } else {
                SupplierExternalDO supplierExternal = supplierExternalService.selectOne(SupplierExternalDO::getSupplierId,id);
                if (supplierExternal != null) {
                    if (StringUtils.isNotBlank(supplierExternal.getNotifyCompanyName())) {
                        stringBuffer.append(supplierExternal.getNotifyCompanyName()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getNotifyEmail())) {
                        stringBuffer.append(supplierExternal.getNotifyEmail()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getNotifyAddress())) {
                        stringBuffer.append(supplierExternal.getNotifyAddress()).append("\r\n");
                    }
                    if (StringUtils.isNotBlank(supplierExternal.getNotifyTel())) {
                        stringBuffer.append(supplierExternal.getNotifyTel()).append("\r\n");
                    }
                }
            }
            return stringBuffer.toString();
        }
    }
}
