package cn.iocoder.yudao.module.shipment.service.boxLadingCopy;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.shipment.convert.BoxLadingCopyConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLadingCopyDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxLadingCopyMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.enums.CpStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyUpdateReqVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 提单copy Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxLadingCopyServiceImpl extends AbstractService<BoxLadingCopyMapper, BoxLadingCopyDO> implements BoxLadingCopyService {

    @Resource
    private BoxLadingCopyMapper boxLadingCopyMapper;

    @Resource
    private BoxMapper boxMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxLadingCopy(BoxLadingCopyCreateReqVO createReqVO) {
        // 插入
        BoxLadingCopyDO boxLadingCopy = BoxLadingCopyConvert.INSTANCE.convert(createReqVO);
        boxLadingCopyMapper.insert(boxLadingCopy);
        // 返回
        updateBoxCpStatus(createReqVO.getShipmentId(), createReqVO.getOperateType());
        return boxLadingCopy.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxLadingCopy(BoxLadingCopyUpdateReqVO updateReqVO) {
        // 更新
        BoxLadingCopyDO updateObj = BoxLadingCopyConvert.INSTANCE.convert(updateReqVO);
        boxLadingCopyMapper.updateById(updateObj);
        updateBoxCpStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType());
    }

    private void updateBoxCpStatus(Long shipmentId, Integer operateType) {
        BoxDO boxDO = boxMapper.selectById(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            CpStatueEnum cpStatueEnum = CpStatueEnum.HAS_UPLOAD;
            boxDO.setCpStatus(cpStatueEnum.getCpStatus());
        }
        boxMapper.updateById(boxDO);
    }

    @Override
    public BoxLadingCopyDO getBoxLadingCopy(Long id) {
        return boxLadingCopyMapper.selectById(id);
    }

    @Override
    public List<BoxLadingCopyDO> getBoxLadingCopyList(Collection<Long> ids) {
        return boxLadingCopyMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxLadingCopyDO> getBoxLadingCopyPage(BoxLadingCopyQueryVO query, PageVO page) {
        return boxLadingCopyMapper.selectPage(page, query);
    }

    @Override
    public List<BoxLadingCopyDO> getBoxLadingCopyList(BoxLadingCopyQueryVO query) {
        return boxLadingCopyMapper.selectList(query);
    }
}
