package cn.iocoder.yudao.module.shipment.service.boxLoadInfo;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLoadInfoDO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 已装柜标签 Service 接口
 *
 * @author wlh
 */
public interface BoxLoadInfoService extends IService<BoxLoadInfoDO> {

    /**
     * 创建已装柜标签
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long singleCreate(@Valid BoxLoadInfoCreateReqVO createReqVO);

    /**
     * 批量装柜
     * @param createReqVO 创建信息
     * @return
     */
    Long batchCreate(@Valid BoxLoadInfoCreateReqVO createReqVO);

    /**
     * 更新已装柜标签
     * @param updateReqVO 更新信息
     */
    void updateBoxLoadInfo(@Valid BoxLoadInfoUpdateReqVO updateReqVO);

    /**
     * 删除已装柜标签
     * @param deleteReq deleteReq
     */
    void deleteBoxLoadInfo(BoxLoadInfoCreateReqVO deleteReq);

    /**
     * 获得已装柜标签
     * @param id 编号
     * @return 已装柜标签
     */
    BoxLoadInfoDO getBoxLoadInfo(Long id);

    /**
     * 获得已装柜标签列表
     * @param ids 编号
     * @return 已装柜标签列表
     */
    List<BoxLoadInfoDO> getBoxLoadInfoList(Collection<Long> ids);

    /**
     * 获得已装柜标签分页
     * @param page 分页查询
     * @param query 查询
     * @return 已装柜标签分页
     */
    PageResult<BoxLoadInfoDO> getBoxLoadInfoPage(BoxLoadInfoQueryVO query, PageVO page);

    /**
     * 获得已装柜标签列表, 用于 Excel 导出
     * @param query 查询
     * @return 已装柜标签列表
     */
    List<BoxLoadInfoDO> getBoxLoadInfoList(BoxLoadInfoQueryVO query);

    /**
     * 移出
     * @param removeReqVO removeReqVO
     */
    void removeBox(BoxLoadInfoRemoveReqVO removeReqVO);

    CommonResult<BoxLoadInfoOrderVO> orderTagList(Long orderId);

    void boxUpdate(BoxInfoUpdateReq boxInfoUpdateReq);

    void unloadBoxInfo(BoxLoadInfoCreateReqVO createReqVO);

    void airUnloadBoxInfo(BoxLoadInfoCreateReqVO createReqVO);

    void unloadAllBoxInfo(BoxLoadInfoCreateReqVO createReqVO);

    /**
     * 批量删除标签号
     * @param orderNumCodeList  标签号列表
     */
    void batchDelete(List<String> orderNumCodeList, Long shipmentId);

    /**
     * 删除装柜的标签
     * @param shipmentId 出货单ID
     * @param orderIdList  订单ID列表
     */
    void deleteLoadInfo(Long shipmentId, List<Long> orderIdList);

    /**
     * 根据箱号和出货号找出订单号
     * @param shipmentId  出货单
     * @param orderNumCode  箱号
     * @return
     */
    Long getOrderIdByOrderNumCode(Long shipmentId, String orderNumCode);

    /**
     * 外部仓装箱
     * @param createReqVO 请求参数
     * @return
     */
    Long externalLoad(BoxExternalLoadReq createReqVO);

    void createByShipment(Long shipmentId);

    void unloadAllAirBoxInfo(BoxLoadInfoCreateReqVO createReqVO);

    Long singleAirCreate(@Valid BoxLoadInfoCreateReqVO createReqVO);

    Long batchAirCreate(@Valid BoxLoadInfoCreateReqVO createReqVO);
}
