package cn.iocoder.yudao.module.shipment.service.boxLoadInfo;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLabel.OrderLabelDO;
import cn.iocoder.yudao.module.order.enums.ApprovalResultStatusEnum;
import cn.iocoder.yudao.module.order.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusMsgEnum;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderLabel.OrderLabelService;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.order.vo.orderLabel.OrderLabelQueryVO;
import cn.iocoder.yudao.module.order.vo.orderLocation.OrderLocationCreateReqVO;
import cn.iocoder.yudao.module.shipment.convert.BoxLoadInfoConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxArrivalOrder.BoxArrivalOrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxClearanceOrder.BoxClearanceOrderDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxLoadInfoMapper;
import cn.iocoder.yudao.module.shipment.enums.*;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxArrivalOrder.BoxArrivalOrderService;
import cn.iocoder.yudao.module.shipment.service.boxCabinet.BoxCabinetService;
import cn.iocoder.yudao.module.shipment.service.boxClearanceOrder.BoxClearanceOrderService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.*;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.base.Joiner;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.BoxApprovalTypeEnum.SEALING_CABINET_UMPIRE;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 已装柜标签 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxLoadInfoServiceImpl extends AbstractService<BoxLoadInfoMapper, BoxLoadInfoDO> implements BoxLoadInfoService {

    @Resource
    private BoxLoadInfoMapper boxLoadInfoMapper;
    @Resource
    @Lazy
    private BoxPreloadGoodsService boxPreloadGoodsService;
    @Resource
    private OrderService orderService;
    @Resource
    private OrderQueryService orderQueryService;
    @Resource
    private OrderItemService orderItemService;
    @Resource
    CabinetService cabinetService;
    @Resource
    @Lazy
    BoxService boxService;
    @Resource
    OrderLabelService orderLabelService;
    @Resource
    @Lazy
    private BoxCabinetService boxCabinetService;
    @Resource
    @Lazy
    private BoxApprovalService boxApprovalService;
    @Resource
    @Lazy
    private BoxArrivalOrderService boxArrivalOrderService;
    @Resource
    @Lazy
    private BoxClearanceOrderService boxClearanceOrderService;

    @Resource
    @Lazy
    private BoxMergePkgService boxMergePkgService;

    @Resource
    @Lazy
    private BoxPkgOrderService boxPkgOrderService;

    public static void main(String[] args) {
        String number = "NG2200491L-01-1";
        int position = number.lastIndexOf("-");
        if(position >= 0) {
            System.out.println(position);
            String str = number.substring(position+1);
            String orderNumber = number.substring(0, position);
            System.out.println(orderNumber);
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long singleCreate(BoxLoadInfoCreateReqVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();

        boxService.checkApproval(shipmentId, null);

        // 插入
        BoxLoadInfoDO boxLoadInfo = BoxLoadInfoConvert.INSTANCE.convert(createReqVO);

        String orderNumCode = createReqVO.getOrderNumCode();

        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        Set<Long> orderIdList = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                .collect(Collectors.toSet());
        //这个出货单下的所有订单
        List<OrderDO> orderDOList = orderQueryService.getOrderList(orderIdList);

        int position = orderNumCode.lastIndexOf("-");
        if(position >= 0) {
            String boxNumber = orderNumCode.substring(position + 1);
            String orderNumber = orderNumCode.substring(0, position);
            OrderDO orderDO = orderService.getOne(new LambdaQueryWrapperX<OrderDO>()
                .eq(OrderDO::getOrderNo, orderNumber)
            );
            if(orderDO == null) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }
            //判断这个订单是不是已经预装订单里面的父订单
            Long orderId = orderDO.getOrderId();
            if(!orderIdList.contains(orderId)) {

                for (OrderDO item : orderDOList) {
                    String parentNumber = item.getParentNumber();
                    if(parentNumber != null && parentNumber.equals(orderNumber)) {
                        //是这个订单的父订单，判断这个子订单是不是01开头
                        String itemOrderNumber = item.getOrderNo();
                        String firstOrderNumber = orderNumber + "-01";
                        if(itemOrderNumber.equals(firstOrderNumber)) {
                            //这个子订单相当于原始订单
                            //修改这个箱号为拆单的箱号
                            orderNumCode = itemOrderNumber + "-" + boxNumber;
                            boxLoadInfo.setOrderNumCode(orderNumCode);
                        }
                    }
                }
            }
        } else {
            //箱号不存在
            //扫描的是空运的合包箱号
            BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
                    .eq(BoxMergePkgDO::getShipmentId, shipmentId)
                    .eq(BoxMergePkgDO::getPkgNum, orderNumCode)
            );
            if (boxMergePkgDO != null) {
                List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapper<BoxPkgOrderDO>()
                        .eq(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId())
                );
                if (CollectionUtil.isNotEmpty(boxPkgOrderDOS)) {
                    List<Long> orderIds = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toList());
                    List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, orderIds);
                    orderDOS.stream().forEach(item -> {
                        BoxLoadInfoCreateReqVO boxLoadInfoCreateReqVO = new BoxLoadInfoCreateReqVO();
                        boxLoadInfoCreateReqVO.setShipmentId(shipmentId);
                        boxLoadInfoCreateReqVO.setOrderNo(item.getOrderNo());
                        batchCreate(boxLoadInfoCreateReqVO);
                    });
                    return 0L;
                }
            } else {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }
        }

        //查找这个是否已经装了

        BoxLoadInfoDO existDO = getOne(new LambdaQueryWrapper<BoxLoadInfoDO>()
            .eq(BoxLoadInfoDO::getOrderNumCode, orderNumCode)
        );
        if(existDO != null) {
            Long loadShipmentId = existDO.getShipmentId();
            if(loadShipmentId.equals(shipmentId)) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_EXISTS);
            } else {
                BoxDO boxDO = boxService.getById(loadShipmentId);
                throw new ServiceException(500, "该箱号已装柜在出货" + boxDO.getSelfNo() + "中!");
            }
        }

        Long orderId = searchOrderIdByOrderNumCode(orderNumCode, orderIdList, orderDOList);

        if(orderId == null) {
            throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
        }

        boxLoadInfo.setOrderId(orderId);

        boxLoadInfoMapper.insert(boxLoadInfo);

        //修改为装柜中
//        updateInCabinet(createReqVO.getShipmentId());
        // 返回

        boxService.addInstallCompleteLog(shipmentId);

        updateInCabinet(shipmentId);

        updateCabinetDate(shipmentId);

        return boxLoadInfo.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long singleAirCreate(BoxLoadInfoCreateReqVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();

        boxService.checkApproval(shipmentId, null);

        // 插入
        BoxLoadInfoDO boxLoadInfo = BoxLoadInfoConvert.INSTANCE.convert(createReqVO);

        String orderNumCode = createReqVO.getOrderNumCode();

        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        Set<Long> orderIdList = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                .collect(Collectors.toSet());
        //这个出货单下的所有订单
        List<OrderDO> orderDOList = orderQueryService.getOrderList(orderIdList);

        int position = orderNumCode.lastIndexOf("-");
        if(position >= 0) {
            String boxNumber = orderNumCode.substring(position + 1);
            String orderNumber = orderNumCode.substring(0, position);
            OrderDO orderDO = orderService.getOne(new LambdaQueryWrapperX<OrderDO>()
                    .eq(OrderDO::getOrderNo, orderNumber)
            );
            if(orderDO == null) {
                throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
            }
            //判断这个订单是不是已经预装订单里面的父订单
            Long orderId = orderDO.getOrderId();
            if(!orderIdList.contains(orderId)) {

                for (OrderDO item : orderDOList) {
                    String parentNumber = item.getParentNumber();
                    if (parentNumber != null && parentNumber.equals(orderNumber)) {
                        //是这个订单的父订单，判断这个子订单是不是01开头
                        String itemOrderNumber = item.getOrderNo();
                        String firstOrderNumber = orderNumber + "-01";
                        if (itemOrderNumber.equals(firstOrderNumber)) {
                            //这个子订单相当于原始订单
                            //修改这个箱号为拆单的箱号
                            orderNumCode = itemOrderNumber + "-" + boxNumber;
                            boxLoadInfo.setOrderNumCode(orderNumCode);
                        }
                    }
                }
            }
//            else {
//                BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
//                        .eq(BoxMergePkgDO::getShipmentId, shipmentId)
//                        .eq(BoxMergePkgDO::getPkgNum, orderNumCode)
//                );
//                Long selectCount = boxPkgOrderService.selectCount(new LambdaQueryWrapper<BoxPkgOrderDO>()
//                        .eq(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId())
//                        .eq(BoxPkgOrderDO::getOrderId, orderDO.getOrderId())
//                );
//                if (selectCount == null || selectCount.longValue() == 0L) {
//                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.ORDER_NOT_IN_BOX);
//                } else {
//                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.CASE_NUM_IN_MERGE_PKG);
//                }
//            }
        } else {
            //箱号不存在
            //扫描的是空运的合包箱号
            BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
                    .eq(BoxMergePkgDO::getShipmentId, shipmentId)
                    .eq(BoxMergePkgDO::getPkgNum, orderNumCode)
            );
            if (boxMergePkgDO != null) {
                List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapper<BoxPkgOrderDO>()
                        .eq(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId())
                );
                if (CollectionUtil.isNotEmpty(boxPkgOrderDOS)) {
                    List<Long> orderIds = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toList());
                    List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, orderIds);
                    orderDOS.stream().forEach(item -> {
                        BoxLoadInfoCreateReqVO boxLoadInfoCreateReqVO = new BoxLoadInfoCreateReqVO();
                        boxLoadInfoCreateReqVO.setShipmentId(shipmentId);
                        boxLoadInfoCreateReqVO.setOrderNo(item.getOrderNo());
                        batchCreate(boxLoadInfoCreateReqVO);
                    });
                    return 0L;
                }
            } else {
                BoxMergePkgDO boxMergePkgDO1 = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
                        .eq(BoxMergePkgDO::getPkgNum, orderNumCode)
                );
                if (boxMergePkgDO1 != null) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.MERGE_PKG_NOT_IN_BOX);
                } else {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }
            }
        }

        //查找这个是否已经装了

        BoxLoadInfoDO existDO = getOne(new LambdaQueryWrapper<BoxLoadInfoDO>()
                .eq(BoxLoadInfoDO::getOrderNumCode, orderNumCode)
        );
        if(existDO != null) {
            throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_AIR_LOAD_EXIST);
        }

        Long orderId = searchOrderIdByOrderNumCode(orderNumCode, orderIdList, orderDOList);

        if(orderId == null) {
            int index = orderNumCode.lastIndexOf("-");
            String orderNumber = orderNumCode.substring(0, index);
            OrderDO orderDO = orderService.selectOne(OrderDO::getOrderNo, orderNumber);
            if (orderDO == null) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            } else {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.ORDER_NOT_IN_BOX);
            }
        } else {
            Long selectCount = boxPkgOrderService.selectCount(BoxPkgOrderDO::getOrderId, orderId);
            if (selectCount != null && selectCount > 0L) {
                throw exception(ORDER_IN_MERGE_PKG);
            }
        }

        boxLoadInfo.setOrderId(orderId);

        boxLoadInfoMapper.insert(boxLoadInfo);

        //修改为装柜中
//        updateInCabinet(createReqVO.getShipmentId());
        // 返回

//        boxService.addInstallCompleteLog(shipmentId);

//        updateInCabinet(shipmentId);

//        updateCabinetDate(shipmentId);

        return boxLoadInfo.getId();
    }

    /**
     * 更新装柜和最后装柜时间
     * @param shipmentId 出货单ID
     */
    private void updateCabinetDate(Long shipmentId) {
        //装柜表单新增装柜时间
        List<BoxCabinetDO> boxCabinetDOList = boxCabinetService.list(new LambdaQueryWrapper<BoxCabinetDO>()
                .eq(BoxCabinetDO::getShipmentId, shipmentId)
                .orderByAsc(BoxCabinetDO::getId)
        );
        if(CollectionUtil.isEmpty(boxCabinetDOList)) {
            //没有装柜表单，则新建一个
            Date ldBoxTime = new Date();
            BoxCabinetDO boxCabinetDO = new BoxCabinetDO();
            boxCabinetDO.setShipmentId(shipmentId);
            boxCabinetDO.setLdBoxTime(ldBoxTime);
            boxCabinetDO.setLastLdBoxTime(ldBoxTime);
            boxCabinetService.save(boxCabinetDO);
        } else {
            //有的话，检查装柜时间是否存在，不存在则更新
            BoxCabinetDO boxCabinetDO = boxCabinetDOList.get(0);
            Date ldBoxTime = boxCabinetDO.getLdBoxTime();
            if(ldBoxTime == null) {
                ldBoxTime = new Date();
                boxCabinetDO.setLdBoxTime(ldBoxTime);
                boxCabinetDO.setLastLdBoxTime(ldBoxTime);
                boxCabinetService.updateById(boxCabinetDO);
            } else {
                //查看是否有反审，有的话则更新最后装柜时间
                List<BoxApprovalDO> cabinetUmpireApprovalList = boxApprovalService.list(new LambdaQueryWrapper<BoxApprovalDO>()
                    .eq(BoxApprovalDO::getShipmentId, shipmentId)
                        .eq(BoxApprovalDO::getApprovalType, SEALING_CABINET_UMPIRE.getType())
                        .eq(BoxApprovalDO::getApprovalStatus, ApprovalResultStatusEnum.pass.getValue())
                );
                if(CollectionUtil.isNotEmpty(cabinetUmpireApprovalList)) {
                    //有封柜反审,修改最后装柜时间
                    boxCabinetDO.setLastLdBoxTime(new Date());
                    boxCabinetService.updateById(boxCabinetDO);
                }
            }
        }
    }

    private Long searchOrderIdByOrderNumCode(String orderNumCode, Set<Long> orderIdList, List<OrderDO> orderDOList) {
        Long orderId = null;

        OrderLabelQueryVO queryVO = new OrderLabelQueryVO();
        queryVO.setOrderIdList(orderIdList);

        List<OrderLabelDO> labelDOList = orderLabelService.getLabelList(queryVO);
        Map<Long, List<OrderLabelDO>> labelMap = labelDOList.stream()
                .collect(Collectors.groupingBy(OrderLabelDO::getOrderId));

        for (OrderDO orderDO : orderDOList) {
            List<OrderLabelDO> orderLabelDOList = labelMap.get(orderDO.getOrderId());
            List<String> tagList = getOrderTagList(orderDO, orderLabelDOList);
            if(tagList.contains(orderNumCode)) {
                orderId = orderDO.getOrderId();
            }
        }
        return orderId;
    }

    @Override
    public Long getOrderIdByOrderNumCode(Long shipmentId, String orderNumCode) {
        //根据shipment和secId，找到这个出货单部分下的所有订单商品
        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        Set<Long> orderIdList = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                .collect(Collectors.toSet());

        List<OrderDO> orderDOList = orderQueryService.getOrderList(orderIdList);

        return searchOrderIdByOrderNumCode(orderNumCode, orderIdList, orderDOList);
    }

    private void updateInCabinet(Long shipmentId) {
        //修改为装柜中
        BoxDO boxDO = boxService.getBox(shipmentId);
        Integer ldStatus = boxDO.getLdStatus();
        if(ldStatus == LdStatusEnum.NO_CABINET.getLdStatus()) {
            boxDO.setLdStatus(LdStatusEnum.IN_CABINET.getLdStatus());
            boxService.updateById(boxDO);
        }
    }

    /**
     * 根据shipment和secId，找到这个出货单部分下的所有订单商品
     * @param shipmentId shipmentId
     * @param secId  secId
     * @return
     */
    private List<BoxPreloadGoodsDO> getPreloadGoods(Long shipmentId, Long secId) {
        BoxPreloadGoodsQueryVO queryVO = new BoxPreloadGoodsQueryVO();
        queryVO.setShipmentId(shipmentId);
        queryVO.setIsRemove(0);
        if(secId != null) {
            queryVO.setSecId(secId);
        }
        return boxPreloadGoodsService.getBoxPreloadGoodsList(queryVO);
    }

    private OrderDO getOrderByNo(String orderNo) {
        //根据订单编号找到订单
        OrderQueryVO orderQueryVO = new OrderQueryVO();
        orderQueryVO.setOrderNo(orderNo);
        List<OrderDO> orderList = orderQueryService.getOrderList(orderQueryVO);
        if(CollectionUtil.isEmpty(orderList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        return orderList.get(0);
    }


    /**
     * 根据订单编号，批量装柜
     * @param createReqVO 创建信息
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long batchCreate(@Valid BoxLoadInfoCreateReqVO createReqVO) {
        boxService.checkApproval(createReqVO.getShipmentId(), null);

        List<BoxLoadInfoDO> loadInfoDOList = new ArrayList<>();

        String orderNos = createReqVO.getOrderNo();
        Long shipmentId = createReqVO.getShipmentId();

        //支持批量多订单
        String[] orderNoArray = orderNos.split(",");

        for (String orderNo : orderNoArray) {
            OrderDO orderDO = getOrderByNo(orderNo);
            Long orderId = orderDO.getOrderId();

            //根据shipment,找到这个出货单下的所有订单商品
            List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
            if (CollectionUtil.isEmpty(goodsDOList)) {
                throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
            }

            //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
            Set<Long> orderIdList = goodsDOList.stream()
                    .map(BoxPreloadGoodsDO::getOrderId)
                    .collect(Collectors.toSet());

            if (!orderIdList.contains(orderId)) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }

            //删除以装柜的编号
            boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .eq(BoxLoadInfoDO::getOrderId, orderId)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );

            //找到这个订单所有需要装柜的编号
            List<String> tagList = getOrderTagList(orderDO);
            for (int i = 0; i < tagList.size(); i++) {
                String tag = tagList.get(i);
                BoxLoadInfoDO boxLoadInfo = new BoxLoadInfoDO();
                boxLoadInfo.setOrderId(orderId);
                boxLoadInfo.setOrderNumCode(tag);
                boxLoadInfo.setShipmentId(shipmentId);
                loadInfoDOList.add(boxLoadInfo);
            }
            saveBatch(loadInfoDOList);
        }
        boxService.addInstallCompleteLog(shipmentId);
        updateCabinetDate(shipmentId);
        return 0L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long batchAirCreate(@Valid BoxLoadInfoCreateReqVO createReqVO) {
        boxService.checkApproval(createReqVO.getShipmentId(), null);

        List<BoxLoadInfoDO> loadInfoDOList = new ArrayList<>();

        String orderNos = createReqVO.getOrderNo();
        Long shipmentId = createReqVO.getShipmentId();

        //支持批量多订单
        String[] orderNoArray = orderNos.split(",");

        for (String orderNo : orderNoArray) {
            OrderDO orderDO = getOrderByNo(orderNo);
            Long orderId = orderDO.getOrderId();

            //根据shipment,找到这个出货单下的所有订单商品
            List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
            if (CollectionUtil.isEmpty(goodsDOList)) {
                throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
            }

            //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
            Set<Long> orderIdList = goodsDOList.stream()
                    .map(BoxPreloadGoodsDO::getOrderId)
                    .collect(Collectors.toSet());

            if (!orderIdList.contains(orderId)) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }

            //删除以装柜的编号
            boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .eq(BoxLoadInfoDO::getOrderId, orderId)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );

            //找到这个订单所有需要装柜的编号
            List<String> tagList = getOrderTagList(orderDO);
            for (int i = 0; i < tagList.size(); i++) {
                String tag = tagList.get(i);
                BoxLoadInfoDO boxLoadInfo = new BoxLoadInfoDO();
                boxLoadInfo.setOrderId(orderId);
                boxLoadInfo.setOrderNumCode(tag);
                boxLoadInfo.setShipmentId(shipmentId);
                loadInfoDOList.add(boxLoadInfo);
            }
            saveBatch(loadInfoDOList);
        }
//        boxService.addInstallCompleteLog(shipmentId);
//        updateCabinetDate(shipmentId);
        return 0L;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createByShipment(Long shipmentId) {
        Set<Long> orderIdSets = boxPreloadGoodsService.getShipOrderIdList(shipmentId);
        if(CollectionUtil.isEmpty(orderIdSets)) return;

        List<BoxLoadInfoDO> loadInfoDOList = new ArrayList<>();
        List<OrderDO> orderList = orderQueryService.getOrderList(orderIdSets);

        for (OrderDO orderDO : orderList) {
            //删除以装柜的编号
            boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .eq(BoxLoadInfoDO::getOrderId, orderDO.getOrderId())
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );

            //找到这个订单所有需要装柜的编号
            List<String> tagList = getOrderTagList(orderDO);
            for (String tag : tagList) {
                BoxLoadInfoDO boxLoadInfo = new BoxLoadInfoDO();
                boxLoadInfo.setOrderId(orderDO.getOrderId());
                boxLoadInfo.setOrderNumCode(tag);
                boxLoadInfo.setShipmentId(shipmentId);
                loadInfoDOList.add(boxLoadInfo);
            }
        }
        saveBatch(loadInfoDOList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void unloadAllAirBoxInfo(BoxLoadInfoCreateReqVO createReqVO) {
        boxService.checkApproval(createReqVO.getShipmentId(), null);

        //根据shipment,找到这个出货单下的所有订单商品
        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(createReqVO.getShipmentId(), null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        BoxLoadInfoDO boxLoadInfoDO = new BoxLoadInfoDO();
        boxLoadInfoDO.setIsUnload(1);
        Date ulBoxTime = createReqVO.getUnloadTime();
        boxLoadInfoDO.setUnloadTime(ulBoxTime);
        boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                .eq(BoxLoadInfoDO::getShipmentId, createReqVO.getShipmentId())
        );
        Set<Long> orderIdSet = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                        .collect(Collectors.toSet());
        //修改订单状态为已到仓
       updateOrderStatusAir(orderIdSet, ulBoxTime, createReqVO.getShipmentId());
//       boxService.updateUlBoxTime(createReqVO.getShipmentId(), ulBoxTime);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void airUnloadBoxInfo(BoxLoadInfoCreateReqVO unloadReq) {
        boxService.checkApproval(unloadReq.getShipmentId(), null);

        Date ulBoxTime = unloadReq.getUnloadTime();
        String orderNos = unloadReq.getOrderNo();
        Long shipmentId = unloadReq.getShipmentId();

        BoxLoadInfoDO boxLoadInfoDO = new BoxLoadInfoDO();
        boxLoadInfoDO.setIsUnload(1);
        boxLoadInfoDO.setUnloadTime(ulBoxTime);

        if(StringUtils.isNotEmpty(orderNos)) {
            //批量卸柜
            String[] orderNoArray = orderNos.split(",");
            for (String orderNo : orderNoArray) {
                OrderDO orderDO = getOrderByNo(orderNo);
                Long orderId = orderDO.getOrderId();

                //根据shipment,找到这个出货单下的所有订单商品
                List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
                if(CollectionUtil.isEmpty(goodsDOList)) {
                    throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
                }

                //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
                Set<Long> orderIdList = goodsDOList.stream()
                        .map(BoxPreloadGoodsDO::getOrderId)
                        .collect(Collectors.toSet());

                if(!orderIdList.contains(orderId)) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }

                List<OrderLocationCreateReqVO> orderLocationList = unloadReq.getOrderLocationList();
                if(CollectionUtil.isNotEmpty(orderLocationList)) {
                    List<String> orderLocationStrList = new ArrayList<>();
                    for (OrderLocationCreateReqVO locationCreateReqVO : orderLocationList) {
                        Long wareId = locationCreateReqVO.getWareId();
                        Long areaId = locationCreateReqVO.getAreaId();
                        Long locationId = locationCreateReqVO.getLocationId();
                        orderLocationStrList.add(wareId + "-" + areaId + "-" + locationId);
                    }
                    String locationStr = Joiner.on(",").join(orderLocationStrList);
                    boxLoadInfoDO.setUnloadPosition(locationStr);
                }

                boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                        .eq(BoxLoadInfoDO::getOrderId, orderId)
                        .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
                );
                updateOrderStatusAir(Collections.singletonList(orderId), ulBoxTime, shipmentId);
            }
        } else {
            //单个卸柜
            String orderNumCode = unloadReq.getOrderNumCode();
            String[] orderNumCodes = orderNumCode.split(",");
            for (String code : orderNumCodes) {
                BoxLoadInfoDO existDO = getOne(new LambdaQueryWrapper<BoxLoadInfoDO>()
                        .eq(BoxLoadInfoDO::getOrderNumCode, code)
                );
                if(existDO == null) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }
            }
            List<String> codeList = Arrays.asList(orderNumCodes);

            // 到仓
            boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderNumCode, codeList)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );
            //opt
            List<BoxLoadInfoDO> loadInfoDOList = boxLoadInfoMapper.selectList(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderNumCode, codeList)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );
            Set<Long> orderIdSet = loadInfoDOList.stream()
                    .map(BoxLoadInfoDO::getOrderId)
                    .collect(Collectors.toSet());
            //查看这些订单是否已经到仓
            List<BoxLoadInfoDO> orderLoadInfoList = boxLoadInfoMapper.selectList(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderId, orderIdSet)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );
            Map<Long, List<BoxLoadInfoDO>> orderLoadMap = orderLoadInfoList.stream()
                    .collect(Collectors.groupingBy(BoxLoadInfoDO::getOrderId));
            List<Long> unloadOrderList = new ArrayList<>();
            for (Map.Entry<Long, List<BoxLoadInfoDO>> entry: orderLoadMap.entrySet()) {
                Long orderId = entry.getKey();
                List<BoxLoadInfoDO> valueList = entry.getValue();
                long count = valueList.stream()
                        .filter(t -> t.getIsUnload() == 0)
                        .count();
                if(count == 0) {
                    unloadOrderList.add(orderId);
                }
            }
            if(CollectionUtil.isNotEmpty(unloadOrderList)) {
                updateOrderStatusAir(unloadOrderList, ulBoxTime, shipmentId);
            }
        }
//        boxService.updateUlBoxTime(shipmentId, ulBoxTime);
    }

    private void updateOrderStatusAir(Collection<Long> orderIdSet, Date ulBoxTime, Long shipmentId) {
        //校验这些订单是否已经到港，已经清关
        checkArrivalAndClearance(orderIdSet, shipmentId);

        boxService.updateOrderStatusByOrderIdAndTransportType(orderIdSet, OrderStatusEnum.UNLOADED_CABINET.getValue(),
                OrderStatusMsgEnum.UNLOADED_CABINET.getInWarehouseState(),
                BoxAirStatusEnum.TO_WAREHOUSED.getStatus(), null, null, ulBoxTime, null, Integer.parseInt(TransportTypeEnum.AIR.getType()));
        boxService.addOrderLog(orderIdSet, OrderShipmentLog.CABINET_WAREHOUSE, "");
    }

    private void updateOrderStatus(Collection<Long> orderIdSet, Date ulBoxTime, Long shipmentId) {
        BoxDO boxDO = boxService.getById(shipmentId);
        boxService.updateOrderStatusByOrderIdAndTransportType(orderIdSet, OrderStatusEnum.UNLOADED_CABINET.getValue(),
                OrderStatusMsgEnum.UNLOADED_CABINET.getInWarehouseState(),
                BoxStatusEnum.UNLOADED.getStatus(), null, null, ulBoxTime, null, Integer.parseInt(boxDO.getTransportType()));
        boxService.addOrderLog(orderIdSet, OrderShipmentLog.CABINET_WAREHOUSE, "");
    }

    /**
     * 校验订单是否已经到港和清关
     * @param orderIdSet  到仓订单
     */
    private void checkArrivalAndClearance(Collection<Long> orderIdSet, Long shipmentId) {
        List<OrderDO> orderDOList = orderQueryService.getOrderList(orderIdSet);
        Map<Long, String> orderNoMap = orderDOList.stream()
                .collect(Collectors.toMap(OrderDO::getOrderId, OrderDO::getOrderNo));

        List<BoxArrivalOrderDO> arrivalOrderDOList = boxArrivalOrderService.list(new LambdaQueryWrapper<BoxArrivalOrderDO>()
                .eq(BoxArrivalOrderDO::getShipmentId, shipmentId)
        );
        List<Long> arrivalOrderIdList = arrivalOrderDOList.stream()
                .map(BoxArrivalOrderDO::getOrderId)
                .collect(Collectors.toList());

        List<BoxClearanceOrderDO> clearanceOrderDOList = boxClearanceOrderService.list(new LambdaQueryWrapper<BoxClearanceOrderDO>()
            .eq(BoxClearanceOrderDO::getShipmentId, shipmentId)
        );
        List<Long> clearanceOrderIdList = clearanceOrderDOList.stream()
                .map(BoxClearanceOrderDO::getOrderId)
                .collect(Collectors.toList());

        // 找出未到港并且未清关的订单ID
        List<Long> notArrivedAndNotClearedIds = orderIdSet.stream()
                .filter(orderId -> !arrivalOrderIdList.contains(orderId) && !clearanceOrderIdList.contains(orderId))
                .collect(Collectors.toList());

        // 找出未到港的订单ID，排除未到港并且未清关的订单ID
        List<Long> notArrivedIds = orderIdSet.stream()
                .filter(orderId -> !notArrivedAndNotClearedIds.contains(orderId) && !arrivalOrderIdList.contains(orderId))
                .collect(Collectors.toList());

        // 找出未清关的订单ID，排除未到港并且未清关的订单ID
        List<Long> notClearedIds = orderIdSet.stream()
                .filter(orderId -> !notArrivedAndNotClearedIds.contains(orderId) && !clearanceOrderIdList.contains(orderId))
                .collect(Collectors.toList());

        List<String> tipList = new ArrayList<>();
        if(CollectionUtil.isNotEmpty(notArrivedAndNotClearedIds)) {
            List<String> orderNoList = new ArrayList<>();
            for (Long id : notArrivedAndNotClearedIds) {
                orderNoList.add(orderNoMap.get(id));
            }
            String tip = "未到港未清关订单:" + Joiner.on(",").join(orderNoList);
            tipList.add(tip);
        }

        if(CollectionUtil.isNotEmpty(notArrivedIds)) {
            List<String> orderNoList = new ArrayList<>();
            for (Long id : notArrivedIds) {
                orderNoList.add(orderNoMap.get(id));
            }
            String tip = "未到港订单:" + Joiner.on(",").join(orderNoList);
            tipList.add(tip);
        }

        if(CollectionUtil.isNotEmpty(notClearedIds)) {
            List<String> orderNoList = new ArrayList<>();
            for (Long id : notClearedIds) {
                orderNoList.add(orderNoMap.get(id));
            }
            String tip = "未清关订单:" + Joiner.on(",").join(orderNoList);
            tipList.add(tip);
        }
        if(CollectionUtil.isNotEmpty(tipList)) {
            String message = "本次操作存在未到港货未清关订单，无法完成到仓。<br>" + Joiner.on("<br>").join(tipList);
            throw new ServiceException(500, message);
        }
    }

    private List<String> getOrderTagList(OrderDO orderDO, List<OrderLabelDO> orderLabelList) {
        //从OrderLabel里面查找订单的箱号
        String orderNo = orderDO.getOrderNo();
        List<String> tagList = new ArrayList<>();
        int start;
        int end;
        if(CollectionUtil.isEmpty(orderLabelList)) {
            //没有存箱号，从订单的总箱数里面取
            int totalNum = orderDO.getSumNum();//总箱数
            start = 1;
            end = totalNum;
            for (int i = start; i <= end; i++) {
                String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                tagList.add(tag);
            }
        } else {
            orderLabelList.sort(Comparator.comparing(OrderLabelDO::getId));
            for (OrderLabelDO orderLabelDO : orderLabelList) {
                start = orderLabelDO.getStart();
                end = orderLabelDO.getEnd();
                for (int i = start; i <= end; i++) {
                    String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                    tagList.add(tag);
                }
            }
        }

        return tagList;
    }

    private List<String> getOrderTagList(OrderDO orderDO) {
        //从OrderLabel里面查找订单的箱号
        Long orderId = orderDO.getOrderId();
        OrderLabelQueryVO queryVO = new OrderLabelQueryVO();
        queryVO.setOrderId(orderId);
        List<OrderLabelDO> orderLabelList = orderLabelService.getLabelList(queryVO);
        return getOrderTagList(orderDO, orderLabelList);
    }

    @Override
    public void updateBoxLoadInfo(BoxLoadInfoUpdateReqVO updateReqVO) {
        // 更新
        BoxLoadInfoDO updateObj = BoxLoadInfoConvert.INSTANCE.convert(updateReqVO);
        boxLoadInfoMapper.updateById(updateObj);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteBoxLoadInfo(BoxLoadInfoCreateReqVO deleteReq) {
        boxService.checkApproval(deleteReq.getShipmentId(), null);

        String orderNos = deleteReq.getOrderNo();
        Long shipmentId = deleteReq.getShipmentId();
        if(StringUtils.isNotEmpty(orderNos)) {
            //批量删除
            String[] orderNoArray = orderNos.split(",");
            for (String orderNo : orderNoArray) {
                OrderDO orderDO = getOrderByNo(orderNo);
                Long orderId = orderDO.getOrderId();

                //根据shipment,找到这个出货单下的所有订单商品
                List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
                if(CollectionUtil.isEmpty(goodsDOList)) {
                    throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
                }

                //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
                Set<Long> orderIdList = goodsDOList.stream()
                        .map(BoxPreloadGoodsDO::getOrderId)
                        .collect(Collectors.toSet());

                if(!orderIdList.contains(orderId)) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }

                boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                        .eq(BoxLoadInfoDO::getOrderId, orderId)
                        .eq(BoxLoadInfoDO::getShipmentId, deleteReq.getShipmentId())
                );

                boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_INFO_REMOVE_BATCH, "");
            }
        } else {
            BoxLoadInfoDO existDO = getOne(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .eq(BoxLoadInfoDO::getOrderNumCode, deleteReq.getOrderNumCode())
            );
            if(existDO == null) {
                //扫描的是空运的合包箱号
                BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
                        .eq(BoxMergePkgDO::getShipmentId, shipmentId)
                        .eq(BoxMergePkgDO::getPkgNum, deleteReq.getOrderNumCode())
                );
                if (boxMergePkgDO != null) {
                    List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapper<BoxPkgOrderDO>()
                            .eq(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId())
                    );
                    if (CollectionUtil.isNotEmpty(boxPkgOrderDOS)) {
                        List<Long> orderIds = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toList());
                        List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, orderIds);
                        String delOrderNos = orderDOS.stream().map(OrderDO::getOrderNo).collect(Collectors.joining(","));
                        BoxLoadInfoCreateReqVO boxLoadInfoCreateReqVO = new BoxLoadInfoCreateReqVO();
                        boxLoadInfoCreateReqVO.setShipmentId(shipmentId);
                        boxLoadInfoCreateReqVO.setOrderNo(delOrderNos);
                        deleteBoxLoadInfo(boxLoadInfoCreateReqVO);
                        return;
                    }
                } else {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }
            }
            // 删除
            boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .eq(BoxLoadInfoDO::getOrderNumCode, deleteReq.getOrderNumCode())
                    .eq(BoxLoadInfoDO::getShipmentId, deleteReq.getShipmentId())
            );
            boxService.addOrderLog(Collections.singletonList(existDO.getOrderId()), OrderShipmentLog.CABINET_INFO_REMOVE_SINGLE, "箱号:" + deleteReq.getOrderNumCode());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeBox(BoxLoadInfoRemoveReqVO removeReqVO) {
        Long shipmentId = removeReqVO.getShipmentId();
        Long orderId = removeReqVO.getOrderId();
        //校验是否在审核中
        boxService.checkApproval(shipmentId, null);

        boxPreloadGoodsService.delCheckStatus(shipmentId);

        Collection<Long> orderIdList = Collections.singletonList(orderId);

        orderIdList = boxPreloadGoodsService.checkDeleteOrder(shipmentId, removeReqVO.getDeleteType(), orderIdList);

        boxPreloadGoodsService.deleteOrderList(shipmentId, orderIdList, 3, false);

        boxService.addOrderLog(Collections.singletonList(orderId), OrderShipmentLog.CABINET_REMOVE, "");
    }

    @Override
    public CommonResult<BoxLoadInfoOrderVO> orderTagList(Long orderId) {
        //找到这个订单所有需要装柜的编号
        OrderDO orderDO = orderService.getById(orderId);
        List<String> tagList = getOrderTagList(orderDO);
        int tagSize = tagList.size();
        //找到这个订单所有已装柜的编号
        List<BoxLoadInfoDO> existList = boxLoadInfoMapper.selectList(new LambdaQueryWrapper<BoxLoadInfoDO>()
                .in(BoxLoadInfoDO::getOrderId, orderId)
        );
        List<String> loadList = existList.stream()
                .map(BoxLoadInfoDO::getOrderNumCode)
                .collect(Collectors.toList());
        List<String> unloadCabinetList = existList.stream()
                .filter(t -> t.getIsUnload() == 1)
                .map(BoxLoadInfoDO::getOrderNumCode)
                .collect(Collectors.toList());
        tagList.removeAll(loadList);
        BoxLoadInfoOrderVO boxLoadInfoOrderVO = new BoxLoadInfoOrderVO();
        boxLoadInfoOrderVO.setLoadList(loadList);
        boxLoadInfoOrderVO.setUnloadCabinetList(unloadCabinetList);
        if(loadList.size() >= tagSize) {
            //拆单后，已装的标签号不正确，和orderLabel表标签不匹配，这里先按数量兼容
            boxLoadInfoOrderVO.setUnLoadList(new ArrayList<>());
        } else {
            boxLoadInfoOrderVO.setUnLoadList(tagList);
        }
        return CommonResult.success(boxLoadInfoOrderVO);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void boxUpdate(BoxInfoUpdateReq boxInfoUpdateReq) {
        Long shipmentId = boxInfoUpdateReq.getShipmentId();
        boxService.checkApproval(shipmentId, null);

        Long cabinetId = boxInfoUpdateReq.getCabinetId();
        //找到这个柜子的详细信息
        //查找出这个出货单的箱子体积和重量
        CabinetDO cabinetDO = cabinetService.getCabinet(cabinetId);
        //根据shipment,找到这个出货单下的所有订单商品
        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        //找出所有订单项
        List<Long> orderItemIdList = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderItemId)
                .collect(Collectors.toList());
        List<OrderItemDO> orderItemDOList = new ArrayList<>();
        if(CollectionUtil.isNotEmpty(orderItemIdList)) {
            orderItemDOList = orderItemService.getOrderItemList(orderItemIdList);
        }
        //统计已装数量体积等信息
        BoxGoodsStatistics boxGoodsStatistics = boxPreloadGoodsService.calculationStatistics(orderItemDOList);
        //计算剩余的体积和重量
        BigDecimal remainVolume = cabinetDO.getVolume().subtract(boxGoodsStatistics.getVolume()).setScale(2, BigDecimal.ROUND_DOWN);
        BigDecimal remainWeight = cabinetDO.getWeight().subtract(boxGoodsStatistics.getWeight()).setScale(2, BigDecimal.ROUND_DOWN);

        if(remainVolume.compareTo(BigDecimal.ZERO) < 0) {
            throw exception(BOX_VOLUME_EXCEED);
        }
        if(remainWeight.compareTo(BigDecimal.ZERO) < 0) {
            throw exception(BOX_WEIGHT_EXCEED);
        }

        BoxDO boxDO = boxService.getBox(shipmentId);
        boxDO.setCabinetId(cabinetId);
        boxDO.setStartWarehouseId(boxInfoUpdateReq.getStartWarehouseId());
        if(StringUtils.isNotEmpty(boxInfoUpdateReq.getCubNo())) {
            boxDO.setCubNo(boxInfoUpdateReq.getCubNo());

            boxService.updateTrailerCubNo(boxDO);

        }
        boxService.updateById(boxDO);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void unloadBoxInfo(BoxLoadInfoCreateReqVO unloadReq) {
        boxService.checkApproval(unloadReq.getShipmentId(), null);

        BoxLoadInfoDO boxLoadInfoDO = new BoxLoadInfoDO();
        boxLoadInfoDO.setIsUnload(1);

        Date ulBoxTime = unloadReq.getUnloadTime();
        String orderNos = unloadReq.getOrderNo();
        Long shipmentId = unloadReq.getShipmentId();
        boxLoadInfoDO.setUnloadTime(ulBoxTime);
        if(StringUtils.isNotEmpty(orderNos)) {
            //批量卸柜
            String[] orderNoArray = orderNos.split(",");
            for (String orderNo : orderNoArray) {
                OrderDO orderDO = getOrderByNo(orderNo);
                Long orderId = orderDO.getOrderId();

                //根据shipment,找到这个出货单下的所有订单商品
                List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
                if(CollectionUtil.isEmpty(goodsDOList)) {
                    throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
                }

                //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
                Set<Long> orderIdList = goodsDOList.stream()
                        .map(BoxPreloadGoodsDO::getOrderId)
                        .collect(Collectors.toSet());

                if(!orderIdList.contains(orderId)) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }

                List<OrderLocationCreateReqVO> orderLocationList = unloadReq.getOrderLocationList();
                if(CollectionUtil.isNotEmpty(orderLocationList)) {
                    List<String> orderLocationStrList = new ArrayList<>();
                    for (OrderLocationCreateReqVO locationCreateReqVO : orderLocationList) {
                        Long wareId = locationCreateReqVO.getWareId();
                        Long areaId = locationCreateReqVO.getAreaId();
                        Long locationId = locationCreateReqVO.getLocationId();
                        orderLocationStrList.add(wareId + "-" + areaId + "-" + locationId);
                    }
                    String locationStr = Joiner.on(",").join(orderLocationStrList);
                    boxLoadInfoDO.setUnloadPosition(locationStr);
                }

                boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                        .eq(BoxLoadInfoDO::getOrderId, orderId)
                        .eq(BoxLoadInfoDO::getShipmentId, unloadReq.getShipmentId())
                );

                updateOrderStatus(Collections.singletonList(orderId), ulBoxTime, shipmentId);
            }
        } else {
            //单个卸柜
            String orderNumCode = unloadReq.getOrderNumCode();
            String[] orderNumCodes = orderNumCode.split(",");
            for (String code : orderNumCodes) {
                BoxLoadInfoDO existDO = getOne(new LambdaQueryWrapper<BoxLoadInfoDO>()
                        .eq(BoxLoadInfoDO::getOrderNumCode, code)
                );
                if(existDO == null) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }
//                boxService.addOrderLog(Collections.singletonList(existDO.getOrderId()), OrderShipmentLog.CABINET_INFO_REMOVE_BATCH, "箱号:" + code);
            }
            List<String> codeList = Arrays.asList(orderNumCodes);

            // 卸柜
            boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderNumCode, codeList)
                    .eq(BoxLoadInfoDO::getShipmentId, unloadReq.getShipmentId())
            );

            //opt
            List<BoxLoadInfoDO> loadInfoDOList = boxLoadInfoMapper.selectList(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderNumCode, codeList)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );
            Set<Long> orderIdSet = loadInfoDOList.stream()
                    .map(BoxLoadInfoDO::getOrderId)
                    .collect(Collectors.toSet());
            //查看这些订单是否已经到仓
            List<BoxLoadInfoDO> orderLoadInfoList = boxLoadInfoMapper.selectList(new LambdaQueryWrapper<BoxLoadInfoDO>()
                    .in(BoxLoadInfoDO::getOrderId, orderIdSet)
                    .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
            );
            Map<Long, List<BoxLoadInfoDO>> orderLoadMap = orderLoadInfoList.stream()
                    .collect(Collectors.groupingBy(BoxLoadInfoDO::getOrderId));
            List<Long> unloadOrderList = new ArrayList<>();
            for (Map.Entry<Long, List<BoxLoadInfoDO>> entry: orderLoadMap.entrySet()) {
                Long orderId = entry.getKey();
                List<BoxLoadInfoDO> valueList = entry.getValue();
                long count = valueList.stream()
                        .filter(t -> t.getIsUnload() == 0)
                        .count();
                if(count == 0) {
                    unloadOrderList.add(orderId);
                }
            }
            if(CollectionUtil.isNotEmpty(unloadOrderList)) {
                updateOrderStatus(unloadOrderList, ulBoxTime, shipmentId);
            }
        }
//        boxService.updateUlBoxTime(shipmentId, ulBoxTime);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void unloadAllBoxInfo(BoxLoadInfoCreateReqVO unloadReq) {
        Long shipmentId = unloadReq.getShipmentId();
        boxService.checkApproval(shipmentId, null);

        //根据shipment,找到这个出货单下的所有订单商品
        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        Date ulBoxTime = unloadReq.getUnloadTime();
        BoxLoadInfoDO boxLoadInfoDO = new BoxLoadInfoDO();
        boxLoadInfoDO.setIsUnload(1);
        boxLoadInfoMapper.update(boxLoadInfoDO, new LambdaQueryWrapper<BoxLoadInfoDO>()
                .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
        );
        Set<Long> orderIdSet = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                .collect(Collectors.toSet());
//        boxService.addOrderLog(new ArrayList<>(orderIdSet), OrderShipmentLog.CABINET_UNLOAD, "一键卸柜");
        //修改订单状态为已到仓
        updateOrderStatus(orderIdSet, ulBoxTime, shipmentId);
//        boxService.updateUlBoxTime(shipmentId, ulBoxTime);
    }

    @Override
    public void batchDelete(List<String> orderNumCodeList, Long shipmentId) {
        // 删除
        boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                .in(BoxLoadInfoDO::getOrderNumCode, orderNumCodeList)
                .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
        );
    }

    @Override
    public void deleteLoadInfo(Long shipmentId, List<Long> orderIdList) {
        boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
            .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
                .in(BoxLoadInfoDO::getOrderId, orderIdList)
        );
    }

    @Override
    public BoxLoadInfoDO getBoxLoadInfo(Long id) {
        return boxLoadInfoMapper.selectById(id);
    }

    @Override
    public List<BoxLoadInfoDO> getBoxLoadInfoList(Collection<Long> ids) {
        return boxLoadInfoMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxLoadInfoDO> getBoxLoadInfoPage(BoxLoadInfoQueryVO query, PageVO page) {
        return boxLoadInfoMapper.selectPage(page, query);
    }

    @Override
    public List<BoxLoadInfoDO> getBoxLoadInfoList(BoxLoadInfoQueryVO query) {
        return boxLoadInfoMapper.selectList(query);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long externalLoad(BoxExternalLoadReq createReqVO) {
        boxService.checkApproval(createReqVO.getShipmentId(), null);

        Integer boxNum = createReqVO.getBoxNum();
        if(boxNum <= 0) {
            throw new ServiceException(500, "装柜数量需要大于0");
        }

        if(createReqVO.getCheckPwd() == 1) {
            String pwd = createReqVO.getPwd();
            if(StringUtils.isEmpty(pwd)) throw new ServiceException(500, "请输入密码");
            boxService.checkPwd(createReqVO.getPwd());
        }


        //首先删除这个订单的所有装箱单。
        Long orderId = createReqVO.getOrderId();
        Long shipmentId = createReqVO.getShipmentId();

        //根据shipment,找到这个出货单下的所有订单商品
        List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
        if(CollectionUtil.isEmpty(goodsDOList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }

        //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
        Set<Long> orderIdList = goodsDOList.stream()
                .map(BoxPreloadGoodsDO::getOrderId)
                .collect(Collectors.toSet());

        if(!orderIdList.contains(orderId)) {
            throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
        }

        //删除以装柜的编号
        boxLoadInfoMapper.delete(new LambdaQueryWrapper<BoxLoadInfoDO>()
                .eq(BoxLoadInfoDO::getOrderId, orderId)
                .eq(BoxLoadInfoDO::getShipmentId, shipmentId)
        );

        //装柜数量
        OrderDO orderDO = orderService.getById(orderId);

        List<String> tagList = getOrderTagList(orderDO);

        if(boxNum > tagList.size()) {
            throw new ServiceException(500, "装柜数量不能大于计划数量");
        }

        List<BoxLoadInfoDO> loadInfoDOList = new ArrayList<>();
        for (int i = 0; i < boxNum; i++) {
            String tag = tagList.get(i);
            BoxLoadInfoDO boxLoadInfo = new BoxLoadInfoDO();
            boxLoadInfo.setOrderId(orderId);
            boxLoadInfo.setOrderNumCode(tag);
            boxLoadInfo.setShipmentId(shipmentId);
            loadInfoDOList.add(boxLoadInfo);
        }
        saveBatch(loadInfoDOList);

        boxService.addInstallCompleteLog(shipmentId);

        updateCabinetDate(shipmentId);
        return 1L;
    }
}
