package cn.iocoder.yudao.module.shipment.service.boxOpLog;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.mybatis.core.enums.SqlConstants;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxOpLogDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxOpLogMapper;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.BoxApprovalBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxOpLog.BoxOpLogBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxOpLog.BoxOpLogQueryVO;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 出货操作日志 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxOpLogServiceImpl extends AbstractService<BoxOpLogMapper, BoxOpLogDO> implements BoxOpLogService {

    @Resource
    private BoxOpLogMapper boxOpLogMapper;

    @Resource
    @Lazy
    private BoxApprovalService boxApprovalService;


    @Override
    public Long createBoxOpLog(Long shipmentId, Integer opStatus, String statusNameZh, String statusNameEn, Long approvalId) {
//        BoxOpLogDO opLogDO = boxOpLogMapper.selectOne(new LambdaQueryWrapperX<BoxOpLogDO>()
//                .eq(BoxOpLogDO::getShipmentId, shipmentId)
//                .eq(BoxOpLogDO::getOpStatus, opStatus)
//                .last(SqlConstants.LIMIT1)
//        );

//        if(opLogDO == null) {
        BoxOpLogDO  opLogDO = new BoxOpLogDO();
//        }
        opLogDO.setShipmentId(shipmentId);
        opLogDO.setOpStatus(opStatus);
        opLogDO.setTitleZh(statusNameZh);
        opLogDO.setTitleEn(statusNameEn);
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        opLogDO.setOperator(loginUser.getUsername());
        opLogDO.setApprovalId(approvalId);
        saveOrUpdate(opLogDO);
        return opLogDO.getId();
    }

    @Override
    public Long createBoxOpLog(Long shipmentId, Integer opStatus, String statusNameZh, String statusNameEn) {
        return createBoxOpLog(shipmentId, opStatus, statusNameZh, statusNameEn, null);
    }

    @Override
    public BoxOpLogDO getBoxOpLog(Long id) {
        return boxOpLogMapper.selectById(id);
    }

    @Override
    public List<BoxOpLogDO> getBoxOpLogList(Collection<Long> ids) {
        return boxOpLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxOpLogDO> getBoxOpLogPage(BoxOpLogQueryVO query, PageVO page) {
        return boxOpLogMapper.selectPage(page, query);
    }

    @Override
    public List<BoxOpLogDO> getBoxOpLogList(BoxOpLogQueryVO query) {
        return boxOpLogMapper.selectList(query);
    }

    @Override
    public void matchBpmProcessId(List<BoxOpLogBackVO> resultList) {
        if(CollectionUtils.isNotEmpty(resultList)) {
            Set<Long> approvalIdList = resultList.stream()
                    .filter(t -> t.getApprovalId() != null && t.getApprovalId() > 0)
                    .map(BoxOpLogBackVO::getApprovalId)
                    .collect(Collectors.toSet());
            if(CollectionUtils.isEmpty(approvalIdList)) return;
            List<BoxApprovalDO> approvalDOList = boxApprovalService.listByIds(approvalIdList);
            Map<Long, String> approvalMap = approvalDOList.stream()
                    .collect(Collectors.toMap(BoxApprovalDO::getId, BoxApprovalDO::getBpmProcessId));
            for (BoxOpLogBackVO item : resultList) {
                String bpmProcessId = approvalMap.get(item.getApprovalId());
                item.setBpmProcessId(bpmProcessId);
            }
        }
    }
}
