package cn.iocoder.yudao.module.shipment.service.boxPkgOrder;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.*;

/**
 * 合包订单 Service 接口
 *
 * @author 江楠
 */
public interface BoxPkgOrderService extends IService<BoxPkgOrderDO> {

    /**
     * 创建合包订单
     * @param createReqVO 创建信息
     * @return 编号
     */
    String createBoxPkgOrder(@Valid BoxPkgOrderCreateReqVO createReqVO);

    /**
     * 更新合包订单
     * @param updateReqVO 更新信息
     */
    void updateBoxPkgOrder(@Valid BoxPkgOrderUpdateReqVO updateReqVO);

    /**
     * 删除合包订单
     * @param id 编号
     */
    void deleteBoxPkgOrder(Long id);

    /**
     * 获得合包订单
     * @param id 编号
     * @return 合包订单
     */
    BoxPkgOrderDO getBoxPkgOrder(Long id);

    /**
     * 获得合包订单列表
     * @param ids 编号
     * @return 合包订单列表
     */
    List<BoxPkgOrderDO> getBoxPkgOrderList(Collection<Long> ids);

    /**
     * 获得合包订单分页
     * @param page 分页查询
     * @param query 查询
     * @return 合包订单分页
     */
    PageResult<BoxPkgOrderDO> getBoxPkgOrderPage(BoxPkgOrderQueryVO query, PageVO page);

    /**
     * 获得合包订单列表, 用于 Excel 导出
     * @param query 查询
     * @return 合包订单列表
     */
    List<BoxPkgOrderDO> getBoxPkgOrderList(BoxPkgOrderQueryVO query);

    String createBatch(List<BoxPkgOrderCreateReqVO> createReqVO);

    void deleteRelate(Long pkgId, Long orderId);

    void deleteBatchRelate(List<BoxPkgOrderQueryVO> list);

    void deleteRelateByOrderIds(Long shipmentId,List<Long> orderIds);

    Map<String, Integer> getMergeOrderQualityByPkgIds(List<Long> pkgIds);

    String scanOrderNoCreate(BoxPkgOrderScanCodeVO createReqVO);
}
