package cn.iocoder.yudao.module.shipment.service.boxPreloadGoods;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.vo.order.OrderPreloadReq;
import cn.iocoder.yudao.module.order.vo.orderLocation.OrderLocationCreateReqVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadGoodsDO;
import cn.iocoder.yudao.module.shipment.vo.box.BoxGoodsStatistics;
import cn.iocoder.yudao.module.shipment.vo.box.PreloadResultVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.*;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadDetailBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadSectionBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionDetailVO;

/**
 * 预装货物 Service 接口
 *
 * @author wlh
 */
public interface BoxPreloadGoodsService extends IService<BoxPreloadGoodsDO> {

    /**
     * 创建预装货物
     * @param createReqVO 创建信息
     * @return 编号
     */
    PreloadResultVO createBoxPreloadGoods(@Valid BoxPreloadGoodsCreateReqVO createReqVO);


    /**
     * 删除预装货物
     * @param ids 编号
     */
    void deleteBoxPreloadGoods(Collection<Long> ids, Integer deleteType);

    /**
     * 获得预装货物
     * @param id 编号
     * @return 预装货物
     */
    BoxPreloadGoodsDO getBoxPreloadGoods(Long id);

    /**
     * 获得预装货物列表
     * @param ids 编号
     * @return 预装货物列表
     */
    List<BoxPreloadGoodsDO> getBoxPreloadGoodsList(Collection<Long> ids);

    /**
     * 获得预装货物分页
     * @param page 分页查询
     * @param query 查询
     * @return 预装货物分页
     */
    PageResult<BoxPreloadGoodsDO> getBoxPreloadGoodsPage(BoxPreloadGoodsQueryVO query, PageVO page);

    /**
     * 获得预装货物列表, 用于 Excel 导出
     * @param query 查询
     * @return 预装货物列表
     */
    List<BoxPreloadGoodsDO> getBoxPreloadGoodsList(BoxPreloadGoodsQueryVO query);

    /**
     * 根据订单项统计
     * @param orderItemDOList 订单项
     * @return
     */
    BoxGoodsStatistics calculationStatistics(List<OrderItemDO> orderItemDOList);

    /**
     * 获得预装部分列表
     * @param query 编号
     * @return 预装部分列表
     */
    BoxPreloadSectionDetailVO getBoxPreloadSectionList(PreloadedReq query);

    /**
     * 删除预装部分
     * @param id  部分ID
     */
    void deleteBoxPreloadSection(Long id, Integer deleteType);

    /**
     * 变换预装部分
     * @param params  参数
     */
    void changeSection(PreloadChange params);

    /**
     * 获得装柜部分列表，
     * @param shipmentId  出货单ID
     * @return
     */
    List<BoxLoadSectionBackVO> loadSecGoodsList(Long shipmentId);

    /**
     * 修改isRemove
     * @param shipmentId
     * @param orderId
     */
    void removeBox(Long shipmentId,  Long orderId);

    /**
     * 装柜订单列表
     * @param shipmentId shipmentId
     * @param secId secId
     * @return
     */
    BoxLoadDetailBackVO loadGoodsList(Long shipmentId, Long secId);

    BoxLoadDetailBackVO boxGoodsList(Long shipmentId, Long secId, Integer isCover, Integer isRemove);

    BoxLoadDetailBackVO boxGoodsList(Long shipmentId, Long secId, Integer isCover, Integer isRemove, Integer isDel);

    /**
     * 查询出货单是否存在这个订单号
     * @param shipmentId shipmentId
     * @param orderNo orderNo
     */
    Long selectByShipmentIdAndOrderNo(Long shipmentId, String orderNo);

    /**
     * 获得待预装分页
     * @param query query
     * @return
     */
    BoxOrderPreloadStatistics orderPreloadPage(OrderPreloadReq query);

    BoxOrderPreloadStatistics orderPreloadPageAir(OrderPreloadReq query);

    BoxLoadDetailBackVO boxGoodsDetail(Long shipmentId, Long secId);

    List<BoxLoadSectionBackVO> boxGoodsDetail(Long shipmentId);

    /**
     * 根据订单ID获取提单号
     * @param orderId  订单ID
     * @return
     */
    String getLadingBillNo(Long orderId);


    /**
     * 获取出货单统计信息
     * @param shipmentId 出货单
     * @return
     */
    BoxGoodsStatistics getBoxStatistics(Long shipmentId);

    Map<Long, BoxGoodsStatistics> getBoxStatistics(List<Long> idList);

    Map<Long, BoxGoodsStatistics> boxStatistics(List<Long> shipmentIdList);

    BoxGoodsStatistics calculationStatisticsByOrderList(Collection<OrderDO> orderDOList);

    /**
     * 获取单证信息
     * @param shipmentId  出货单
     * @return
     */
    String getDocumentInfo(Long shipmentId);

    /**
     * 根据出货单获取装柜商品项
     * @param shipmentId  出货单
     * @return
     */
    List<BoxPreloadGoodsBackVO> getShipGoodItemList(Long shipmentId);

    /**
     * 把订单从预装单中移出
     * @param shipmentId  出货单ID
     * @param orderIdList  订单ID
     * @param deleteType  删除类型
     * @param hasExit  是否退场业务
     */
    void remove(Long shipmentId, Collection<Long> orderIdList, Integer deleteType, boolean hasExit);

    Collection<Long> checkDeleteOrder(Long shipmentId, Integer deleteType, Collection<Long> orderIdList);

    /**
     * 获取正常的报关订单
     * @param shipmentId  出货单ID
     * @return
     */
    List<OrderDO> getNormalOrderList(Long shipmentId);

    /**
     * 预装关联订单
     * @param createReqVO 请求参数
     */
    void loadRelationOrder(PreloadResultVO createReqVO);

    List<BoxPreloadGoodsBackVO> orderItems(PreloadedReq query);

    void generateBillLadingNo(Long shipmentId, boolean updateOrder, boolean tempSort);

    void deleteOrderList(Long shipmentId, Collection<Long> orderIdCollection, Integer type, boolean hasExit);

    Set<Long> getShipOrderIdList(Long shipmentId);

    PreloadResultVO createBoxPreloadGoodsV2(BoxPreloadGoodsCreateReqVO createReqVO);

    PreloadResultVO createBoxPreloadGoodsAir(BoxPreloadGoodsCreateReqVO createReqVO);

    void delCheckStatus(Long shipmentId);
}
