package cn.iocoder.yudao.module.shipment.service.boxSettlement;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.module.shipment.convert.BoxSettlementConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAbnormalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxSettlementDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxSettlementMapper;
import cn.iocoder.yudao.module.shipment.enums.ClStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.SlStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementUpdateReqVO;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 结算 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxSettlementServiceImpl extends AbstractService<BoxSettlementMapper, BoxSettlementDO> implements BoxSettlementService {

    @Resource
    private BoxSettlementMapper boxSettlementMapper;
    @Resource
    @Lazy
    private BoxService boxService;
    @Resource
    private BoxOpLogService boxOpLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxSettlement(BoxSettlementCreateReqVO createReqVO) {
        // 插入
        BoxSettlementDO boxSettlement = BoxSettlementConvert.INSTANCE.convert(createReqVO);
        boxSettlementMapper.insert(boxSettlement);
        // 返回
        updateBoxSlStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(),
                createReqVO.getSlSettlementTime(), createReqVO.getSlSettledTime());
        return boxSettlement.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxSettlement(BoxSettlementUpdateReqVO updateReqVO) {
        // 更新
        BoxSettlementDO updateObj = BoxSettlementConvert.INSTANCE.convert(updateReqVO);
        boxSettlementMapper.updateById(updateObj);
        updateBoxSlStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(),
                updateReqVO.getSlSettlementTime(), updateReqVO.getSlSettledTime());
    }

    private void updateBoxSlStatus(Long shipmentId, Integer operateType, Date slSettlementTime, Date slSettledTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {

            checkWhetherSet(shipmentId);


            SlStatueEnum slStatueEnum = SlStatueEnum.NO_SETTLEMENT;
            if(slSettlementTime != null && slSettledTime != null) {
                if(slSettlementTime.compareTo(slSettledTime) == 0) {
                    slStatueEnum = SlStatueEnum.HAS_SETTLEMENT;
                } else {
                    slStatueEnum = SlStatueEnum.SETTLEMENT_ING;
                }
            } else {
                if(slSettlementTime != null) {
                    slStatueEnum = SlStatueEnum.SETTLEMENT_ING;
                }
            }
            boxDO.setSlStatus(slStatueEnum.getSlStatus());

            //操作日志
            boxOpLogService.createBoxOpLog(shipmentId, slStatueEnum.getSlStatus(),
                    slStatueEnum.getText(), "", null);
        }
        boxService.updateById(boxDO);
    }

    private void checkWhetherSet(Long shipmentId) {
        //检查异常是否已处理
        List<BoxAbnormalDO> abnormalList = boxService.getAbnormalList(shipmentId);
        if(CollectionUtils.isNotEmpty(abnormalList)) {
            throw new ServiceException(500, "有异常未处理，不能结算");
        }
        long count = boxService.getUnWriteOffCount(shipmentId);
        if(count > 0) {
            throw new ServiceException(500, "有应收费用未核销，不能结算");
        }
    }


    @Override
    public BoxSettlementDO getBoxSettlement(Long id) {
        return boxSettlementMapper.selectById(id);
    }

    @Override
    public List<BoxSettlementDO> getBoxSettlementList(Collection<Long> ids) {
        return boxSettlementMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxSettlementDO> getBoxSettlementPage(BoxSettlementQueryVO query, PageVO page) {
        return boxSettlementMapper.selectPage(page, query);
    }

    @Override
    public List<BoxSettlementDO> getBoxSettlementList(BoxSettlementQueryVO query) {
        return boxSettlementMapper.selectList(query);
    }
}
