package cn.iocoder.yudao.module.shipment.service.boxShipping;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxShippingConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxShippingDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxShippingMapper;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.DtStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingUpdateReqVO;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 起运 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxShippingServiceImpl extends AbstractService<BoxShippingMapper, BoxShippingDO> implements BoxShippingService {

    @Resource
    private BoxShippingMapper boxShippingMapper;

    @Resource
    @Lazy
    BoxService boxService;

    @Resource
    private BoxOpLogService boxOpLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxShipping(BoxShippingCreateReqVO createReqVO) {
        // 插入
        BoxShippingDO boxShipping = BoxShippingConvert.INSTANCE.convert(createReqVO);
        boxShippingMapper.insert(boxShipping);
        // 返回
        updateBoxDtStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), boxShipping.getDtRealShipTime(), boxShipping.getDtEstArrivalTime());
        return boxShipping.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxShipping(BoxShippingUpdateReqVO updateReqVO) {
        // 更新
        BoxShippingDO updateObj = BoxShippingConvert.INSTANCE.convert(updateReqVO);
        boxShippingMapper.updateById(updateObj);

        updateBoxDtStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateObj.getDtRealShipTime(), updateObj.getDtEstArrivalTime());
    }

    private void updateBoxDtStatus(Long shipmentId, Integer operateType, Date dtRealShipTime, Date dtEstArrivalTime) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if (boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }

        if (operateType == OperateTypeEnum.COMMIT.getType()) {
            DtStatueEnum shipping = DtStatueEnum.HAS_SHIPPING;
            //已经起运，不执行下面的业务
            if (boxDO.getDtStatus() == shipping.getDtStatus()) {
                boxService.updateOrderArriveTime(boxDO.getSelfNo(), Integer.parseInt(boxDO.getTransportType()), dtEstArrivalTime);
                return;
            }

            boxDO.setDtStatus(shipping.getDtStatus());
            boxDO.setQyDate(dtRealShipTime);

            //更新日志，修改为起运中
            boxOpLogService.createBoxOpLog(shipmentId, shipping.getDtStatus(), shipping.getStatusNameZh(), shipping.getStatusNameEn(), null);

            //修改订单状态为航运中
            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.IN_SHIPPING.getValue(), null,
                    BoxStatusEnum.SHIPPING.getStatus(), null, null, dtRealShipTime, dtEstArrivalTime, Integer.parseInt(boxDO.getTransportType()));

            String date = DateUtils.formatDate(dtEstArrivalTime);
            boxService.shipmentInternalMessage(shipmentId, boxDO.getSelfNo(), date, BoxStatusEnum.SHIPPING, Integer.parseInt(boxDO.getTransportType()));
        }
        boxService.updateById(boxDO);
    }

    @Override
    public BoxShippingDO getBoxShipping(Long id) {
        return boxShippingMapper.selectById(id);
    }

    @Override
    public List<BoxShippingDO> getBoxShippingList(Collection<Long> ids) {
        return boxShippingMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxShippingDO> getBoxShippingPage(BoxShippingQueryVO query, PageVO page) {
        return boxShippingMapper.selectPage(page, query);
    }

    @Override
    public List<BoxShippingDO> getBoxShippingList(BoxShippingQueryVO query) {
        return boxShippingMapper.selectList(query);
    }
}
