package cn.iocoder.yudao.module.shipment.service.boxTakeoff;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.shipment.convert.BoxTakeoffConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxTakeoffDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxTakeoffMapper;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.DtStatueEnum;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.StkStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffUpdateReqVO;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 起飞 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxTakeoffServiceImpl extends AbstractService<BoxTakeoffMapper, BoxTakeoffDO> implements BoxTakeoffService {

    @Resource
    private BoxTakeoffMapper boxTakeoffMapper;
    @Resource
    @Lazy
    BoxService boxService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createBoxTakeoff(BoxTakeoffCreateReqVO createReqVO) {
        // 插入
        BoxTakeoffDO boxTakeoff = BoxTakeoffConvert.INSTANCE.convert(createReqVO);
        boxTakeoffMapper.insert(boxTakeoff);
        // 返回
        updateBoxStkStatus(createReqVO.getShipmentId(), createReqVO.getOperateType(), boxTakeoff.getActHeadTime());
        return boxTakeoff.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBoxTakeoff(BoxTakeoffUpdateReqVO updateReqVO) {
        // 更新
        BoxTakeoffDO updateObj = BoxTakeoffConvert.INSTANCE.convert(updateReqVO);
        boxTakeoffMapper.updateById(updateObj);
        updateBoxStkStatus(updateReqVO.getShipmentId(), updateReqVO.getOperateType(), updateReqVO.getActHeadTime());
    }

    private void updateBoxStkStatus(Long shipmentId, Integer operateType, Date actHeadTime ) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }
        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            StkStatueEnum stkStatueEnum = StkStatueEnum.HAS_TAKE_OFF;

            boxDO.setStkStatus(stkStatueEnum.getStkStatus());

            boxService.updateOrderStatusByShipmentIdAndTransportType(shipmentId, OrderStatusEnum.TWP_WAY_TAKEOFF.getValue(),
                    null, BoxStatusEnum.TWO_WAY_TAKEOFF.getStatus(),
                    null, null, actHeadTime, null, Integer.parseInt(boxDO.getTransportType()));

            //更新日志，修改为起运中
//            boxOpLogService.createBoxOpLog(shipmentId, shipping.getDtStatus(), shipping.getStatusNameZh(), shipping.getStatusNameEn());
        }
        boxService.updateById(boxDO);
    }


    @Override
    public BoxTakeoffDO getBoxTakeoff(Long id) {
        return boxTakeoffMapper.selectById(id);
    }

    @Override
    public List<BoxTakeoffDO> getBoxTakeoffList(Collection<Long> ids) {
        return boxTakeoffMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxTakeoffDO> getBoxTakeoffPage(BoxTakeoffQueryVO query, PageVO page) {
        return boxTakeoffMapper.selectPage(page, query);
    }

    @Override
    public List<BoxTakeoffDO> getBoxTakeoffList(BoxTakeoffQueryVO query) {
        return boxTakeoffMapper.selectList(query);
    }
}
