package cn.iocoder.yudao.module.shipment.service.boxTrailer;

import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.module.shipment.convert.BoxTrailerConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxTrailerDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxMapper;
import cn.iocoder.yudao.module.shipment.dal.mysql.BoxTrailerMapper;
import cn.iocoder.yudao.module.shipment.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.TlStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerUpdateReqVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_NOT_EXISTS;

/**
 * 拖车 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class BoxTrailerServiceImpl extends AbstractService<BoxTrailerMapper, BoxTrailerDO> implements BoxTrailerService {

    @Resource
    private BoxTrailerMapper boxTrailerMapper;

    @Lazy
    @Resource
    private BoxService boxService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long createBoxTrailer(BoxTrailerCreateReqVO createReqVO) {
        // 插入
        BoxTrailerDO boxTrailer = BoxTrailerConvert.INSTANCE.convert(createReqVO);
        boxTrailerMapper.insert(boxTrailer);

        //更新拖车货柜状态
        updateBoxTlContainerStatus(createReqVO.getShipmentId(), createReqVO.getTlContainerStatus(),
                createReqVO.getOperateType(), createReqVO.getTlContainerNo());
        // 返回
        return boxTrailer.getId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateBoxTrailer(BoxTrailerUpdateReqVO updateReqVO) {
        // 更新
        BoxTrailerDO updateObj = BoxTrailerConvert.INSTANCE.convert(updateReqVO);
        boxTrailerMapper.updateById(updateObj);

        //更新拖车货柜状态
        updateBoxTlContainerStatus(updateReqVO.getShipmentId(), updateReqVO.getTlContainerStatus(),
                updateReqVO.getOperateType(), updateReqVO.getTlContainerNo());
    }

    private void updateBoxTlContainerStatus(Long shipmentId, Integer tlContainerStatus, Integer operateType, String containerNo) {
        BoxDO boxDO = boxService.getBox(shipmentId);
        if(boxDO == null) {
            throw exception(BOX_NOT_EXISTS);
        }

        if(operateType == OperateTypeEnum.COMMIT.getType()) {
            Integer tlStatus = boxDO.getTlStatus();

            boxDO.setTlContainerStatus(tlContainerStatus);
            boxDO.setTlStatus(TlStatueEnum.HAS_SEND_CAR.getTlStatus());
            if(StringUtils.isNotEmpty(containerNo)) {
                boxDO.setCubNo(containerNo);
            }

            //如果是已经派车，则不用再次更新订单状态
            if(tlStatus != TlStatueEnum.HAS_SEND_CAR.getTlStatus()) {
                boxService.updateOrderStatusByShipmentId(shipmentId, null, null,
                        boxDO.getShipmentStatus(), null, null);
            }
        }
        boxService.updateById(boxDO);
    }

    @Override
    public BoxTrailerDO getBoxTrailer(Long id) {
        return boxTrailerMapper.selectById(id);
    }

    @Override
    public List<BoxTrailerDO> getBoxTrailerList(Collection<Long> ids) {
        return boxTrailerMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxTrailerDO> getBoxTrailerPage(BoxTrailerQueryVO query, PageVO page) {
        return boxTrailerMapper.selectPage(page, query);
    }

    @Override
    public List<BoxTrailerDO> getBoxTrailerList(BoxTrailerQueryVO query) {
        return boxTrailerMapper.selectList(query);
    }
}
