package cn.iocoder.yudao.module.shipment.service.futureBox;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.FutureBoxDO;
import cn.iocoder.yudao.module.shipment.vo.futureBox.*;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 预定舱计划 Service 接口
 *
 * @author 系统管理员
 */
public interface FutureBoxService extends IService<FutureBoxDO> {

    /**
     * 创建预定舱计划
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createFutureBox(@Valid FutureBoxCreateReqVO createReqVO);

    /**
     * 更新预定舱计划
     *
     * @param updateReqVO 更新信息
     */
    void updateFutureBox(@Valid FutureBoxUpdateReqVO updateReqVO);

    /**
     * 删除预定舱计划
     *
     * @param id 编号
     */
    void deleteFutureBox(Long id);

    /**
     * 获得预定舱计划
     *
     * @param id 编号
     * @return 预定舱计划
     */
    FutureBoxDO getFutureBox(Long id);

    /**
     * 获得预定舱计划列表
     *
     * @param ids 编号
     * @return 预定舱计划列表
     */
    List<FutureBoxDO> getFutureBoxList(Collection<Long> ids);

    /**
     * 获得预定舱计划分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 预定舱计划分页
     */
    PageResult<FutureBoxDO> getFutureBoxPage(FutureBoxQueryVO query, PageVO page);


    /**
     * 获得预定舱计划分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 预定舱计划分页
     */
    PageResult<FutureBoxBackVO> getFutureBoxBackPage(FutureBoxQueryVO query, PageVO page);

    /**
     * 获得预定舱计划列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 预定舱计划列表
     */
    List<FutureBoxDO> getFutureBoxList(FutureBoxQueryVO query);


    /**
     * 根据运输方式ID得到待入仓方数
     *
     * @param transportId
     * @return
     */
    BigDecimal getToBeWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到已入仓方数
     *
     * @param transportId
     * @return
     */
    BigDecimal getWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到已封柜方数
     *
     * @return
     */
    BigDecimal getBoxedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到已封柜方数
     *
     * @return
     */
    BigDecimal getBoxedWeight(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到异常订单方数
     *
     * @param futureBoxTotalQueryVo
     * @return
     */
    BigDecimal getExceptionVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到自单代报订单方数
     * @param futureBoxTotalQueryVo
     * @return
     */
    BigDecimal getOrderReportVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);

    /**
     * 根据运输方式ID得到待排单订单方数
     * @param futureBoxTotalQueryVo
     * @return
     */
    BigDecimal getWarehousingVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo);
}
