package cn.iocoder.yudao.module.shipment.service.futureBox;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.FutureBoxConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.FutureBoxDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.FutureBoxMapper;
import cn.iocoder.yudao.module.shipment.vo.futureBox.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.FUTURE_BOX_NOT_EXISTS;

/**
 * 预定舱计划 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class FutureBoxServiceImpl extends AbstractService<FutureBoxMapper, FutureBoxDO> implements FutureBoxService {

    @Resource
    private FutureBoxMapper futureBoxMapper;

    @Override
    public Long createFutureBox(FutureBoxCreateReqVO createReqVO) {
        // 插入
        FutureBoxDO futureBox = FutureBoxConvert.INSTANCE.convert(createReqVO);
        futureBoxMapper.insert(futureBox);
        // 返回
        return futureBox.getId();
    }

    @Override
    public void updateFutureBox(FutureBoxUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateFutureBoxExists(updateReqVO.getId());
        // 更新
        FutureBoxDO updateObj = FutureBoxConvert.INSTANCE.convert(updateReqVO);
        futureBoxMapper.updateById(updateObj);
    }

    @Override
    public void deleteFutureBox(Long id) {
        // 校验存在
        this.validateFutureBoxExists(id);
        // 删除
        futureBoxMapper.deleteById(id);
    }

    private void validateFutureBoxExists(Long id) {
        if (futureBoxMapper.selectById(id) == null) {
            throw exception(FUTURE_BOX_NOT_EXISTS);
        }
    }

    @Override
    public FutureBoxDO getFutureBox(Long id) {
        return futureBoxMapper.selectById(id);
    }

    @Override
    public List<FutureBoxDO> getFutureBoxList(Collection<Long> ids) {
        return futureBoxMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<FutureBoxDO> getFutureBoxPage(FutureBoxQueryVO query, PageVO page) {
        return futureBoxMapper.selectPage(page, query);
    }

    @Override
    public PageResult<FutureBoxBackVO> getFutureBoxBackPage(FutureBoxQueryVO query, PageVO page) {
        IPage<FutureBoxBackVO> mpPage = MyBatisUtils.buildPage(page);
        futureBoxMapper.selectBackPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<FutureBoxDO> getFutureBoxList(FutureBoxQueryVO query) {
        return futureBoxMapper.selectList(query);
    }

    @Override
    public BigDecimal getToBeWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getToBeWareHousedVolume(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getWareHousedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getWareHousedVolume(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getBoxedVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getBoxedVolume(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getBoxedWeight(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getBoxedWeight(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getExceptionVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo){
        return futureBoxMapper.getExceptionVolume(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getOrderReportVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getOrderReportVolume(futureBoxTotalQueryVo);
    }

    @Override
    public BigDecimal getWarehousingVolume(FutureBoxTotalQueryVo futureBoxTotalQueryVo) {
        return futureBoxMapper.getWarehousingVolume(futureBoxTotalQueryVo);
    }


}
