package cn.iocoder.yudao.module.shipment.utils;

import cn.iocoder.yudao.module.shipment.dto.AirLoadDto;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author huyufeng
 * @date 2022/12/14
 * @description 自定义单元格合并策略
 */

public class CustomAirLoadMergeStrategyNew extends AbstractMergeStrategy {

    /**
     * 分组，每几行合并一次
     */
    private List<Integer> exportFieldGroupCountList;

    private List<Integer> partMergeList;

    private List<Integer> colorRowList;

    private List<Integer> mergePkgCountList;



    private int mergeType;

    /**
     * 目标合并列index
     */
    private Integer targetColumnIndex;

    // 需要开始合并单元格的首行index
    private Integer rowIndex;

    // exportDataList为待合并目标列的值   partMerge为另外一种合并方式计算好的合并行数，mergeType 合并类型  1为订单维度合并  2为入仓记录维度的合并
    public CustomAirLoadMergeStrategyNew(List<AirLoadDto> exportDataList, List<Integer> partMergeList, List<Integer> colorRowList,List<Integer> mergePkgCountList, Integer targetColumnIndex, int mergeType) {
        this.exportFieldGroupCountList = getGroupCountList(exportDataList,targetColumnIndex);
        this.partMergeList = partMergeList;
        this.mergeType = mergeType;
        this.targetColumnIndex = targetColumnIndex;
        this.colorRowList = colorRowList;
        this.mergePkgCountList = mergePkgCountList;
    }


    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

        if (null == rowIndex) {
            rowIndex = cell.getRowIndex();
        }
        // 仅从首行以及目标列的单元格开始合并，忽略其他
        if (cell.getRowIndex() == rowIndex && cell.getColumnIndex() == targetColumnIndex ) {
            if (mergeType == 1) {
                mergeGroupColumn(sheet,exportFieldGroupCountList);
            } else if (mergeType == 2) {
                mergeGroupColumn(sheet,partMergeList);
            } else if (mergeType == 3) {
                mergeGroupColumn(sheet,mergePkgCountList);
            }
        }
    }

    private void mergeGroupColumn(Sheet sheet,List<Integer> list) {
        int rowCount = rowIndex;
        for (Integer count : list) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }


    // 该方法将目标列根据值是否相同连续可合并，存储可合并的行数
    private List<Integer> getGroupCountList(List<AirLoadDto> exportDataList,Integer targetColumnIndex) {
        if (CollectionUtils.isEmpty(exportDataList)) {
            return new ArrayList<>();
        }

        List<Integer> groupCountList = new ArrayList<>();
        int count = 1;

        LinkedHashMap<String, List<AirLoadDto>> collect = exportDataList.stream().collect(Collectors.groupingBy(AirLoadDto::getId, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<String, List<AirLoadDto>> entry : collect.entrySet()) {
            groupCountList.add(entry.getValue().size());
        }
        return groupCountList;
    }

}
