package cn.iocoder.yudao.module.shipment.utils;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemDo;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsQueryVO;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author huyufeng
 * @date 2022/12/14
 * @description 自定义单元格合并策略
 */

public class CustomMergeStrategy extends AbstractMergeStrategy {

    /**
     * 分组，每几行合并一次
     */
    private List<Integer> exportFieldGroupCountList;

    private List<Integer> warehouseInCountList;

    private OrderWarehouseInService orderWarehouseInService;

    //预装单按照入仓数据合并的列
    private static List indexNumList = new ArrayList();
    static {
        indexNumList.add(2);
        indexNumList.add(4);
        indexNumList.add(5);
        indexNumList.add(6);
        indexNumList.add(7);
        indexNumList.add(8);
        indexNumList.add(9);
        indexNumList.add(10);
        indexNumList.add(11);
        indexNumList.add(18);
        indexNumList.add(20);
    }

    /**
     * 目标合并列index
     */
    private Integer targetColumnIndex;

    // 需要开始合并单元格的首行index
    private Integer rowIndex;

    // exportDataList为待合并目标列的值
    public CustomMergeStrategy(Map<Long, List<BoxPreloadGoodsBackVO>> exportDataList, OrderWarehouseInService orderWarehouseInService, OrderItemService orderItemService, Integer targetColumnIndex, boolean flag) {
        this.exportFieldGroupCountList = getGroupCountList(exportDataList,orderWarehouseInService,orderItemService,targetColumnIndex,flag);
        this.targetColumnIndex = targetColumnIndex;
        this.orderWarehouseInService = orderWarehouseInService;
        this.warehouseInCountList = getWarehouseInCountList(exportDataList,orderWarehouseInService,orderItemService,targetColumnIndex,flag);
    }


    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

        if (null == rowIndex) {
            rowIndex = cell.getRowIndex();
        }
        // 仅从首行以及目标列的单元格开始合并，忽略其他
        if (cell.getRowIndex() == rowIndex && cell.getColumnIndex() == targetColumnIndex) {
            if (orderWarehouseInService == null || !indexNumList.contains(targetColumnIndex.intValue())) {
                mergeGroupColumn(sheet);
            } else {
                mergeSpecialColumn(sheet);
            }
        }
    }

    private void mergeGroupColumn(Sheet sheet) {
        int rowCount = rowIndex;
        for (Integer count : exportFieldGroupCountList) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }

    private void mergeSpecialColumn(Sheet sheet) {
        int rowCount = rowIndex;
        for (Integer count : warehouseInCountList) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }

    // 该方法将目标列根据值是否相同连续可合并，存储可合并的行数
    private List<Integer> getGroupCountList(Map<Long, List<BoxPreloadGoodsBackVO>> exportDataList,OrderWarehouseInService orderWarehouseInService,OrderItemService orderItemService,Integer targetColumnIndex,boolean flag) {
        if (CollectionUtils.isEmpty(exportDataList)) {
            return new ArrayList<>();
        }

        List<Integer> groupCountList = new ArrayList<>();
        int count = 1;

        if (flag) {
            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : exportDataList.entrySet()) {
                List<BoxPreloadGoodsBackVO> list = entry.getValue();
                if (orderWarehouseInService == null) {
                    count = entry.getValue().size();
                } else {
                    List<Long> orderItemIds = orderItemService.selectList(OrderItemDO::getOrderId, entry.getKey()).stream().map(OrderItemDO::getOrderItemId).collect(Collectors.toList());
                    int value = orderWarehouseInService.selectCount(new LambdaQueryWrapperX<OrderWarehouseInDO>().eq(OrderWarehouseInDO::getOrderId, entry.getKey()).in(OrderWarehouseInDO::getOrderItemId,orderItemIds)).intValue();
                    count = value;
                }
                groupCountList.add(count);
            }
        }
        return groupCountList;
    }

    private List<Integer> getWarehouseInCountList(Map<Long, List<BoxPreloadGoodsBackVO>> exportDataList,OrderWarehouseInService orderWarehouseInService,OrderItemService orderItemService,Integer targetColumnIndex,boolean flag) {
        if (CollectionUtils.isEmpty(exportDataList)) {
            return new ArrayList<>();
        }

        List<Integer> groupCountList = new ArrayList<>();
        int count = 1;

        if (!flag) {
            for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : exportDataList.entrySet()) {
                List<BoxPreloadGoodsBackVO> list = entry.getValue();
                List<Long> orderItemIds = orderItemService.selectList(OrderItemDO::getOrderId, entry.getKey()).stream().map(OrderItemDO::getOrderItemId).collect(Collectors.toList());
                List<OrderWarehouseInBackVO> orderWarehouseInList1 = orderWarehouseInService.getOrderWarehouseInList(entry.getKey());
                Map<Long, List<OrderWarehouseInBackItemDo>> collect = orderWarehouseInList1.stream().flatMap(item -> item.getOrderWarehouseInBackItemDoList().stream()).collect(Collectors.groupingBy(OrderWarehouseInBackItemDo::getId));
                List<OrderWarehouseInDO> orderWarehouseInList = orderWarehouseInService.selectList(new QueryWrapper<OrderWarehouseInDO>()
                        .select("id","order_item_id","in_time","unit","(select IFNULL(GROUP_CONCAT(concat(a.area_id,a.location_id)),'') from ecw_order_location a where a.warehouse_in_id=ecw_order_warehouse_in.id) as locationId ")
                        .eq("order_id", entry.getKey()).in("order_item_id",orderItemIds));
                orderWarehouseInList = orderWarehouseInList.stream().sorted(Comparator.comparing(OrderWarehouseInDO::getOrderItemId).thenComparing(OrderWarehouseInDO::getInTime).thenComparing(OrderWarehouseInDO::getUnit).thenComparing(OrderWarehouseInDO::getLocationId)).collect(Collectors.toList());
                if (orderWarehouseInList != null && orderWarehouseInList.size() >=2) {
                    for (int i = 1; i < orderWarehouseInList.size(); i++) {
                        if (orderWarehouseInList.get(i).getOrderItemId().longValue()==orderWarehouseInList.get(i - 1).getOrderItemId().longValue()
                                && orderWarehouseInList.get(i).getInTime().getTime() == orderWarehouseInList.get(i - 1).getInTime().getTime()
                                && orderWarehouseInList.get(i).getUnit().equals(orderWarehouseInList.get(i - 1).getUnit())
                                && collect.get(orderWarehouseInList.get(i).getId()).get(0).getOrderLocationStr4Merge().equals(collect.get(orderWarehouseInList.get(i - 1).getId()).get(0).getOrderLocationStr4Merge())) {
                            count++;
                        } else {
                            groupCountList.add(count);
                            count = 1;
                        }
                    }
                }
                groupCountList.add(count);
                count = 1;
            }
        }
        return groupCountList;
    }

    private int getMergeCount (Long orderId,List<BoxPreloadGoodsBackVO> boxPreload,OrderWarehouseInService orderWarehouseInService,OrderItemService orderItemService,List<Integer> groupCountList) {
        int count = 1;
        List<OrderWarehouseInBackVO> orderWarehouseInList = orderWarehouseInService.getOrderWarehouseInList(orderId);
        List<OrderItemDO> orderItems = boxPreload.stream().map(item -> orderItemService.getOrderItem(item.getOrderItemId())).collect(Collectors.toList());
        for(OrderItemDO item : orderItems) {
            List<OrderWarehouseInBackVO> warehouseInBackVOS = orderWarehouseInList.stream().filter(it -> it.getOrderItemId().longValue() == item.getOrderItemId().longValue()).collect(Collectors.toList());
            List<OrderWarehouseInBackItemDo> orderWarehouseInBackItemDoList = warehouseInBackVOS.get(0).getOrderWarehouseInBackItemDoList();
            if (CollectionUtil.isNotEmpty(orderWarehouseInBackItemDoList) && orderWarehouseInBackItemDoList.size() >= 2 ) {
                for (int i = 1;i< orderWarehouseInBackItemDoList.size(); i++) {
                    if (DateUtil.isSameTime(orderWarehouseInBackItemDoList.get(i).getInTime(),orderWarehouseInBackItemDoList.get(i-1).getInTime())) {
                        count ++;
                    } else {
                        count = 1;
                    }
                }
            }
            groupCountList.add(count);
        }
        return count;
    }

}
