package cn.iocoder.yudao.module.shipment.utils;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.service.order.OrderItemService;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemDo;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackVO;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author huyufeng
 * @date 2022/12/14
 * @description 自定义单元格合并策略
 */

public class CustomMergeStrategyNew extends AbstractMergeStrategy {

    /**
     * 分组，每几行合并一次
     */
    private List<Integer> exportFieldGroupCountList;

    private List<Integer> partMergeList;

    private List<Integer> colorRowList;

    private int mergeType;

    /**
     * 目标合并列index
     */
    private Integer targetColumnIndex;

    // 需要开始合并单元格的首行index
    private Integer rowIndex;

    // exportDataList为待合并目标列的值   partMerge为另外一种合并方式计算好的合并行数，mergeType 合并类型  1为订单维度合并  2为入仓记录维度的合并
    public CustomMergeStrategyNew(List<PreloadDto> exportDataList,List<Integer> partMergeList,List<Integer> colorRowList, Integer targetColumnIndex, int mergeType) {
        this.exportFieldGroupCountList = getGroupCountList(exportDataList,targetColumnIndex);
        this.partMergeList = partMergeList;
        this.mergeType = mergeType;
        this.targetColumnIndex = targetColumnIndex;
        this.colorRowList = colorRowList;
    }


    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

        if (null == rowIndex) {
            rowIndex = cell.getRowIndex();
        }
        // 仅从首行以及目标列的单元格开始合并，忽略其他
        if (cell.getRowIndex() == rowIndex && cell.getColumnIndex() == targetColumnIndex ) {
            if (mergeType == 1) {
                mergeGroupColumn(sheet);
            } else if (mergeType == 2) {
                mergeSpecialColumn(sheet);
            }
        }
    }

    private void mergeGroupColumn(Sheet sheet) {
        int rowCount = rowIndex;
        for (Integer count : exportFieldGroupCountList) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }

    private void mergeSpecialColumn(Sheet sheet) {
        int rowCount = rowIndex;
        for (Integer count : partMergeList) {
            if (count == 1) {
                rowCount += count;
                continue;
            }
            // 合并单元格
            CellRangeAddress cellRangeAddress = new CellRangeAddress(rowCount, rowCount + count - 1, targetColumnIndex, targetColumnIndex);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
            rowCount += count;
        }
    }


    // 该方法将目标列根据值是否相同连续可合并，存储可合并的行数
    private List<Integer> getGroupCountList(List<PreloadDto> exportDataList,Integer targetColumnIndex) {
        if (CollectionUtils.isEmpty(exportDataList)) {
            return new ArrayList<>();
        }

        List<Integer> groupCountList = new ArrayList<>();
        int count = 1;

        LinkedHashMap<String, List<PreloadDto>> collect = exportDataList.stream().collect(Collectors.groupingBy(PreloadDto::getId, LinkedHashMap::new, Collectors.toList()));
        for (Map.Entry<String, List<PreloadDto>> entry : collect.entrySet()) {
            groupCountList.add(entry.getValue().size());
        }
        return groupCountList;
    }

}
