package cn.iocoder.yudao.module.shipment.vo.box;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.depository.vo.warehouse.WarehouseRespVO;
import cn.iocoder.yudao.module.ecw.controller.admin.cabinet.vo.CabinetRespVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.sale.vo.channel.ChannelRespVO;
import cn.iocoder.yudao.module.shipment.enums.BoxAirStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxApprovalTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.BoxStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.TransportTypeEnum;
import com.alibaba.excel.annotation.ExcelProperty;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Data
@ApiModel("管理后台 - 出货基本信息 Response VO")
public class BoxBaseBackVO {

    @ExcelProperty("主键ID")
    @ApiModelProperty(value = "主键ID", required = true)
    private Long id;

    @ExcelProperty("自编号")
    @ApiModelProperty(value = "自编号")
    private String selfNo;

    @ExcelProperty("柜号")
    @ApiModelProperty(value = "柜号")
    private String cubNo;

    @ExcelProperty("柜型ID")
    @ApiModelProperty(value = "柜型ID")
    private Long cabinetId;

    @ApiModelProperty(value = "柜型详情")
    private CabinetRespVO cabinetRespVO;

    @ExcelProperty("始发地id")
    @ApiModelProperty(value = "始发地id")
    private Long startWarehouseId;

    @ApiModelProperty(value = "始发仓库详情")
    private WarehouseRespVO startWarehouseResp;

    @ExcelProperty("目的地id")
    @ApiModelProperty(value = "目的地id")
    private Long destWarehouseId;

    @ApiModelProperty(value = "目的仓库详情")
    private WarehouseRespVO destWarehouseResp;

    @ExcelProperty("运输方式")
    @ApiModelProperty(value = "运输方式")
    private String transportType;

    @ExcelProperty("出货渠道ID")
    @ApiModelProperty(value = "出货渠道ID")
    private Long shippingChannelId;

    @ApiModelProperty(value = "出货渠道详情")
    private ChannelRespVO channelRespVO;

    @ApiModelProperty(value = "统计信息")
    private BoxGoodsStatistics boxStatistics;

    @ApiModelProperty(value = "单证信息字典,多个以逗号分割.出货单下所有订单包含的单证要求")
    private String documentInfo;

    @ApiModelProperty(value = "按照要求过滤后显示的单证信息")
    private String documentInfoShow;

    @ApiModelProperty(value = "正常报关订单列表,自单代报的")
    private List<OrderDO> normalOrderList;

    @ExcelProperty("预装时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "预装时间")
    private Date yzDate;

    @ExcelProperty("装柜时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "装柜时间")
    private Date zgDate;

    @ExcelProperty("起运时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "起运时间")
    private Date qyDate;

    @ExcelProperty("到港时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "到港时间")
    private Date dgDate;

    @ExcelProperty("清关时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "清关时间")
    private Date qgDate;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "到仓时间")
    private Date dcDate;

    @ExcelProperty("提货时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "提货时间")
    private Date thDate;

    @ExcelProperty(value = "订舱状态")
    @ApiModelProperty(value = "订舱状态")
    private Integer bkStatus;

    @ExcelProperty(value = "预装状态")
    @ApiModelProperty(value = "预装状态")
    private Integer prStatus;

    @ApiModelProperty(value = "理货状态：2111、未理货；2112、已理货")
    private Integer tyStatus;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "提交理货时间")
    private Date tyTime;

    @ApiModelProperty(value = "理货操作人")
    private String tyOperator;

    @ExcelProperty(value = "拖车状态")
    @ApiModelProperty(value = "拖车状态")
    private Integer tlStatus;

    @ExcelProperty(value = "拖车提柜状态")
    @ApiModelProperty(value = "拖车提柜状态")
    private Integer tlContainerStatus;

    @ExcelProperty(value = "装柜状态")
    @ApiModelProperty(value = "装柜状态")
    private Integer ldStatus;

    @ExcelProperty(value = "报关状态")
    @ApiModelProperty(value = "报关状态:51、未报关；52、报关中；53、已报关")
    private Integer dcStatus;

    @ExcelProperty(value = "报关放行状态")
    @ApiModelProperty(value = "报关放行状态:1：报关中，2：放行，3：查验")
    private Integer dcCustomsStatus;

    @ExcelProperty(value = "查验状态")
    @ApiModelProperty(value = "查验状态:1：退场，2：部分退场，3：查验后放行")
    private Integer dcCheckStatus;

    @ApiModelProperty(value = "退场审核状态,1-审核中，2-审核成功，3-审核失败，4-取消")
    private Integer checkExamineStatus;

    @ApiModelProperty(value = "退场处理状态,1-已处理,0-未处理")
    private Integer checkDealStatus;

    @ApiModelProperty(value = "过机状态,1：顺利，2：异常")
    private Integer overMachineStatus;

    @ApiModelProperty(value = "过机异常状态，1：敏感货，2：重量误报")
    private Integer overMachineAbnormalStatus;

    @ApiModelProperty(value = "删单退场审核状态，1：退场审核中，2：退场审核成功，3：退场审核失败，4：退场审核取消")
    private Integer overMachineExamineStatus;

    @ExcelProperty(value = "配船状态")
    @ApiModelProperty(value = "配船状态")
    private Integer saStatus;

    @ExcelProperty(value = "提单补料状态")
    @ApiModelProperty(value = "提单补料状态")
    private Integer blStatus;

    @ExcelProperty(value = "驳船状态")
    @ApiModelProperty(value = "驳船状态")
    private Integer bgStatus;

    @ExcelProperty(value = "起运状态")
    @ApiModelProperty(value = "起运状态")
    private Integer dtStatus;

    @ExcelProperty(value = "提单COPY状态")
    @ApiModelProperty(value = "提单COPY状态")
    private Integer cpStatus;

    @ExcelProperty(value = "清关文件状态")
    @ApiModelProperty(value = "清关文件状态")
    private Integer cdStatus;

    @ExcelProperty(value = "到港状态")
    @ApiModelProperty(value = "到港状态")
    private Integer apStatus;

    @ExcelProperty(value = "清关状态")
    @ApiModelProperty(value = "清关状态")
    private Integer clStatus;

    @ExcelProperty(value = "二程起飞状态")
    @ApiModelProperty(value = "二程起飞状态")
    private Integer stkStatus;

    @ExcelProperty(value = "二程到港状态")
    @ApiModelProperty(value = "二程到港状态")
    private Integer sapStatus;

    @ExcelProperty(value = "卸柜状态")
    @ApiModelProperty(value = "卸柜状态")
    private Integer ulStatus;

    @ExcelProperty(value = "结算状态")
    @ApiModelProperty(value = "结算状态")
    private Integer slStatus;

    @ApiModelProperty(value = "正在反审的类型")
    private Integer backApprovalType;

    @ApiModelProperty(value = "出货状态")
    private Integer shipmentStatus;

    @ApiModelProperty(value = "空运出货状态")
    private Integer shipmentStatusAir;

    @ApiModelProperty(value = "出货状态描述")
    private String shipmentStatusText;

    @ApiModelProperty(value = "可见状态")
    private Integer status;

    @ApiModelProperty(value = "出货单订单数量")
    private Long orderNum;

    @ApiModelProperty(value = "出货单订单退场数量")
    private Long orderExitNum;

    @ApiModelProperty(value = "目的港清关")
    private Integer destinationClearance;

    @ApiModelProperty(value = "发货方式")
    private Integer deliveryType;

    @ApiModelProperty(value = "合包状态")
    private Integer mergePkgStatus;

    @ApiModelProperty(value = "空运出货状态")
    private Integer airShipmentStatus;

    @ApiModelProperty(value = "到仓状态")
    private Integer toWarehouseStatus;

    @ApiModelProperty(value = "起飞状态")
    private Integer flyStatus;

    @ExcelProperty("创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "创建时间", required = true)
    private Date createTime;

    @ExcelProperty(value = "分拣状态")
    @ApiModelProperty(value = "分拣状态")
    private Integer sortingStatus;

    @ExcelProperty(value = "出仓状态")
    @ApiModelProperty(value = "出仓状态")
    private Integer checkoutStatus;

    //提单状态
    private Integer ladingBillStatus;

    @ApiModelProperty(value = "正在审核的类型")
    private Integer approvalType;

    @ApiModelProperty(value = "是否正在审核")
    private boolean isApprovaling = false;

    @ApiModelProperty(value = "目的国ID")
    private Long destCountryId;

    public String getShipmentStatusText() {
        if (backApprovalType != null) {
            shipmentStatusText = BoxApprovalTypeEnum.getApprovalTypeByTypeValue(backApprovalType).getText();
        } else {
            if (TransportTypeEnum.AIR.getType().equals(transportType)) {
                if (null != shipmentStatusAir) {
                    shipmentStatusText = I18nMessage.getMessage(BoxAirStatusEnum.getStatusEnumByStatus(shipmentStatusAir).getText());
                }
            } else {
                if (null != shipmentStatus) {
                    shipmentStatusText = I18nMessage.getMessage(BoxStatusEnum.getStatusEnumByStatus(shipmentStatus).getText());
                }
            }
        }
        return shipmentStatusText;
    }

    public void setShipmentStatusText(String shipmentStatusText) {
        this.shipmentStatusText = shipmentStatusText;
    }
}
