package cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.module.order.dto.OrderRemindExceptionDto;
import cn.iocoder.yudao.module.order.enums.OrderAbnormalStateEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.enums.OrderStatusMsgEnum;
import cn.iocoder.yudao.module.order.vo.order.GuanLianOrderStatusVO;
import cn.iocoder.yudao.module.order.vo.order.WarehouseInInfoVO;
import cn.iocoder.yudao.module.order.vo.orderLocation.OrderLocationBackVO;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
* 预装货物 Response VO
* @author wlh
*/
@Data
@ApiModel("管理后台 - 预装货物 Response VO")
public class BoxPreloadGoodsBackVO {

    @ExcelProperty("")
    @ApiModelProperty(value = "", required = true)
    private Long id;

    @ExcelProperty("出货ID")
    @ApiModelProperty(value = "出货ID")
    private Long shipmentId;

    @ExcelProperty("部分ID")
    @ApiModelProperty(value = "部分ID")
    private Long secId;

    @ExcelProperty("订单ID")
    @ApiModelProperty(value = "订单ID")
    private Long orderId;

    @ExcelProperty("订单项ID")
    @ApiModelProperty(value = "订单项ID")
    private Long orderItemId;

    @ApiModelProperty("订单编号")
    private String orderNo;

    @ApiModelProperty("运输方式ID")
    private Integer transportId;

    @ApiModelProperty(value = "始发仓ID")
    private Long startWarehouseId;

    @ApiModelProperty(value = "目的仓ID")
    private Long destWarehouseId;

    @ApiModelProperty(value = "始发仓名称")
    private String startWarehouseName;

    @ApiModelProperty(value = "调仓目的地")
    private Long adjustToDestWarehouseId;

    @ApiModelProperty(value = "入仓类型")
    private Integer warehouseType;

    @ApiModelProperty(value = "商品特性ID列表，多个以,连接,eg:2,3,1")
    private String prodAttrIds;

    @ApiModelProperty(value = "入仓商品特性")
    private String warehouseInProdAttrIds;

    @ApiModelProperty(value = "入仓商品特性名称列表")
    private List<String> warehouseInAttrNameList;

    @ApiModelProperty(value = "商品特性名称列表")
    private List<String> attrNameList;

    @ApiModelProperty(value = "目的仓名称")
    private String destWarehouseName;

    @ApiModelProperty(value = "总货值")
    private BigDecimal totalWorth;

    @ApiModelProperty(value = "货值")
    private BigDecimal worth;

    @ApiModelProperty(value = "是否是外部仓,1是")
    private Integer isExternalWarehouse;

    @ApiModelProperty(value = "入仓时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date rucangTime;

    @ApiModelProperty(value = "报关方式")
    private Integer customsType;

    @ApiModelProperty(value = "备案属性")
    private Integer productRecord;

    @ApiModelProperty(value = "产品中文标题")
    private String prodTitleZh;

    @ApiModelProperty(value = "产品英文标题")
    private String prodTitleEn;

    @ApiModelProperty(value = "0 无牌 1 有牌 2 中性")
    private Integer brandType;

    @ApiModelProperty(value = "无牌价0，有牌价1，中性品牌价2")
    private Integer feeType;

    @ApiModelProperty(value = "品牌ID")
    private Long brand;

    private String brandName;

    @ApiModelProperty(value = "箱规")
    private String boxGauge;

    @ApiModelProperty(value = "是否多规格")
    private Boolean multiSpecification;

    @ApiModelProperty(value = "运费货币单位")
    private Integer seaFreightCurrency;

    @ApiModelProperty(value = "清关费货币单位")
    private Integer clearanceFreightCurrency;

    @ApiModelProperty(value = "运费体积单位")
    private Integer seaFreightVolume;

    @ApiModelProperty(value = "清关费体积单位")
    private Integer clearanceFreightVolume;

    @ApiModelProperty(value = "数量")
    private Integer quantity;

    @ApiModelProperty(value = "件数")
    private Integer num;

    @ApiModelProperty(value = "单位")
    private String unit;

    @ApiModelProperty(value = "体积")
    private BigDecimal volume;

    @ApiModelProperty(value = "重量")
    private BigDecimal weight;

    @ApiModelProperty(value = "收费体积")
    BigDecimal chargeVolume = BigDecimal.ZERO;

    @ApiModelProperty(value = "收费重量")
    BigDecimal chargeWeight = BigDecimal.ZERO;

    @ApiModelProperty(value = "材质")
    private String material;

    @ApiModelProperty(value = "垫付类型(多个):10内陆运输费，20打木架/托，30纸箱打包")
    private String advanceType;

    private Integer tidanNum;

    private String warehouseInInfo;

    private WarehouseInInfoVO warehouseInInfoVO;

    @ApiModelProperty(value = "合包id")
    private Long pkgId;

    @ApiModelProperty(value = "合包箱号")
    private String pkgNum;

    private WarehouseInInfoVO getWarehouseInInfoVO() {
        if(warehouseInInfoVO != null) return warehouseInInfoVO;
        if(StringUtils.isNotEmpty(this.warehouseInInfo)) {
            warehouseInInfoVO = JSONObject.parseObject(this.warehouseInInfo, WarehouseInInfoVO.class);
        }
        return warehouseInInfoVO;
    }

    public Integer getQuantity() {
        if(getWarehouseInInfoVO() != null) {
            quantity = getWarehouseInInfoVO().getQuantityAll();
        }
        return quantity;
    }

    public Integer getNum() {
        if(getWarehouseInInfoVO() != null) {
            num = getWarehouseInInfoVO().getCartonsNum();
        }
        return num;
    }

    public BigDecimal getVolume() {
        if(getWarehouseInInfoVO() != null) {
            volume = getWarehouseInInfoVO().getVolume();
        }
        return volume;
    }

    public BigDecimal getWeight() {
        if(getWarehouseInInfoVO() != null) {
            weight = getWarehouseInInfoVO().getWeight();
        }
        return weight;
    }

    public Date getRucangTime() {
        if(getWarehouseInInfoVO() != null) {
            rucangTime = getWarehouseInInfoVO().getFirstInTime();
        }
        return rucangTime;
    }

        public String getBoxGauge() {
        if(getWarehouseInInfoVO() != null) {
            boxGauge = getWarehouseInInfoVO().getBoxGauge();
        }
        return boxGauge;
    }

    public Boolean getMultiSpecification() {
        if(getWarehouseInInfoVO() != null) {
            multiSpecification = getWarehouseInInfoVO().getIsMultiSpecification();
        }
        return multiSpecification;
    }

    @ApiModelProperty(value = "储位")
    private String positionNo;

    @ApiModelProperty(value = "储位列表")
    private List<OrderLocationBackVO> orderItemLocationList;

    @ExcelProperty("订单状态详情见字典")
    @ApiModelProperty(value = "订单状态详情见字典：order_status")
    private Integer status;

    @ExcelProperty("订单异常状态")
    @ApiModelProperty(value = "订单异常状态(字典 order_abnormal_state)")
    private Integer abnormalState;

    @ExcelProperty("订单入仓状态")
    @ApiModelProperty(value = "订单入仓状态(字典 order_warehouse_in_status)")
    private Integer inWarehouseState;

    @ExcelProperty("订单出货状态")
    @ApiModelProperty(value = "订单出货状态(字典 order_shipment_state)")
    private Integer shipmentState;

    @ExcelProperty("审核类型(字典 order_approval_type) 0 为正常")
    @ApiModelProperty(value = "审核类型(字典 order_approval_type) 0 为正常")
    private Integer auditType;

    @ExcelProperty("审核结果")
    @ApiModelProperty(value = "审核结果")
    private String auditResult;

    @ApiModelProperty(value = "空运可出货状态： 0 默认值 1 待出 2 可出 3 备货中 4 已备货")
    private Integer airShipment;

    @ApiModelProperty(value = "不阻塞流程的异常说明(支持多个异常说明)")
    private String exceptionReason;

    @ApiModelProperty(value = "不阻塞流程的异常说明(支持多个异常说明)")
    private List<OrderRemindExceptionDto> remindExceptionDtoList;

    /**
     * 状态说明
     */
    @ApiModelProperty(value = "状态说明")
    private String statusMsg;

    /**
     * 关联订单状态
     */
    @ApiModelProperty(value = "关联订单状态")
    private String guanLianOrderStatus;

    @ApiModelProperty(value = "关联订单状态")
    private List<GuanLianOrderStatusVO> guanLianOrderStatusVOList;

    /**
     * 端口 {@link UserTypeEnum}
     */
    @ApiModelProperty(value = "端口(默认管理端)")
    private Integer userType = 2;

    @ApiModelProperty(value = "语言")
    private Integer lang;


    public void setGuanLianOrderStatus(String guanLianOrderStatus) {
        this.guanLianOrderStatus = guanLianOrderStatus;
        if (StringUtils.isNotBlank(guanLianOrderStatus)) {
            this.guanLianOrderStatusVOList = JSONObject.parseArray(guanLianOrderStatus, GuanLianOrderStatusVO.class);
            this.getOrderDesc();
        }
    }

    public void setGuanLianOrderStatusVOList(List<GuanLianOrderStatusVO> guanLianOrderStatusVOList) {
        this.guanLianOrderStatusVOList = guanLianOrderStatusVOList;
        if (CollectionUtil.isNotEmpty(guanLianOrderStatusVOList)) {
            this.guanLianOrderStatus = JSONObject.toJSONString(guanLianOrderStatusVOList);
        }
    }


    public void setOrderNo(String orderNo) {
        this.orderNo = orderNo.contains("DRAFT") ? null : orderNo;
    }

    public void setStatus(Integer status) {
        this.status = status;
        this.getOrderDesc();

    }

    public void setAbnormalState(Integer abnormalState) {
        this.abnormalState = abnormalState;
        this.getOrderDesc();
    }

    public void setInWarehouseState(Integer inWarehouseState) {
        this.inWarehouseState = inWarehouseState;
        this.getOrderDesc();
    }

    public void setShipmentState(Integer shipmentState) {
        this.shipmentState = shipmentState;
        this.getOrderDesc();
    }

    public void setAuditType(Integer auditType) {
        this.auditType = auditType;
        this.getOrderDesc();
    }

    public void setAuditResult(String auditResult) {
        this.auditResult = auditResult;
        this.getOrderDesc();

    }

    public void setLang(Integer lang) {
        this.lang = lang;
        if (StringUtils.isNotBlank(this.exceptionReason) && Objects.nonNull(lang)) {
            this.remindExceptionDtoList = JSONObject.parseArray(this.exceptionReason, OrderRemindExceptionDto.class)
                    .stream()
                    .peek(ern -> ern.setReason(lang == 0 ? I18nMessage.getZhMessage(ern.getReason()) : I18nMessage.getEnMessage(ern.getReason())))
                    .collect(Collectors.toList());
        }
        this.getOrderDesc();
    }

    public void setExceptionReason(String exceptionReason) {
        this.exceptionReason = exceptionReason;
        if (StringUtils.isNotBlank(exceptionReason) && Objects.nonNull(this.lang)) {
            this.remindExceptionDtoList = JSONObject.parseArray(exceptionReason, OrderRemindExceptionDto.class)
                    .stream()
                    .peek(ern -> ern.setReason(lang == 0 ? I18nMessage.getZhMessage(ern.getReason()) : I18nMessage.getEnMessage(ern.getReason())))
                    .collect(Collectors.toList());
        }
        this.getOrderDesc();
    }

    private void getOrderDesc() {
        // TODO 这里的文案国际化现在只支持两种语言，如果需要支持更多的语言，需要优化
        Integer lang = Objects.nonNull(this.lang) ? this.lang : I18nMessage.getLang();
        if (null != this.status && this.status.equals(OrderStatusEnum.CANCEL.getValue()) && this.userType != null) {
            String message = this.userType == 2 ? OrderStatusMsgEnum.CANCEL.getAdminSourceName() : OrderStatusMsgEnum.CANCEL.getCustomerSourceName();
            this.statusMsg = lang == 0 ? I18nMessage.getZhMessage(message) : I18nMessage.getEnMessage(message);
        } else if (null != this.abnormalState && this.abnormalState > 0) {
            OrderAbnormalStateEnum abnormalStateEnum = OrderAbnormalStateEnum.valueOf(abnormalState);
            if (Objects.isNull(abnormalStateEnum)) {
                this.statusMsg = lang == 0 ? "未知异常" : "UNKNOWN_EXCEPTION";
            } else {
                this.statusMsg = lang == 0 ? I18nMessage.getZhMessage(abnormalStateEnum.getDesc()) : I18nMessage.getEnMessage(abnormalStateEnum.getDesc());
            }
        } else if (StringUtils.isNotBlank(this.auditResult)) {
            this.statusMsg = lang == 0 ? I18nMessage.getZhMessage(auditResult) : I18nMessage.getEnMessage(auditResult);
        } else if (CollectionUtil.isNotEmpty(this.remindExceptionDtoList) && this.remindExceptionDtoList.size() > 0) {
            this.statusMsg = remindExceptionDtoList.get(0).getReason();
        } else if (null != this.status && this.status == 5 && CollectionUtil.isNotEmpty(this.guanLianOrderStatusVOList)) {
            // 已入仓的订单才展示关联异常信息
            // 如果查询返回了关联订单状态，表示需要显示关联订单状态，关联订单状态冗余的是国际化key值
            // 排序
            this.guanLianOrderStatusVOList = this.guanLianOrderStatusVOList.stream().sorted(Comparator.comparing(GuanLianOrderStatusVO::getType)).collect(Collectors.toList());
            GuanLianOrderStatusVO guanlianOrderStatusVO = guanLianOrderStatusVOList.get(0);
            switch (guanlianOrderStatusVO.getType()) {
                case 1:
                    // 关联异常状态封装，订单状态显示“关联单#订单号#，#异常名称#”Associated order %s, %s
                    String abnormalMsg = lang == 0 ? I18nMessage.getZhMessage("guanlian.order.abnormalState") : I18nMessage.getEnMessage("guanlian.order.abnormalState");
                    this.statusMsg = String.format(abnormalMsg, guanlianOrderStatusVO.getOrderNo(), I18nMessage.getMessage(guanlianOrderStatusVO.getStatusMsg()));
                    break;
                case 2:
                    // 关联审批状态封装，订单状态显示“关联单#订单号##审核状态#”
                    String auditMsg = lang == 0 ? I18nMessage.getZhMessage("guanlian.order.audit") : I18nMessage.getEnMessage("guanlian.order.audit");
                    this.statusMsg = String.format(auditMsg, guanlianOrderStatusVO.getOrderNo(), I18nMessage.getMessage(guanlianOrderStatusVO.getStatusMsg()));
                    break;
                case 3:
                    // 关联未入仓状态封装，订单状态显示“关联单#订单号#未入仓”
                    String waitWarehouseInMsg = lang == 0 ? I18nMessage.getZhMessage(guanlianOrderStatusVO.getStatusMsg()) : I18nMessage.getEnMessage(guanlianOrderStatusVO.getStatusMsg());
                    this.statusMsg = String.format(waitWarehouseInMsg, guanlianOrderStatusVO.getOrderNo());
                    break;
                default:
                    this.statusMsg = lang == 0 ? I18nMessage.getZhMessage(guanlianOrderStatusVO.getStatusMsg()) : I18nMessage.getEnMessage(guanlianOrderStatusVO.getStatusMsg());
            }
        } else if (null != this.status && null != this.inWarehouseState && null != this.shipmentState && null != this.airShipment && null != this.transportId) {
            this.statusMsg = OrderStatusMsgEnum.getMsg(this.status, this.inWarehouseState, this.shipmentState, this.airShipment, this.transportId, this.userType, lang);
        }
    }

    @ApiModelProperty(value = "混箱状态, 1-混箱")
    private Integer mixStatus = 0;

    public Integer getMixStatus() {
        WarehouseInInfoVO warehouseInInfoVO = getWarehouseInInfoVO();
        if(warehouseInInfoVO != null) {
            //入仓纪录明细不为空则是混箱
            mixStatus = warehouseInInfoVO.getMixStatus();
        }
        return mixStatus;
    }
}
