package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.order.dal.dataobject.approval.OrderApprovalDO;
import cn.iocoder.yudao.module.order.enums.ApprovalResultStatusEnum;
import cn.iocoder.yudao.module.order.enums.OrderApprovalTypeEnum;
import cn.iocoder.yudao.module.order.service.approval.OrderApprovalService;
import cn.iocoder.yudao.module.order.service.orderSplit.OrderSplitService;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitApprovalBackVO;
import cn.iocoder.yudao.module.order.vo.orderSplit.OrderSplitBackVO;
import cn.iocoder.yudao.module.shipment.convert.BoxApprovalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.enums.BoxApprovalTypeEnum;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.vo.boxApproval.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 出货审核")
@RequestMapping("/ecw/box-approval")
public class BoxApprovalController {

    @Resource
    private BoxApprovalService boxApprovalService;

    @Resource
    private OrderSplitService splitService;

    @Resource
    private OrderApprovalService orderApprovalService;

    @PostMapping("/create")
    @ApiOperation("创建出货审核， use this")
    @Idempotent(timeout = 10)
//    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:create')")
    public CommonResult<Long> createBoxApproval(@Valid @RequestBody BoxApprovalCreateReqVO createReqVO) {
        return success(boxApprovalService.createBoxApproval(createReqVO));
    }

    @PostMapping("/approvalDetail")
    @ApiImplicitParam(name = "approvalId", value = "编号", required = true, example = "1", dataTypeClass = Long.class)
    @ApiOperation("审核详情， use this")
    public CommonResult<BoxLoadDetailVO> approvalDetail(@RequestParam("approvalId") Long approvalId) {
        return success(boxApprovalService.approvalDetail(approvalId));
    }



    @GetMapping("/list-by-order-id")
    @ApiOperation("通过订单号获取拆单列表和拆单项明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "lang", value = "语言 0中文 1英文", required = true, example = "0", dataTypeClass = Integer.class)
    })
    public CommonResult<OrderSplitApprovalBackVO> listByOrderId(@RequestParam("orderId") Long orderId, @RequestParam("lang") Integer lang) {
        List<OrderSplitBackVO> list = splitService.listByOrderId(orderId, I18nMessage.getLang());

        String formId = null;

        //普通 拆单申请
        OrderApprovalDO orderApprovalDO = orderApprovalService.getLast(orderId, OrderApprovalTypeEnum.ORDER_SPLIT);

        //预装/部分退场 折单申请
        BoxApprovalDO boxApprovalDO = null;
        List<BoxApprovalDO> boxApprovalDOList = boxApprovalService.list(new LambdaQueryWrapperX<BoxApprovalDO>().eq(BoxApprovalDO::getOrderId, orderId)
                .in(BoxApprovalDO::getApprovalType, BoxApprovalTypeEnum.SPLIT_ORDER.getType(),BoxApprovalTypeEnum.PART_REFUND_SPLIT_ORDER.getType())
                .eq(BoxApprovalDO::getApprovalStatus, ApprovalResultStatusEnum.processing.getValue()));
        if (CollectionUtil.isNotEmpty(boxApprovalDOList)) {
            boxApprovalDO = boxApprovalDOList.stream().sorted((o1, o2) -> o2.getId().compareTo(o1.getId())).findFirst().get();
        }
        if (orderApprovalDO != null && ApprovalResultStatusEnum.processing.getValue().equals(orderApprovalDO.getStatus()) && boxApprovalDO == null) {
            formId = orderApprovalDO.getFormId();
        } else if (orderApprovalDO == null && boxApprovalDO != null && ApprovalResultStatusEnum.processing.getValue().equals(boxApprovalDO.getApprovalStatus())) {
            formId = boxApprovalDO.getBpmProcessId();
        }

        OrderSplitApprovalBackVO orderSplitApprovalBackVO = new OrderSplitApprovalBackVO();
        orderSplitApprovalBackVO.setApplyingFormId(formId);
        orderSplitApprovalBackVO.setOrderSplitBackVOList(list);
        return success(orderSplitApprovalBackVO);
    }




    @PutMapping("/update")
    @ApiOperation("更新出货审核")
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:update')")
    public CommonResult<Boolean> updateBoxApproval(@Valid @RequestBody BoxApprovalUpdateReqVO updateReqVO) {
        boxApprovalService.updateBoxApproval(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/cancel")
    @ApiOperation("取消申请, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelApproval(@RequestBody BoxApprovalUpdateReqVO updateReqVO) {
        boxApprovalService.cancelApproval(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除出货审核")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:delete')")
    public CommonResult<Boolean> deleteBoxApproval(@RequestParam("id") Long id) {
        boxApprovalService.deleteBoxApproval(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得出货审核")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:query')")
    public CommonResult<BoxApprovalBackVO> getBoxApproval(@RequestParam("id") Long id) {
        BoxApprovalDO boxApproval = boxApprovalService.getBoxApproval(id);
        return success(BoxApprovalConvert.INSTANCE.convert(boxApproval));
    }

    @GetMapping("/list")
    @ApiOperation("获得出货审核列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:query')")
    public CommonResult<List<BoxApprovalBackVO>> getBoxApprovalList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxApprovalDO> list = boxApprovalService.getBoxApprovalList(ids);
        return success(BoxApprovalConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得出货审核分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:query')")
    public CommonResult<PageResult<BoxApprovalBackVO>> getBoxApprovalPage(@Valid BoxApprovalQueryVO query, PageVO page) {
        PageResult<BoxApprovalDO> pageResult = boxApprovalService.getBoxApprovalPage(query, page);
        return success(BoxApprovalConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出出货审核 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-approval:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxApprovalExcel(@Valid BoxApprovalQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxApprovalDO> list = boxApprovalService.getBoxApprovalList(query);
        // 导出 Excel
        List<BoxApprovalBackVO> datas = BoxApprovalConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "出货审核.xls", "数据", BoxApprovalBackVO.class, datas);
    }
}
