package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCostConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCostDO;
import cn.iocoder.yudao.module.shipment.service.boxCost.BoxCostService;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCost.BoxCostUpdateReqVO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.service.payable.PayableService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 费用登记")
@RequestMapping("/ecw/box-cost")
public class BoxCostController {

    @Resource
    private BoxCostService boxCostService;

    @Resource
    private PayableService payableService;

    @PostMapping("/create")
    @ApiOperation("创建费用登记, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-cost:create')")
    public CommonResult<Long> createBoxCost(@Valid @RequestBody BoxCostCreateReqVO createReqVO) {
        return success(boxCostService.createBoxCost(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新费用登记, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-cost:update')")
    public CommonResult<Boolean> updateBoxCost(@Valid @RequestBody BoxCostUpdateReqVO updateReqVO) {
        boxCostService.updateBoxCost(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除费用登记")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-cost:delete')")
    public CommonResult<Boolean> deleteBoxCost(@RequestParam("id") Long id) {
        boxCostService.deleteBoxCost(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得费用登记")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-cost:query')")
    public CommonResult<BoxCostBackVO> getBoxCost(@RequestParam("id") Long id) {
        BoxCostDO boxCost = boxCostService.getBoxCost(id);
        return success(BoxCostConvert.INSTANCE.convert(boxCost));
    }

    @GetMapping("/list")
    @ApiOperation("获得费用登记列表, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-cost:query')")
    public CommonResult<List<BoxCostBackVO>> getBoxCostList(@Valid BoxCostQueryVO query) {
        List<BoxCostDO> list = boxCostService.getBoxCostList(query);

        List<BoxCostBackVO> costList =  BoxCostConvert.INSTANCE.convertList(list);

        //找到对应的核销记录
//        Map<Long, List<>>
        if(CollectionUtil.isNotEmpty(costList)) {
            List<Long> idList = costList.stream()
                    .map(BoxCostBackVO::getId)
                    .collect(Collectors.toList());
            List<PayableDO> payableList = payableService.list(new LambdaQueryWrapperX<PayableDO>()
                .in(PayableDO::getShippingFeeId, idList)
            );
            if(CollectionUtil.isNotEmpty(payableList)) {
                Map<Long, PayableDO> payableMap = payableList.stream()
                        .collect(Collectors.toMap(PayableDO::getShippingFeeId, t -> t));
                for (BoxCostBackVO boxCostBackVO : costList) {
                    PayableDO payableDO = payableMap.get(boxCostBackVO.getId());
                    if(payableDO != null) {
                        boxCostBackVO.setPayPrice(payableDO.getTotal());
                        boxCostBackVO.setPayTime(payableDO.getUpdateTime());
                    }
                }
            }
        }
        return success(costList);
    }

    @GetMapping("/page")
    @ApiOperation("获得费用登记分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-cost:query')")
    public CommonResult<PageResult<BoxCostBackVO>> getBoxCostPage(@Valid BoxCostQueryVO query, PageVO page) {
        PageResult<BoxCostDO> pageResult = boxCostService.getBoxCostPage(query, page);
        return success(BoxCostConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出费用登记 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-cost:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCostExcel(@Valid BoxCostQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCostDO> list = boxCostService.getBoxCostList(query);
        // 导出 Excel
        List<BoxCostBackVO> datas = BoxCostConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "费用登记.xls", "数据", BoxCostBackVO.class, datas);
    }
}
