package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.number.NumberUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.ecw.dal.dataobject.cabinet.CabinetDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.service.cabinet.CabinetService;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderGuanlian.OrderGuanlianDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderItem.OrderItemDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLocation.OrderLocationDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderWarehouseIn.OrderWarehouseInDO;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.service.orderWarehouseIn.OrderWarehouseInService;
import cn.iocoder.yudao.module.order.vo.order.OrderPreloadReq;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackItemDo;
import cn.iocoder.yudao.module.order.vo.orderWarehouseIn.OrderWarehouseInBackVO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrank.ProductBrankDO;
import cn.iocoder.yudao.module.product.service.productbrank.ProductBrankService;
import cn.iocoder.yudao.module.shipment.convert.BoxPreloadGoodsConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDocDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadGoodsDO;
import cn.iocoder.yudao.module.shipment.dto.LoadDto;
import cn.iocoder.yudao.module.shipment.dto.PreloadDto;
import cn.iocoder.yudao.module.shipment.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.shipment.enums.TyStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import cn.iocoder.yudao.module.shipment.service.boxClearanceDoc.BoxClearanceDocService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.utils.CustomMergeStrategy;
import cn.iocoder.yudao.module.shipment.vo.box.PreloadResultVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.*;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadDetailBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadSectionBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionDetailVO;
import cn.iocoder.yudao.module.shipment.vo.boxTally.BoxOrderRemoveReq;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableOrderPayedFeeTypeVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.WriteTable;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.*;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.FILE_NOT_EXISTS;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;

@Validated
@RestController
@Api(tags = "管理后台 - 预装货物")
@RequestMapping("/ecw/box-preload-goods")
public class BoxPreloadGoodsController {

    @Value("${ue.templates-url}")
    private String path;

    @Value("${ue.root-path}")
    private String rootPath;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private BoxService boxService;

    @Resource
    private BoxClearanceDocService boxClearanceDocService;

    @Resource
    private FileService fileService;

    @Resource
    private FileMakeApi fileMakeApi;

    @PostMapping("/create")
    @ApiOperation("创建预装货物, use this")
    @Idempotent(timeout = 5)
    public CommonResult<PreloadResultVO> createBoxPreloadGoods(@Valid @RequestBody BoxPreloadGoodsCreateReqVO createReqVO) {
        return success(boxPreloadGoodsService.createBoxPreloadGoods(createReqVO));
    }

    @PostMapping("/v2/create")
    @ApiOperation("创建预装货物, use this")
    @Idempotent(timeout = 5)
    public CommonResult<PreloadResultVO> createBoxPreloadGoodsV2(@Valid @RequestBody BoxPreloadGoodsCreateReqVO createReqVO) {
        return success(boxPreloadGoodsService.createBoxPreloadGoodsV2(createReqVO));
    }

    @PostMapping("/loadRelationOrder")
    @ApiOperation("预装关联订单, use this")
    public CommonResult<Boolean> loadRelationOrder(@Valid @RequestBody PreloadResultVO createReqVO) {
        throw new ServiceException(500, "接口已废弃");
//        boxPreloadGoodsService.loadRelationOrder(createReqVO);
//        return success(true);
    }

    @PostMapping("/preloadPage")
    @ApiOperation("获得待预装分页，预装页面use this")
    public CommonResult<BoxOrderPreloadStatistics> getOrderPreloadPage(@RequestBody OrderPreloadReq query) {
        BoxOrderPreloadStatistics pageResult = boxPreloadGoodsService.orderPreloadPage(query);
        return success(pageResult);
    }

    @PostMapping("/preloadPageAir")
    @ApiOperation("获得待预装分页-空运，预装页面use this")
    public CommonResult<BoxOrderPreloadStatistics> getOrderPreloadPageAir(@RequestBody OrderPreloadReq query) {
        BoxOrderPreloadStatistics pageResult = boxPreloadGoodsService.orderPreloadPageAir(query);
        return success(pageResult);
    }

    @PostMapping("/createAir")
    @ApiOperation("创建预装货物-空运, use this")
    public CommonResult<PreloadResultVO> createBoxPreloadGoodsAir(@Valid @RequestBody BoxPreloadGoodsCreateReqVO createReqVO) {
        return success(boxPreloadGoodsService.createBoxPreloadGoodsAir(createReqVO));
    }

    @PostMapping("/secGoodsList")
    @ApiOperation("获得已预装部分列表, 预装页面use this")
    public CommonResult<BoxPreloadSectionDetailVO> getBoxPreloadSectionList(@RequestBody PreloadedReq query) {
        BoxPreloadSectionDetailVO data = boxPreloadGoodsService.getBoxPreloadSectionList(query);
        return success(data);
    }

    @PostMapping("/orderItems")
    @ApiOperation("获得已预装商品项, use this")
    public CommonResult<List<BoxPreloadGoodsBackVO>> orderItems(@RequestBody PreloadedReq query) {
        query.setLang(I18nMessage.getLang());
        return success(boxPreloadGoodsService.orderItems(query));
    }

    @GetMapping("/loadSecGoodsList")
    @ApiOperation("获得装柜部分列表，装柜页面数据use this")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<List<BoxLoadSectionBackVO>> loadSecGoodsList(@RequestParam("shipmentId") Long shipmentId) {
        List<BoxLoadSectionBackVO> list = boxPreloadGoodsService.loadSecGoodsList(shipmentId);
        return success(list);
    }

    @GetMapping("/downloadAgentListFiles")
    @ApiOperation("下载agent list文件")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadAgentListFiles(@RequestParam("shipmentId") Long shipmentId) {
        BoxClearanceDocQueryVO queryVO = new BoxClearanceDocQueryVO();
        queryVO.setShipmentId(shipmentId);
        List<BoxClearanceDocDO> boxClearanceDocList = boxClearanceDocService.getBoxClearanceDocList(queryVO);
        if (!CollectionUtil.isEmpty(boxClearanceDocList)) {
            BoxClearanceDocDO boxClearanceDocDO = boxClearanceDocList.get(0);
            String cdAgentlistFile = boxClearanceDocDO.getCdAgentlistFile();
            if (StringUtils.isNotBlank(cdAgentlistFile)) {
                return success(cdAgentlistFile);
            }
        }
        return error(FILE_NOT_EXISTS);
    }

    @GetMapping("/downloadSoncapFiles")
    @ApiOperation("下载son cap文件")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadSoncapFiles(@RequestParam("shipmentId") Long shipmentId) {
        BoxClearanceDocQueryVO queryVO = new BoxClearanceDocQueryVO();
        queryVO.setShipmentId(shipmentId);
        List<BoxClearanceDocDO> boxClearanceDocList = boxClearanceDocService.getBoxClearanceDocList(queryVO);
        if (!CollectionUtil.isEmpty(boxClearanceDocList)) {
            BoxClearanceDocDO boxClearanceDocDO = boxClearanceDocList.get(0);
            String cdSoncapFile = boxClearanceDocDO.getCdSoncapFile();
            if (StringUtils.isNotBlank(cdSoncapFile)) {
                return success(cdSoncapFile);
            }
        }
        return error(FILE_NOT_EXISTS);
    }

//    @NotNull
//    private List<InputStream> getInputStreams(List<String> agentListFiles) {
//        return agentListFiles.stream().map(item -> {
//            HttpURLConnection conn = null;
//            InputStream inputStream = null;
//            try {
//                //url解码
//                URL url = new URL(URLDecoder.decode(item, "UTF-8"));
//                conn = (HttpURLConnection) url.openConnection();
//                //设置超时间为3秒
//                conn.setConnectTimeout(3 * 1000);
//                //防止屏蔽程序抓取而返回403错误
//                conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
//                //得到输入流
//                inputStream = conn.getInputStream();
//            } catch (Exception e) {
//                e.printStackTrace();
//            } finally {
//                if (inputStream != null) {
//                    try {
//                        inputStream.close();
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//                }
//            }
//            return inputStream;
//        }).collect(Collectors.toList());
//    }

    @GetMapping("/downloadSeaPreloadGoodsList")
    @ApiOperation("下载海运预装单")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadSeaPreloadGoodsList(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_SEA_PRELOAD_EXCEL_EXPORT.getType());
        reqDTO.setName("预装单("+box.getSelfNo()+")");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadPreloadGoodsList")
    @ApiOperation("下载预装单")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadPreloadGoodsList(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        if(StringUtils.equals(box.getTransportType(), TransportTypeEnum.SEA_COMBINED_CABINET.getType())){
            reqDTO.setType(DownloadTypeEnum.SHIPMENT_SEA_PRELOAD_EXCEL_EXPORT.getType());
        }else {
            reqDTO.setType(DownloadTypeEnum.SHIPMENT_PRELOAD_EXCEL_EXPORT.getType());
        }
        reqDTO.setName("预装单("+box.getSelfNo()+")");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadLoadGoodsList")
    @ApiOperation("下载已装单")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadLoadGoodsList(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_LOAD_EXCEL_EXPORT.getType());
        reqDTO.setName("已装单("+box.getSelfNo()+")");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadAirLoadGoodsList")
    @ApiOperation("下载空运已装单")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadAirLoadGoodsList(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_AIR_LOAD_EXCEL_EXPORT.getType());
        reqDTO.setName("已装单("+box.getSelfNo()+")");
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadReceivableList")
    @ApiOperation("下载应收汇总")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult downloadReceivableList(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        reqDTO.setType(DownloadTypeEnum.SHIPMENT_RECEIVABLE_EXCEL_EXPORT.getType());
        reqDTO.setName("manifest form China-" + box.getSelfNo());
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/downloadAirBillReceivable")
    @ApiOperation("下载空运应收汇总")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult downloadAirBillReceivable(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) throws Exception {
        BoxDO box = boxService.getBox(shipmentId);
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("shipmentId",shipmentId);
        reqDTO.setType(DownloadTypeEnum.AIR_BILL_RECEIVABLE_EXCEL_EXPORT.getType());
        reqDTO.setName("manifest form China-" + box.getSelfNo());
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(jsonObject.toJSONString());
        fileMakeApi.sendFileMake(reqDTO);
        return success("");
    }

    @GetMapping("/loadGoodsList")
    @ApiOperation("获得装柜订单列表，卸柜页面数据use this")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "secId", value = "部分ID", required = false, example = "20", dataTypeClass = Long.class)
    })
    public CommonResult<BoxLoadDetailBackVO> loadGoodsList(@RequestParam("shipmentId") Long shipmentId, @RequestParam("secId") Long secId) {
        BoxLoadDetailBackVO boxLoadDetailBackVO = boxPreloadGoodsService.loadGoodsList(shipmentId, secId);
        return success(boxLoadDetailBackVO);
    }

    @PostMapping("/remove")
    @ApiOperation("移出,理货页面，use this")
    public CommonResult<Boolean> remove(@RequestBody BoxOrderRemoveReq boxOrderRemoveReq) {
        //已经理货则不能移出
        BoxDO boxDO = boxService.getBox(boxOrderRemoveReq.getShipmentId());
        if(boxDO.getTyStatus() == TyStatueEnum.HAS_TALLY.getTyStatus()) {
            throw new ServiceException(500, "已完成理货，不能移出");
        }
        boxPreloadGoodsService.remove(boxOrderRemoveReq.getShipmentId(), boxOrderRemoveReq.getOrderIdLIst(), boxOrderRemoveReq.getDeleteType(), false);
        return success(true);
    }


    @DeleteMapping("/delete")
    @ApiOperation("删除预装货物, use this")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "编号", required = true, example = "1024,2048", dataTypeClass = List.class),
            @ApiImplicitParam(name = "deleteType", value = "删除类型(1-不删除关联订单，2-删除关联订单,默认可以不传，不传则会弹窗提示关联单)", example = "1", dataTypeClass = List.class)
    })
    public CommonResult<Boolean> deleteBoxPreloadGoods(@RequestParam("ids") Collection<Long> ids, @RequestParam(value = "deleteType", required = false) Integer deleteType) {
        boxPreloadGoodsService.deleteBoxPreloadGoods(ids, deleteType);
        return success(true);
    }

    @DeleteMapping("/deleteSection")
    @ApiOperation("删除预装部分, use this")
    @ApiImplicitParams({
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class),
    @ApiImplicitParam(name = "deleteType", value = "删除类型(1-不删除关联订单，2-删除关联订单,默认可以不传，不传则会弹窗提示关联单)", example = "1", dataTypeClass = List.class)
    })
    public CommonResult<Boolean> deleteBoxPreloadSection(@RequestParam("id") Long id, @RequestParam(value = "deleteType", required = false) Integer deleteType) {
        boxPreloadGoodsService.deleteBoxPreloadSection(id, deleteType);
        return success(true);
    }

    @PostMapping("/changeSection")
    @ApiOperation("变换预装部分, use this")
    public CommonResult<Boolean> changeSection(@RequestBody PreloadChange params) {
        boxPreloadGoodsService.changeSection(params);
        return success(true);
    }

    @PostMapping("/boxGoodsDetail")
    @ApiOperation("装柜订单明细, use this")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "secId", value = "部分ID", required = false, example = "20", dataTypeClass = Long.class)
    })
    public CommonResult<BoxLoadDetailBackVO> boxGoodsDetail(@RequestParam("shipmentId") Long shipmentId, @RequestParam("secId") Long secId) {
        BoxLoadDetailBackVO boxLoadDetailBackVO = boxPreloadGoodsService.boxGoodsDetail(shipmentId, secId);
        return success(boxLoadDetailBackVO);
    }

    @GetMapping("/get")
    @ApiOperation("获得预装货物")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-goods:query')")
    public CommonResult<BoxPreloadGoodsBackVO> getBoxPreloadGoods(@RequestParam("id") Long id) {
        BoxPreloadGoodsDO boxPreloadGoods = boxPreloadGoodsService.getBoxPreloadGoods(id);
        return success(BoxPreloadGoodsConvert.INSTANCE.convert(boxPreloadGoods));
    }

    @GetMapping("/list")
    @ApiOperation("获得预装货物列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-goods:query')")
    public CommonResult<List<BoxPreloadGoodsBackVO>> getBoxPreloadGoodsList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxPreloadGoodsDO> list = boxPreloadGoodsService.getBoxPreloadGoodsList(ids);
        return success(BoxPreloadGoodsConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得预装货物分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-goods:query')")
    public CommonResult<PageResult<BoxPreloadGoodsBackVO>> getBoxPreloadGoodsPage(@Valid BoxPreloadGoodsQueryVO query, PageVO page) {
        PageResult<BoxPreloadGoodsDO> pageResult = boxPreloadGoodsService.getBoxPreloadGoodsPage(query, page);
        return success(BoxPreloadGoodsConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出预装货物 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-goods:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxPreloadGoodsExcel(@Valid BoxPreloadGoodsQueryVO query,
                                           HttpServletResponse response) throws IOException {
        List<BoxPreloadGoodsDO> list = boxPreloadGoodsService.getBoxPreloadGoodsList(query);
        // 导出 Excel
        List<BoxPreloadGoodsBackVO> datas = BoxPreloadGoodsConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "预装货物.xls", "数据", BoxPreloadGoodsBackVO.class, datas);
    }

    @GetMapping("/downloadCustomFiles")
    @ApiOperation("下载报关单")
    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
    public CommonResult<String> downloadCustomFiles(@RequestParam("shipmentId") Long shipmentId, HttpServletResponse response) {
        List<OrderDO> orderList = boxPreloadGoodsService.getNormalOrderList(shipmentId);
        if (CollectionUtil.isEmpty(orderList)) {
            return error(new ServiceException(500, "报关单不存在"));
        }
        BoxDO boxDO = boxService.getBox(shipmentId);

        File file = new File(rootPath, boxDO.getSelfNo());
        if (file.exists()) {
            FileUtil.del(file);
        }
        file.mkdirs();
        File zipFile = null;
        String errorMsg = "打包失败";
        try {
            List<String> urlList = new ArrayList<>();
            for (OrderDO orderDO : orderList) {
                String documentFile = orderDO.getDocumentsFile();
                if (StringUtils.isEmpty(documentFile)) continue;
                String[] urlArray = documentFile.split(",");
                //下载文件到本地
                File orderDir = new File(file, orderDO.getOrderNo());
                orderDir.mkdirs();
                for (String url : urlArray) {
                    urlList.add(url);
                    String fileName = url.substring(url.lastIndexOf("/") + 1);
                    File orderFile = new File(orderDir, fileName);
                    HttpUtil.downloadFile(url, orderFile);
                }
            }

            if(CollectionUtil.isEmpty(urlList)) {
                errorMsg = "自单代报和混合报关的订单未上传单证";
                throw new ServiceException(500, errorMsg);
            }
            zipFile = jodd.io.ZipUtil.zip(file);
            String fix = FileNameUtil.getSuffix(zipFile.getName());
            String path = DateUtils.getDir() + IdUtil.objectId() + (StrUtil.isBlank(fix) ? StrUtil.EMPTY : StrUtil.DOT + fix);
            String url = fileService.createFile(path, FileUtil.readBytes(zipFile));
            return success(url);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FileUtil.del(file);
            if (zipFile != null) {
                zipFile.delete();
            }
        }
        return error(new ServiceException(500, errorMsg));
    }

}
