package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.vo.box.*;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetLoadVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.BoxUnloadApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsApplyReq;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadBatchRemoveReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadInfoBatchAddReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderExit.BoxOrderExitCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderExit.BoxOrderExitUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxLoadOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxLoadSectionBackVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 我的出货")
@RequestMapping("/my/shipment")
public class MyBoxController {

    @Resource
    private BoxService boxService;

    @PostMapping("/loadPage")
    @ApiOperation("获得装柜出货分页, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxLoadPage(@Valid @RequestBody MyBoxLoadQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxLoadPage(query);
        return success(pageResult);
    }


    @PostMapping("/tallyPage")
    @ApiOperation("获得待理货分页, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxTallyPage(@Valid @RequestBody MyBoxLoadQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxTallyPage(query);
        return success(pageResult);
    }

    @PostMapping("/tallyPageAir")
    @ApiOperation("获得待理货分页空运, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxTallyPageAir(@Valid @RequestBody MyBoxLoadQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxTallyPageAir(query);
        return success(pageResult);
    }

    @PostMapping("/unloadPage")
    @ApiOperation("获得卸柜出货分页, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxUnloadPage(@Valid @RequestBody MyBoxUnloadQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxUnloadPage(query);
        return success(pageResult);
    }

    @PostMapping("/exitPage")
    @ApiOperation("获得退场出货分页, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxExitPage(@Valid @RequestBody MyBoxExitQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxExitPage(query);
        return success(pageResult);
    }

    @PostMapping("/exitPageAir")
    @ApiOperation("获得退场出货分页, use this")
    public CommonResult<PageResult<BoxBackVO>> getBoxExitPageAir(@Valid @RequestBody MyBoxExitQueryVO query) {
        PageResult<BoxBackVO> pageResult = boxService.getBoxExitPageAir(query);
        return success(pageResult);
    }


    @GetMapping("/cabinetDetail")
    @ApiImplicitParam(name = "shipmentId", value = "ID", required = true, example = "1", dataTypeClass = Long.class)
    @ApiOperation("获得装柜详情")
    public CommonResult<BoxCabinetLoadVO> getBoxCabinetDetail(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.getBoxCabinetDetail(shipmentId));
    }

    @GetMapping("/cabinetUnloadDetail")
    @ApiImplicitParam(name = "shipmentId", value = "ID", required = true, example = "1", dataTypeClass = Long.class)
    @ApiOperation("获得卸柜详情")
    public CommonResult<BoxCabinetLoadVO> getBoxCabinetUnloadDetail(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.getBoxCabinetUnloadDetail(shipmentId));
    }

    @PostMapping("/tallyList")
    @ApiImplicitParams({@ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "1024", dataTypeClass = Long.class)})
    @ApiOperation("理货列表， use this")
    public CommonResult<List<BoxLoadSectionBackVO>> tallyList(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.pdaTallyList(shipmentId));
    }

    @GetMapping("/cabinetExitDetail")
    @ApiImplicitParam(name = "shipmentId", value = "ID", required = true, example = "1", dataTypeClass = Long.class)
    @ApiOperation("获得退场详情")
    public CommonResult<BoxCabinetLoadVO> getBoxCabinetExitDetail(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.getBoxCabinetExitDetail(shipmentId));
    }

    @GetMapping("/cabinetExitList")
    @ApiImplicitParam(name = "shipmentId", value = "ID", required = true, example = "1", dataTypeClass = Long.class)
    @ApiOperation("获得退场订单列表")
    public CommonResult<List<BoxLoadOrderVO>> getBoxCabinetExitList(@RequestParam("shipmentId") Long shipmentId) {
        return success(boxService.getBoxCabinetExitList(shipmentId));
    }

    @PostMapping("/closureApply")
    @ApiOperation("封柜申请")
    public CommonResult<Boolean> closureApply(@RequestBody BoxCabinetApplyReq applyReq) {
        boxService.closureApply(applyReq);
        return success(true);
    }

    @PostMapping("/unloadApply")
    @ApiOperation("卸柜申请")
    public CommonResult<Boolean> unloadApply(@RequestBody BoxUnloadApplyReq applyReq) {
        boxService.unloadApply(applyReq);
        return success(true);
    }

    @PostMapping("/confirmArrival")
    @ApiOperation("确认到仓")
    public CommonResult<Boolean> confirmArrival(@RequestBody BoxCustomsApplyReq applyReq) {
        boxService.confirmArrival(applyReq);
        return success(true);
    }

    @PostMapping("/changeContainer")
    @ApiOperation("更换货柜号重发")
    public CommonResult<Boolean> changeContainer(@RequestBody BoxChangeReqVO req) {
        boxService.changeContainer(req);
        return success(true);
    }

    @PostMapping("/batchCreate")
    @ApiOperation("批量装柜")
    public CommonResult<Boolean> batchCreate(@Valid @RequestBody BoxLoadInfoBatchAddReqVO createReqVO) {
        boxService.boxLoadBatchCreate(createReqVO);
        return success(true);
    }

    @PostMapping("/batchRemove")
    @ApiOperation("批量纠错")
    public CommonResult<Boolean> batchRemove(@Valid @RequestBody BoxLoadBatchRemoveReqVO createReqVO) {
        boxService.batchRemove(createReqVO);
        return success(true);
    }

    @PostMapping("/exitOrder")
    @ApiOperation("退场订单")
    public CommonResult<Boolean> exitOrder(@RequestBody BoxOrderExitCreateReqVO createReqVO) {
        boxService.exitOrder(createReqVO);
        return success(true);
    }

    @PostMapping("update/exitOrder/RefundNum")
    @ApiOperation("修改退场订单的退场箱数")
    public CommonResult<Boolean> updateExitOrderRefundNum(@RequestBody BoxOrderExitUpdateReqVO updateReqVO) {
        boxService.updateExitOrderRefundNum(updateReqVO.getId(), updateReqVO.getRefundNum());
        return success(true);
    }

    @PostMapping("/searchLoadOrderByBoxNo")
    @ApiOperation("根据箱号、订单号查找订单号码，装柜使用")
    public CommonResult<BoxLoadOrderVO> searchLoadOrderByBoxNo(@RequestBody BoxOrderSearchVO boxOrderSearchVO) {
        return success(boxService.searchLoadOrderByBoxNo(boxOrderSearchVO));
    }

    @PostMapping("/searchOrderByBoxNo")
    @ApiOperation("根据箱号、订单号查找订单号码，退场和卸柜使用")
    public CommonResult<BoxLoadOrderVO> searchOrderByBoxNo(@RequestBody BoxOrderSearchVO boxOrderSearchVO) {
        return success(boxService.searchOrderByBoxNo(boxOrderSearchVO));
    }

}
